//#include "bussim_arc.h"
#include "bussim_global.h"
#include "math.h"
#include <iostream>
#include <stdio.h>
#
//#include <GL/gl.h>
//#include <freeglut/freeglut.h>

BUSSIM_ARC_IN_BLOCK::BUSSIM_ARC_IN_BLOCK()
{
    this->prev = nullptr;
    this->next = nullptr;
    this->ref = nullptr;
}

BUSSIM_ARC_IN_BLOCK::~BUSSIM_ARC_IN_BLOCK()
{

}

BUSSIM_ARC::BUSSIM_ARC()
{
    this->ID = -1;

    this->orig_node = -1;
    this->dest_node = -1;
    this->orig_screen_x = -1;
    this->orig_screen_y = -1;
    this->dest_screen_x = -1;
    this->dest_screen_y = -1;

    this->length_screen = 0.0;
    this->length = 0.0;

    this->pos = -1;

    this->first_line = nullptr;
    this->last_line = nullptr;

    this->first_block = nullptr;
    this->last_block = nullptr;

    this->lines_on_arc = 0;

    this->type = -1;
    this->vehicletype = BUSSIM_ARCTYPE_ALL;

    this->first_waiting_vehicle = nullptr;
    this->last_waiting_vehicle = nullptr;

    this->in_blocks = 0;

    this->ROUTING_TYPE = BUSSIM_RT_VEHICLE;

    this->SERVED_BY_REGULAR_LINE = 0;
}

BUSSIM_ARC::BUSSIM_ARC(int _ID, int _ORIGIN, int _DESTINATION, double _ORIG_SCREEN_X, double _ORIG_SCREEN_Y, double _DEST_SCREEN_X, double _DEST_SCREEN_Y)
{
    this->ID = _ID;
    this->orig_node = _ORIGIN;
    this->dest_node = _DESTINATION;
    this->orig_screen_x = _ORIG_SCREEN_X;
    this->orig_screen_y = _ORIG_SCREEN_Y;
    this->dest_screen_x = _DEST_SCREEN_X;
    this->dest_screen_y = _DEST_SCREEN_Y;
    this->pos = -1;

    this->first_line = nullptr;
    this->last_line = nullptr;

    this->first_block = nullptr;
    this->last_block = nullptr;

    this->lines_on_arc = 0;

    this->length = 0.0;
    this->length_screen = 0.0;

    this->type = -1;
    this->vehicletype = BUSSIM_ARCTYPE_ALL;

    this->first_waiting_vehicle = nullptr;
    this->last_waiting_vehicle = nullptr;

    this->in_blocks = 0;

    this->ROUTING_TYPE = BUSSIM_RT_VEHICLE;

    this->SERVED_BY_REGULAR_LINE = 0;
}

BUSSIM_ARC::~BUSSIM_ARC()
{
    // sometimes this will be realized
}

void BUSSIM_ARC::update_length(class BUSSIM_NETWORK *_NET)
{
    this->length = sqrt(pow(BUSSIM_LONG_DIST_KM*(_NET->PT[this->dest_node].world_longitude - _NET->PT[this->orig_node].world_longitude),2)+pow(BUSSIM_LAT_DIST_KM*(_NET->PT[this->dest_node].world_latitude - _NET->PT[this->orig_node].world_latitude),2));
    this->length_screen = sqrt(pow(this->dest_screen_x-this->orig_screen_x,2)+pow(this->dest_screen_y-this->orig_screen_y,2));

    //std::cout << this->ID << ": " << this->length << "->" << ceil(60.0* this->length / 30.0) << " min. duration" << std::endl;
}

void BUSSIM_ARC::print_on_screen(class BUSSIM_NETWORK *_NET)
{
    if( this->ROUTING_TYPE == 0 )
    {
        // we have atravelling arc

    if( this->in_blocks < 1 )
    {
        // set the arc color
        glColor3f(BUSSIM_ARC_COLOR_RED, BUSSIM_ARC_COLOR_GREEN, BUSSIM_ARC_COLOR_BLUE);
        // print the line
        glLineWidth((GLfloat)2);
        glBegin(GL_LINES);
            //glLineWidth((GLfloat)30);
            glVertex2f(this->orig_screen_x, this->orig_screen_y);
            glVertex2f(this->dest_screen_x, this->dest_screen_y);
        glEnd();
    }
    else
    {
        //std::cout << "ID=" << this->ID <<": " << this->in_blocks << std::endl;
        double segment_size;
        segment_size = 1.0 / (double)(this->in_blocks);
        double x,y, x_diff, y_diff;
        x = this->orig_screen_x;
        y = this->orig_screen_y;
        x_diff = segment_size * (this->dest_screen_x - this->orig_screen_x);
        y_diff = segment_size * (this->dest_screen_y - this->orig_screen_y);
        class BUSSIM_ARC_IN_BLOCK * Ptr;
        Ptr = this->first_block;
        for( int i=0 ; i < this->in_blocks ; i++ )
        {
            glColor3f(Ptr->ref->RED,Ptr->ref->GREEN,Ptr->ref->BLUE);
            if( Ptr->ref->CAPACITY > Ptr->ref->USED_CAP )
                glLineWidth((GLfloat)2);
            else
                glLineWidth((GLfloat)8);
            glBegin(GL_LINES);
                //glLineWidth((GLfloat)30);
                glVertex2f(x, y);
                glVertex2f(x + x_diff, y + y_diff);
            glEnd();
            x = x + x_diff;
            y = y + y_diff;
            Ptr = Ptr->next;
        }
    }

    }
    else
    {
        // we have a transfer connection
        // print the line
        /*
        glColor3f(BUSSIM_CONNECTION_COLOR_RED,BUSSIM_CONNECTION_COLOR_GREEN,BUSSIM_CONNECTION_COLOR_BLUE);
        glLineWidth((GLfloat)2);
        glBegin(GL_LINES);
            //glLineWidth((GLfloat)30);
            glVertex2f(this->orig_screen_x, this->orig_screen_y);
            glVertex2f(this->dest_screen_x, this->dest_screen_y);
        glEnd();
        */
    }

    // print the arc num
    /*
    glRasterPos2d(this->orig_screen_x + 0.1* (this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 0.1* (this->dest_screen_y-this->orig_screen_y));

    char S[256];
    sprintf(S,"NUM=%d",this->ID);
    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_10, ubuf);
    */

    double r;
    r = 1.0 / (double)(pow(1.05,_NET->Current_Zoom_Level));

    double diff_x, diff_orth_x;
    double diff_y, diff_orth_y;

    diff_x = this->dest_screen_x - this->orig_screen_x;
    diff_y = this->dest_screen_y - this->orig_screen_y;

    double length_diff;
    length_diff = sqrt(pow(diff_x,2)+pow(diff_y,2));

    diff_orth_x = diff_y;
    diff_orth_y = -diff_x;

    double length_orth;
    length_orth = sqrt(pow(diff_orth_x,2)+pow(diff_orth_y,2));

        glBegin(GL_LINES);
            glVertex2f(this->orig_screen_x + 1.0*(this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 1.0*(this->dest_screen_y-this->orig_screen_y));
            glVertex2f(this->orig_screen_x + 1.0*(this->dest_screen_x-this->orig_screen_x) - r*0.01*(this->dest_screen_x-this->orig_screen_x)/length_diff +r*0.01*diff_orth_x/length_orth, this->orig_screen_y + 1.0*(this->dest_screen_y-this->orig_screen_y)- r*0.01*(this->dest_screen_y-this->orig_screen_y)/length_diff+r*0.01*diff_orth_y/length_orth);
        glEnd();

        glBegin(GL_LINES);
            glVertex2f(this->orig_screen_x + 1.0*(this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 1.0*(this->dest_screen_y-this->orig_screen_y));
            glVertex2f(this->orig_screen_x + 1.0*(this->dest_screen_x-this->orig_screen_x) - r*0.01*(this->dest_screen_x-this->orig_screen_x)/length_diff -r*0.01*diff_orth_x/length_orth, this->orig_screen_y + 1.0*(this->dest_screen_y-this->orig_screen_y)- r*0.01*(this->dest_screen_y-this->orig_screen_y)/length_diff-r*0.01*diff_orth_y/length_orth);
        glEnd();

}

void BUSSIM_ARC::print_on_screen_no_heads(class BUSSIM_NETWORK *_NET)
{
    if( this->ROUTING_TYPE == 0 )
    {
        // we have atravelling arc

    if( this->in_blocks < 1 )
    {
        // set the arc color
        glColor3f(BUSSIM_ARC_COLOR_RED, BUSSIM_ARC_COLOR_GREEN, BUSSIM_ARC_COLOR_BLUE);
        // print the line
        glLineWidth((GLfloat)2);
        glBegin(GL_LINES);
            //glLineWidth((GLfloat)30);
            glVertex2f(this->orig_screen_x, this->orig_screen_y);
            glVertex2f(this->dest_screen_x, this->dest_screen_y);
        glEnd();
    }
    else
    {
        //std::cout << "ID=" << this->ID <<": " << this->in_blocks << std::endl;
        double segment_size;
        segment_size = 1.0 / (double)(this->in_blocks);
        double x,y, x_diff, y_diff;
        x = this->orig_screen_x;
        y = this->orig_screen_y;
        x_diff = segment_size * (this->dest_screen_x - this->orig_screen_x);
        y_diff = segment_size * (this->dest_screen_y - this->orig_screen_y);
        class BUSSIM_ARC_IN_BLOCK * Ptr;
        Ptr = this->first_block;
        for( int i=0 ; i < this->in_blocks ; i++ )
        {
            glColor3f(Ptr->ref->RED,Ptr->ref->GREEN,Ptr->ref->BLUE);
            if( Ptr->ref->CAPACITY > Ptr->ref->USED_CAP )
                glLineWidth((GLfloat)2);
            else
                glLineWidth((GLfloat)8);
            glBegin(GL_LINES);
                //glLineWidth((GLfloat)30);
                glVertex2f(x, y);
                glVertex2f(x + x_diff, y + y_diff);
            glEnd();
            x = x + x_diff;
            y = y + y_diff;
            Ptr = Ptr->next;
        }
    }

    }
    else
    {

    }


    // print the arc num
    /*
    glRasterPos2d(this->orig_screen_x + 0.1* (this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 0.1* (this->dest_screen_y-this->orig_screen_y));

    char S[256];
    sprintf(S,"NUM=%d",this->ID);
    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_10, ubuf);
    */

    /*
    double r;
    r = 1.0 / (double)(pow(1.05,_NET->Current_Zoom_Level));

    double diff_x, diff_orth_x;
    double diff_y, diff_orth_y;

    diff_x = this->dest_screen_x - this->orig_screen_x;
    diff_y = this->dest_screen_y - this->orig_screen_y;

    double length_diff;
    length_diff = sqrt(pow(diff_x,2)+pow(diff_y,2));

    diff_orth_x = diff_y;
    diff_orth_y = -diff_x;

    double length_orth;
    length_orth = sqrt(pow(diff_orth_x,2)+pow(diff_orth_y,2));

        glBegin(GL_LINES);
            glVertex2f(this->orig_screen_x + 1.0*(this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 1.0*(this->dest_screen_y-this->orig_screen_y));
            glVertex2f(this->orig_screen_x + 1.0*(this->dest_screen_x-this->orig_screen_x) - r*0.01*(this->dest_screen_x-this->orig_screen_x)/length_diff +r*0.01*diff_orth_x/length_orth, this->orig_screen_y + 1.0*(this->dest_screen_y-this->orig_screen_y)- r*0.01*(this->dest_screen_y-this->orig_screen_y)/length_diff+r*0.01*diff_orth_y/length_orth);
        glEnd();

        glBegin(GL_LINES);
            glVertex2f(this->orig_screen_x + 1.0*(this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 1.0*(this->dest_screen_y-this->orig_screen_y));
            glVertex2f(this->orig_screen_x + 1.0*(this->dest_screen_x-this->orig_screen_x) - r*0.01*(this->dest_screen_x-this->orig_screen_x)/length_diff -r*0.01*diff_orth_x/length_orth, this->orig_screen_y + 1.0*(this->dest_screen_y-this->orig_screen_y)- r*0.01*(this->dest_screen_y-this->orig_screen_y)/length_diff-r*0.01*diff_orth_y/length_orth);
        glEnd();
    */
}

void BUSSIM_ARC::print_transfer_arc(class BUSSIM_NETWORK *_NET)
{
    // we have a transfer connection
    if( this->ROUTING_TYPE != 0 )
    {
        glColor3f(_NET->PT[this->dest_node].ref2stop->RED,_NET->PT[this->dest_node].ref2stop->GREEN,_NET->PT[this->dest_node].ref2stop->BLUE);
        glLineWidth((GLfloat)2);
        glBegin(GL_LINES);
            //glLineWidth((GLfloat)30);
            glVertex2f(this->orig_screen_x, this->orig_screen_y);
            glVertex2f(this->dest_screen_x, this->dest_screen_y);
        glEnd();
    }
}

void BUSSIM_ARC::print_queue_on_screen(class BUSSIM_NETWORK *_NET)
{
    int TYPE = -1;
    double x_orth = 0.0;
    double y_orth = 0.0;
    double a = 0.03;

    // determine the middle of the arc
    double x = this->dest_screen_x - this->orig_screen_x;
    double y = this->dest_screen_y - this->orig_screen_y;

    // make necessary calculations
    if( fabs(x) > 0 )
    {
        if( x > 0 )
        {
            if( y > 0 )
            {
                // unten links nach oben rechts (1;1)
                TYPE = 0;
            }
            else
            {
                // oben links nach unten rechts (1;-1)
                TYPE = 1;
            }
        }
        else
        {
            if( y > 0 )
            {
                // unten rechts nach oben links
                TYPE = 2;
            }
            else
            {
                // oben rechts nach unten links
                TYPE = 3;
            }
        }

                if( (TYPE == 0))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = -sqrt( pow(a,2) - pow(x_orth,2) );
                }
                if( (TYPE == 1))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = -sqrt( pow(a,2) - pow(x_orth,2) );
                }
                if( (TYPE == 2))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = sqrt( pow(a,2) - pow(x_orth,2) );
                }
                if( (TYPE == 3))
                {
                    x_orth = y * a * 1.0 / sqrt(pow(y,2) + pow(x,2));
                    y_orth = sqrt( pow(a,2) - pow(x_orth,2) );
                }
    }
    else
    {

    }
    // print the arc num
    double r;
    r = 1.0 / (double)(pow(1.05,_NET->Current_Zoom_Level));

    glRasterPos2d(this->orig_screen_x + 0.5*x+ r* x_orth, this->orig_screen_y + 0.5*y+r *y_orth);

    char S[256] ,T[256];
    sprintf(S,"queue on arc %d:",this->ID);
    class BUSSIM_ARC_WAITING_VEHICLE *Ptr;
    Ptr = this->first_waiting_vehicle;
    while( Ptr != nullptr )
    {
        sprintf(T," %d",Ptr->veh_id);
        strcat(S, T);
        Ptr = Ptr->next;
    }
    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);

}

void BUSSIM_ARC::append_line(class BUSSIM_ARC_IN_LINE *_Ptr, class BUSSIM_NETWORK *_NET)
{

    if( _Ptr != nullptr )
    {
        if( this->lines_on_arc < 1 )
        {
            // add the first line to this arc
            this->first_line = _Ptr;
            this->last_line = _Ptr;
            this->lines_on_arc++;
            if( _NET->LINE[_Ptr->LINE_ID].type == BUSSIM_REGULARLINE )
                this->SERVED_BY_REGULAR_LINE = 1;
        }
        else
        {
            // add another line to this arc
            this->last_line->next = _Ptr;
            this->last_line = _Ptr;
            this->lines_on_arc++;
            if( _NET->LINE[_Ptr->LINE_ID].type == BUSSIM_REGULARLINE )
                this->SERVED_BY_REGULAR_LINE = 1;

        }
    }
}

void BUSSIM_ARC::set_direction_type(void)
{
    if( this->orig_screen_x < this->dest_screen_x )
        if( this->orig_screen_y < this->dest_screen_y)
            this->type = BUSSIM_NORTHEAST;
        else
            if( this->orig_screen_y > this->dest_screen_y)
                this->type = BUSSIM_SOUTHEAST;
            else
                this->type = BUSSIM_EAST;
    else
        if( this->orig_screen_x > this->dest_screen_x )
            if( this->orig_screen_y < this->dest_screen_y)
                this->type = BUSSIM_NORTHWEST;
            else
                if( this->orig_screen_y > this->dest_screen_y)
                    this->type = BUSSIM_SOUTHWEST;
                else
                    this->type = BUSSIM_WEST;
        else
            if( this->orig_screen_y < this->dest_screen_y)
                this->type = BUSSIM_NORTH;
            else
                this->type = BUSSIM_SOUTH;
}

void BUSSIM_ARC::append_arrived_vehicle(int _VEH_ID)
{
    class BUSSIM_ARC_WAITING_VEHICLE *Ptr;
    Ptr = new class BUSSIM_ARC_WAITING_VEHICLE;
    Ptr->veh_id = _VEH_ID;
    Ptr->next = nullptr;
    if( this->last_waiting_vehicle != nullptr )
    {
        // append to existing sequence
        this->last_waiting_vehicle->next = Ptr;
        this->last_waiting_vehicle = Ptr;

    }
    else
    {
        // store the first vehicle into the queue
        this->first_waiting_vehicle = Ptr;
        this->last_waiting_vehicle = this->first_waiting_vehicle;
    }
}

int BUSSIM_ARC::remove_vehicle_from_queue(int _VEH_ID, class BUSSIM_NETWORK *_NET)
{
    int ret_val = 0;

    //std::cout << "BEGIN REMOVE VEHICLE " << _VEH_ID << " on arc " << this->ID << std::endl;
    if( this->first_waiting_vehicle != nullptr )
    {
        // there is at least one vehicle in the queue
        if( this->first_waiting_vehicle->next == nullptr )
        {
            // there is exactly one vehicle in the queue
            if( this->first_waiting_vehicle->veh_id == _VEH_ID )
            {
                // the one and only saved vehicle must be removed
                delete this->first_waiting_vehicle;
                this->first_waiting_vehicle = nullptr;
                this->last_waiting_vehicle = nullptr;
            }
        }
        else
        {
            // there are two or even more vehicles in the queue

            // first look for the requested vehicle
            class BUSSIM_ARC_WAITING_VEHICLE *Ptr, *Tmp;
            Ptr = this->first_waiting_vehicle;
            if ( Ptr->veh_id == _VEH_ID )
            {
                // the first entry must be removed
                this->first_waiting_vehicle = this->first_waiting_vehicle->next;
                delete Ptr;
            }
            else
            {

                // std::cout << "A+" << std::endl;
                Tmp = this->first_waiting_vehicle;
                // std::cout << "Current Queue" << std::endl;
                /*
                while( Tmp != nullptr )
                {
                    std::cout << Tmp->veh_id << " POS="
                    << _NET->VEHICLE[_VEH_ID].POS
                    << " " << _NET->VEHICLE[_VEH_ID].ON_ARC.ID
                    << " " << _NET->VEHICLE[_VEH_ID].km_travelled_on_arc
                    << " " << _NET->VEHICLE[_VEH_ID].remaing_waiting_time
                    << std::endl;
                    Tmp = Tmp->next;
                }
                std::cout << "LAST=" << this->last_waiting_vehicle->veh_id << std::endl;
                */

                // first identify the associated entry
                Tmp = nullptr;
                while( Ptr != nullptr )
                {
                    if( Ptr->veh_id == _VEH_ID )
                    {
                        Tmp = Ptr;
                        Ptr = nullptr;
                        //delete Tmp;
                    }
                    else
                    {
                        Ptr = Ptr->next;
                    }
                }
                //std::cout << "A" << std::endl;
                // now remove the identified queue entry
                if( Tmp != nullptr )
                {
                    if( Tmp == this->first_waiting_vehicle )
                    {
                        // we have to remove the first vehicle in queue
                        this->first_waiting_vehicle = this->first_waiting_vehicle->next;
                        delete Tmp;
                    }
                    else
                    {
                        ret_val = -1;
                        /*
                        // goto the predecessor vehicle
                        Ptr = this->first_waiting_vehicle;
                        while( Ptr->next != Tmp)
                            Ptr = Ptr->next;

                        if( Tmp == this->last_waiting_vehicle )
                        {
                            // we have to remove the last vehicle in queue
                            Ptr->next = nullptr;
                            this->last_waiting_vehicle = Ptr;
                            delete Tmp;
                        }
                        else
                        {
                            // we have to remove a vehicle in between two other queued vehicles
                            Ptr->next = Ptr->next->next;
                            delete Tmp;
                        }
                        */
                    }
                }
                //std::cout << "A-" << std::endl;
            }
        }
    }
    else
    {
        std::cout << "ARC "<< this->ID <<" ERROR REMOVE VEHICLE " << _VEH_ID <<" FROM QUEUE" << std::endl;
        exit(0);
    }

    //std::cout << "END REMOVE VEHICLE" << std::endl;

    return ret_val;
}

void BUSSIM_ARC::add_block(class BUSSIM_BLOCK *_RefToBlock)
{
        // create a new BUSSIM_BLOCK_MEMBER-object
    class BUSSIM_ARC_IN_BLOCK *NewPtr;
    NewPtr = new class BUSSIM_ARC_IN_BLOCK;

    // fill this new object
    NewPtr->ref = _RefToBlock;
    NewPtr->prev = nullptr;
    NewPtr->next = nullptr;

    // append the new member to the list
    if( this->first_block == nullptr )
    {
        // no member stored so far
        this->first_block = NewPtr;
        this->last_block = NewPtr;
        NewPtr = nullptr;
    }
    else
    {
        // there are members already stored
        this->last_block->next = NewPtr;
        NewPtr->prev = this->last_block;
        this->last_block = NewPtr;
        NewPtr = nullptr;
    }

    // increase the block counter
    this->in_blocks++;
}

int BUSSIM_ARC::get_allowance(class BUSSIM_ARC *_Ptr2NextArc)
{
    // this procedure checks if a vehicle can enter this arc w.r.t. to block
    // constraints
    // = 1 if this arc can be entered
    // = 0 in all other cases

    // what is checked
    // =1 is returned if a) the set of blocks of this arc is a subset of the blocks
    // associated with _Ptr2Arc or b) additional blocks of this arc provide enough
    // capacity
    // =0 in all other cases


    int ret_val=1;

    //if( this->in_blocks < 1 )
    if(2 < 1)
    {
        // no blocks involved
        ret_val = 1;
    }
    else
    {
        // this arc is part of at least one block
        class BUSSIM_ARC_IN_BLOCK *PtrToBlock;

        // start to collect the information from the recent block A
        PtrToBlock = this->first_block;
        while( PtrToBlock != nullptr )
        {
            PtrToBlock->ref->check_A = 1;
            PtrToBlock = PtrToBlock->next;
        }

        // start to collect the information from the recent block B (next block)
        PtrToBlock = _Ptr2NextArc->first_block;
        while( PtrToBlock != nullptr )
        {
            PtrToBlock->ref->check_B = 1;
            if( (PtrToBlock->ref->check_A > 0 ) && (PtrToBlock->ref->check_B > 0) )
            {
                // recent and next arc in same block: nothing to do further
            }
            else
            {
                //exit(17);
                if(PtrToBlock->ref->check_B > 0)
                {
                    // next arc not in the same block as last arc
                    // check if next arc provides sufficient capacity
                    if( PtrToBlock->ref->CAPACITY > PtrToBlock->ref->USED_CAP )
                    {
                        // the next arc is in a new block and the vehicle can be allowed to enter
                        // nothing to do here
                    }
                    else
                    {
                        // the next arc is in a new block without free capacity
                        // allowance to enter cannot be granted
                        ret_val = 0;
                    }
                }
                else
                {

                }
            }
            if( ret_val > 0 )
                PtrToBlock = PtrToBlock->next;
            else
                PtrToBlock = nullptr;
        }


    // reset the attributes check_A and check_B
    PtrToBlock = this->first_block;
    while( PtrToBlock != nullptr )
    {
        PtrToBlock->ref->check_A = 0;
        PtrToBlock = PtrToBlock->next;
    }
    PtrToBlock = _Ptr2NextArc->first_block;
    while( PtrToBlock != nullptr )
    {
        PtrToBlock->ref->check_B = 0;
        PtrToBlock = PtrToBlock->next;
    }
    }
    if( ret_val < 1 )
    {
    }
    return(ret_val);
}

void BUSSIM_ARC::check_in_vehicle(void)
{
    class BUSSIM_ARC_IN_BLOCK *Ptr;
    Ptr = this->first_block;
    while( Ptr != nullptr )
    {
        Ptr->ref->check_in_vehicle();
        Ptr = Ptr->next;
    }
}

void BUSSIM_ARC::check_out_vehicle(void)
{
    class BUSSIM_ARC_IN_BLOCK *Ptr;
    Ptr = this->first_block;
    while( Ptr != nullptr )
    {
        Ptr->ref->check_out_vehicle();
        Ptr = Ptr->next;
    }
}

void BUSSIM_ARC::specify(int _ID, int _ORIG_NODE, int _DEST_NODE)
{
    this->ID = _ID;
    this->orig_node = _ORIG_NODE;
    this->dest_node = _DEST_NODE;

}

void BUSSIM_ARC::set_rt_type(int _RT_TYPE)
{
    this->ROUTING_TYPE = _RT_TYPE;
}

int BUSSIM_ARC::get_rt_type(void)
{
    return( this->ROUTING_TYPE);
}

void BUSSIM_ARC::print_label(class BUSSIM_NETWORK *_NET)
{

    // print the arc num
    double r;
    r = 1.0 / (double)(pow(1.05,_NET->Current_Zoom_Level));

    // print the gray box
    glColor3f((float)(BUSSIM_BGCOLOR_MAIN_RED),(float)(BUSSIM_BGCOLOR_MAIN_GREEN),(float)(BUSSIM_BGCOLOR_MAIN_BLUE));

    double x_avg;
    double y_avg;

    x_avg = ( this->orig_screen_x + this->dest_screen_x ) / 2.0;
    y_avg = ( this->orig_screen_y + this->dest_screen_y ) / 2.0;

    double f=0.01;
    glBegin(GL_POLYGON);
                glVertex2f(x_avg - r*1.1*f, y_avg + r*f);
                glVertex2f(x_avg + r*f, y_avg + r*f);
                glVertex2f(x_avg + r*f, y_avg - r*1.0*f);
                glVertex2f(x_avg - r*1.1*f, y_avg - r*1.0*f);
                glVertex2f(x_avg - r*1.1*f, y_avg + r*f);
            glEnd();

    // set the print color
    glColor3f((float)(BUSSIM_ARC_COLOR_RED),(float)(BUSSIM_ARC_COLOR_GREEN),(float)(BUSSIM_ARC_COLOR_BLUE));



    // glRasterPos2d(this->orig_screen_x + 0.5*x+ r* x_orth, this->orig_screen_y + 0.5*y+r *y_orth);
    glRasterPos2d(x_avg, y_avg);

    char S[256];
    sprintf(S,"%d",this->ID);

    //glRotatef(45,0.0f, 0.0f, 1.0f);
    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);

}
