#include "bussim_global.h"

BUSSIM_BLOCK_MEMBER::BUSSIM_BLOCK_MEMBER()
{
    // this is the standard constructor
    this->prev = nullptr;
    this->next = nullptr;
    this->arc = nullptr;
}

BUSSIM_BLOCK_MEMBER::~BUSSIM_BLOCK_MEMBER()
{
    // this is the standard destructor
    this->prev = nullptr;
    this->next = nullptr;
    this->arc = nullptr;
}

BUSSIM_BLOCK::BUSSIM_BLOCK()
{
    // this is the standard constructor
    this->ID = -1;
    this->CAPACITY = 1;
    this->USED_CAP = 0;
    this->check_A = 0;
    this->check_B = 0;
    this->DESC = "block desc not yet defined";
    this->RED = 0.5;
    this->GREEN = 0.5;
    this->BLUE = 0.5;
    this->FIRST_MEMBER = nullptr;
    this->LAST_MEMBER = nullptr;
}

BUSSIM_BLOCK::BUSSIM_BLOCK(int _ID, int _CAPACITY, std::string _DESC, double _RED, double _GREEN, double _BLUE)
{
    // this is an overloaded constructor
    this->ID = _ID;
    this->CAPACITY = _CAPACITY;
    this->USED_CAP = 0;
    this->check_A = 0;
    this->check_B = 0;
    this->DESC = _DESC;
    this->RED = _RED;
    this->GREEN = _GREEN;
    this->BLUE = _BLUE;
    this->FIRST_MEMBER = nullptr;
    this->LAST_MEMBER = nullptr;
}

BUSSIM_BLOCK::~BUSSIM_BLOCK()
{
    // this is the standard destructor
    // mainly used to dismantle the list of associated
    // BUSSIM_BLOCK_MEMBERS
}

void BUSSIM_BLOCK::add_arc(struct BUSSIM_ARC *_ArcRef)
{
    // create a new BUSSIM_BLOCK_MEMBER-object
    class BUSSIM_BLOCK_MEMBER *NewPtr;
    NewPtr = new class BUSSIM_BLOCK_MEMBER;

    // fill this new object
    NewPtr->arc = _ArcRef;
    NewPtr->prev = nullptr;
    NewPtr->next = nullptr;

    // append the new member to the list
    if( this->FIRST_MEMBER == nullptr )
    {
        // no member stored so far
        this->FIRST_MEMBER = NewPtr;
        this->LAST_MEMBER = NewPtr;
        NewPtr = nullptr;
    }
    else
    {
        // there are members already stored
        this->LAST_MEMBER->next = NewPtr;
        NewPtr->prev = this->LAST_MEMBER;
        this->LAST_MEMBER = NewPtr;
        NewPtr = nullptr;
    }

    // finally add this block to the list of blocks to which _ArcRef belongs
    _ArcRef->add_block(this);
}

void BUSSIM_BLOCK::check_in_vehicle(void)
{
    this->USED_CAP++;
}

void BUSSIM_BLOCK::check_out_vehicle(void)
{
    this->USED_CAP--;
}

void BUSSIM_BLOCK::specify(int _ID, int _CAPACITY, std::string _DESC, double _RED, double _GREEN, double _BLUE)
{
    this->ID = _ID;
    this->CAPACITY = _CAPACITY;
    this->USED_CAP = 0;
    this->check_A = 0;
    this->check_B = 0;
    this->DESC = _DESC;
    this->RED = _RED;
    this->GREEN = _GREEN;
    this->BLUE = _BLUE;
    this->FIRST_MEMBER = nullptr;
    this->LAST_MEMBER = nullptr;

}
