#ifndef BUSSIM_GLOBAL_H_INCLUDED
#define BUSSIM_GLOBAL_H_INCLUDED

#include <GL/gl.h>
#include <freeglut/freeglut.h>

#include "bussim_extern.h"      // added in version 3.0

#ifdef BUSSIM_EXTERN_SOIL       // added in version 3.0
#include "SOIL2/SOIL2.h"
#else
#endif // BUSSIM_EXTERN_SOIL

#include "bussim_arc.h"
#include "bussim_duty.h"
#include "bussim_pax.h"         // added in version 3.0

#include "bussim_network.h"
#include "bussim_vehicle.h"
#include "bussim_bunchpoint.h"
#include "bussim_event.h"
#include "bussim_line.h"
#include "bussim_point.h"
#include "bussim_toi.h"         // added in version 1.05
#include "bussim_block.h"       // added in version 3.0
#include "bussim_schedule.h"
#include "bussim_rotation.h"
#include "bussim_stop.h"        // added in version 3.0

#define BUSSIM_NO_EVENT -1
#define BUSSIM_ARRIVAL 0
#define BUSSIM_DEPARTURE 1
#define BUSSIM_VEHICLE_ACTIVATION 2
#define BUSSIM_VEHICLE_DEACTIVATION 3
#define BUSSIM_VEHICLE_END_OF_WORK 4
#define BUSSIM_VEHICLE_BLOCKAGE 5
#define BUSSIM_WAIT_FOR_BLOCK 6

#define BUSSIM_SHOW_PAX_WINDOW 0   // should the pax window be shown (1) or not (0)

#define BUSSIM_END_OF_DUTY 1440// number of minutes of the working day / start time of densing the SERVICES_ACTIVE_DATABASE

#define BUSSIM_TIME_STEP 0.1               // time offset in every iteration

#define BUSSIM_INITIAL_SPEEDUP_FACTOR 10    // this is the initially used speed factor to be modified during the simulation by the keys <f> (acceleration) and <s> (deceleration)

#define BUSSIM_MINUTES_OFFSET 330       // this is the real time when the first vehicle StartService (in minutes)

#define BUSSIM_CLOCK_POS_X -0.9       // this describes the position of the system clock on the screen
#define BUSSIM_CLOCK_POS_Y -0.9

#define BUSSIM_BGCOLOR_MAIN_RED 0.8     // defines the background color of the main simulation window
#define BUSSIM_BGCOLOR_MAIN_GREEN 0.8
#define BUSSIM_BGCOLOR_MAIN_BLUE 0.8

#define BUSSIM_ARC_COLOR_RED 0.0     // defines the color of arcs forming the graph
#define BUSSIM_ARC_COLOR_GREEN 0.0
#define BUSSIM_ARC_COLOR_BLUE 0.0

#define BUSSIM_NODE_DEFAULT_COLOR_RED 0.0     // defines the color of nodes (non-stop) forming the graph
#define BUSSIM_NODE_DEFAULT_COLOR_GREEN 0.0
#define BUSSIM_NODE_DEFAULT_COLOR_BLUE 0.0

#define BUSSIM_BGCOLOR_VEHICLES_RED 0.8     // defines the background color of the vehicle report window
#define BUSSIM_BGCOLOR_VEHICLES_GREEN 0.8
#define BUSSIM_BGCOLOR_VEHICLES_BLUE 0.8

#define BUSSIM_CONNECTION_COLOR_RED 0.5   // defines the color of connection arcs
#define BUSSIM_CONNECTION_COLOR_GREEN 1.0
#define BUSSIM_CONNECTION_COLOR_BLUE 0.5

#define BUSSIM_RANDOM_SEED 0

#define BUSSIM_MEAN_WAITING_TIME 0.75   // mean value of the waiting time of a vehicle at a stop
#define BUSSIM_DEVIATION_WAITING_TIME 0.0   // interval around BUSSIM_STANDARD_WAITING_TIME from which a waiting time at a stop of a vehicle is drawn uniformly distributed
#define BUSSIM_DEPOT_WAITING_TIME 5.00  // default value for a stop at a depot

#define BUSSIM_SHOW_LINE_AGENDA 0       // displays the list of lines on the left screen edge if = 1
#define BUSSIM_SHOW_ACTIVE_VEHICLES 0   // displays the list of active vehicles on the right screen edge if = 1
#define BUSSIM_SHOW_BUNCHES 0           // displays bunchpoints if = 1
#define BUSSIM_SHOW_TOI 1               // V1.05: displays tois if = 1
#define BUSSIM_SHOW_VEHICLE_LABELS 0    // V3.00: shows vehicle information if = 1
#define BUSSIM_WINDOWS_SIZE_X 1000      // width of simulation window (in pixels)
#define BUSSIM_WINDOWS_SIZE_Y 1000      // heigth of simulation window (in pixels)

#define BUSSIM_MENUE_POSITION 0.95     // position of menue

#define BUSSIM_DEFAULT_LABEL_POS 8      // default position of node labels on screen
#define BUSSIM_STOP_LABEL_OFFSET 0.025  // label position of stop labels
#define BUSSIM_DEFAULT_INFRASTRUCTURE_POINT 0
                                        // default declaration of the bussim point infrastructure attribute
#define BUSSIM_NORTH 0
#define BUSSIM_NORTHEAST 1
#define BUSSIM_EAST 2
#define BUSSIM_SOUTHEAST 3
#define BUSSIM_SOUTH 4
#define BUSSIM_SOUTHWEST 5
#define BUSSIM_WEST 6
#define BUSSIM_NORTHWEST 7

#define BUSSIM_REGULARLINE 1
#define BUSSIM_EXTRALINE 2
#define BUSSIM_DEPLOYMENTLINE 3
#define BUSSIM_PARKINGLINE 4

#define BUSSIM_NODE_LINE 1
#define BUSSIM_NODE_SERVICE 2

#define BUSSIM_LONG_DIST_KM 71       // added in version 1.05
#define BUSSIM_LAT_DIST_KM 101       // added in version 1.05

#define BUSSIM_TOI_WATER 1           // added in version 1.05
#define BUSSIM_TOI_MOUNTAIN 2        // added in version 1.05
#define BUSSIM_TOI_HIGHWAY 3        // added in version 1.05
//#define BUSSIM_TOI_OTHER 4        // added in version 1.05
#define BUSSIM_TOI_FOREST 5         // green area in version 1.05
#define BUSSIM_TOI_LAKE 6         // blue area in version 1.05
#define BUSSIM_TOI_RAILTRACK 7      // railtracks in version 1.05

#define BUSSIM_ARCTYPE_ALL 0         // no limitation
#define BUSSIM_ARCTYPE_BUSONLY 1     // the arc can be used only by busses
#define BUSSIM_ARCTYPE_TRAMONLY 2     // the arc can be used only by trams
#define BUSSIM_ARCTYPE_TRAMBUS 3      // the arc can be used by bus and by trams

#define BUSSIM_BGFILE "bremen.png"     // defines the file name of the background texture
#define BUSSIM_BGFILE_ONSTART 0     // = 1 active / = 0 inactive

#define BUSSIM_VEHICLE_SIZE 0.015   // V3_00: size of the circle representing a vehicle

#define BUSSIM_INFRA_SWITCH 0         // V3_00: attribute of an infrastructure node if it is a switch
#define BUSSIM_INFRA_TRACKPOS 1       // V3_00: attribute of an infrastructure node if it is a track position
#define BUSSIM_INFRA_UNDEF -1         // V3_00: default type of an infrastructure node

#define BUSSIM_VIEWPORT_LEFT 0          // to code the perspective manipulation
#define BUSSIM_VIEWPORT_RIGHT 1
#define BUSSIM_VIEWPORT_UP 2
#define BUSSIM_VIEWPORT_DOWN 3
#define BUSSIM_VIEWPORT_IN 4
#define BUSSIM_VIEWPORT_OUT 5

#define BUSSIM_BIG_M 10000000.0     // a multi-purpose parameter to represent infinity

#define BUSSIM_PAX_UNKNOWN 0        // V3_00: passenger state - not yet shown up
#define BUSSIM_PAX_IN 1             // V3_00: passenger state - in system
#define BUSSIM_PAX_ARRIVED 2        // V3_00: passenger state - has arrived at its final dest / out of system

#define BUSSIM_PAX_NOTHING 0       // V3_00: passenger activity - it does nothing
#define BUSSIM_PAX_WAITING 1       // V3_00: passenger activity - it waits for a vehicle at a node
#define BUSSIM_PAX_RIDING 2        // V3_00: passenger activity - it is riding inside a vehicle
#define BUSSIM_PAX_WALKING 3       // V3_00: passenger activity - it walks towards a node

#define BUSSIM_RT_VEHICLE 0        // V3_00: arc is traversed by a vehicle
#define BUSSIM_RT_WALK 1           // V3_00: arc is travered by walking

#define BUSSIM_PAX_WALKING_V_KMH 5  // V3_00: velocity of a pax during transfer walks

#define BUSSIM_VEHCAT_TRAM 0        // V3_00: used to distinguish trams, busses, trains, ...
#define BUSSIM_VEHCAT_BUS 1

// windows definition functions
void set_simulation_window(void);

// opengl callback function definition
void set_callback_funcs(void);

// the prototypes of the opengl callback functions
void display(void);
void tasten( unsigned char key, int x, int y );
void time_loop(int passed_value);

void display_vehicles(void);
void tasten_vehicles( unsigned char key, int x, int y );
void time_loop_vehicles(int passed_value);

void display_paxes(void);

// the menue function
void create_menue(void);
void menu(int id);
void menu_a(int id);
void update_line_list_menue();
#endif // BUSSIM_GLOBAL_H_INCLUDED
