#ifndef BUSSIM_LINE_H_INCLUDED
#define BUSSIM_LINE_H_INCLUDED

#include <string>

class BUSSIM_LINE{
public:
    int ID;                         // unique identifier
    int SERVICE;                  // used to group several lines
    int ARCS;                       // number of arcs that form this line
    class BUSSIM_ARC *USED_ARC;     // array of network arcs that form the line
    int ID_NEXT_LINE;               // ID of line served next
    char LineName[256];                // To be used to give a clear name

    std::string AUTO_LINE_NAME_DEST;       // short automatically generated line descriptor

    double x_offset;                // used to display non-overlapping lines
    double y_offset;

    double RED;
    double GREEN;
    double BLUE;

    int displayed;                  // is this line showed? (default = yes, 1)

    int type;                       // regular or extra line (default = regular, 1)

    int REGULAR_STOP_NUM;           // 3.00: counts the number of nodes with pax exchange

    BUSSIM_LINE();
    BUSSIM_LINE(int _ID, int _SERVICE, int _ID_NEXT_LINE, double _RED, double _GREEN, double _BLUE, int _TYPE);
    ~BUSSIM_LINE();
    void append_arc(class BUSSIM_ARC _ARC);
    void set_offset(void);
    void print_on_screen(class BUSSIM_NETWORK *_NET);
    void print_on_screen_indicators(class BUSSIM_NETWORK *_NET);
    void configure(int _ID, int _SERVICE, double _RED, double _GREEN, double _BLUE, const char _LINENAME[256]);
    void check_consistency(class BUSSIM_NETWORK *_NET);
    void print_on_console(class BUSSIM_NETWORK *_NET);  // V3.00
    void set_automatic_desc(class BUSSIM_NETWORK *_NET); // V3.00
};

#endif // BUSSIM_LINE_H_INCLUDED
