#include "bussim_global.h"
#include "cmath"

BUSSIM_PAX_LIST::BUSSIM_PAX_LIST()
{
    this->PAXES = 0;
    this->LISTNAME = "unnamed pax list";
    this->first_pax_item = nullptr;
    this->last_pax_item = nullptr;
    this->ref2schedule_seg = nullptr;
}

BUSSIM_PAX_LIST::BUSSIM_PAX_LIST(std::string _LISTNAME)
{
    this->PAXES = 0;
    this->LISTNAME = _LISTNAME;
    this->first_pax_item = nullptr;
    this->last_pax_item = nullptr;
    this->ref2schedule_seg = nullptr;
}

BUSSIM_PAX_LIST::~BUSSIM_PAX_LIST()
{

}

class BUSSIM_SEGMENT *BUSSIM_PAX_LIST::get_ref2schedule_seg(void)
{
    return(this->ref2schedule_seg);
}

void BUSSIM_PAX_LIST::set_ref2schedule_seg(class BUSSIM_SEGMENT *_REF2SCHEDULE_SEG)
{
    this->ref2schedule_seg = _REF2SCHEDULE_SEG;
}

void BUSSIM_PAX_LIST::append_pax(class BUSSIM_PAX *_REF2PAX, int _MARKER)
{
    // check if there is already a paxlistitem stored
    if( this->first_pax_item != nullptr )
    {
        // append to existing list

        // create the list item
        class BUSSIM_PAX_LISTITEM *TmpPtr;
        TmpPtr = new class BUSSIM_PAX_LISTITEM;

        // fill it
        TmpPtr->ref2pax = _REF2PAX;
        TmpPtr->previous = this->last_pax_item;
        TmpPtr->previous->next = TmpPtr;

        // update the reference to last stored item
        this->last_pax_item = TmpPtr;

        if( _MARKER > 0 )
        {
            // set the backreferring pointer
            TmpPtr->ref2pax->Ref2WalkingPaxList = TmpPtr;

        }


        TmpPtr->next = nullptr;
    }
    else
    {
        // nothing stored so far

        // create the list item
        class BUSSIM_PAX_LISTITEM *TmpPtr;
        TmpPtr = new class BUSSIM_PAX_LISTITEM;

        // fill it
        TmpPtr->ref2pax = _REF2PAX;
        TmpPtr->previous = nullptr;
        TmpPtr->next = nullptr;

        // insert
        this->first_pax_item = TmpPtr;
        this->last_pax_item = TmpPtr;

        if( _MARKER > 0)
        {
            // set the backreferring pointer
            TmpPtr->ref2pax->Ref2WalkingPaxList = TmpPtr;
        }

        //TmpPtr = nullptr;

    }



    // increase the pax counter
    this->PAXES++;
}

void BUSSIM_PAX_LIST::print(void)
{
    class BUSSIM_PAX_LISTITEM *Ptr;

    std::cout << "This is PAX list " << this->LISTNAME << " (" << this->PAXES << " paxes)" << std::endl;
    Ptr = this->first_pax_item;
    while( Ptr != nullptr )
    {
        Ptr->ref2pax->print();
        Ptr = Ptr->next;
    }
}

long BUSSIM_PAX_LIST::get_pax_num()
{
    return( this->PAXES );
}

std::string BUSSIM_PAX_LIST::get_listname(void)
{
    return( this->LISTNAME );
}

class BUSSIM_PAX *BUSSIM_PAX_LIST::get_last_pax(void)
{
    return( this->last_pax_item->ref2pax);
}

class BUSSIM_PAX *BUSSIM_PAX_LIST::get_first_pax(void)
{
    return( this->first_pax_item->ref2pax);
}

void BUSSIM_PAX_LIST::set_listname(std::string _LISTNAME)
{
    this->LISTNAME = _LISTNAME;
}

class BUSSIM_PAX_LISTITEM *BUSSIM_PAX_LIST::get_last_pax_item(void)
{
    return( this->last_pax_item);
}

class BUSSIM_PAX_LISTITEM *BUSSIM_PAX_LIST::get_first_pax_item(void)
{
    return( this->first_pax_item);
}

double BUSSIM_PAX_LIST::get_first_show_up_time(void)
{
    return( this->first_pax_item->ref2pax->Show_Up_Time );
}

void BUSSIM_PAX_LIST::extract_item(class BUSSIM_PAX_LISTITEM *_PTR)
{
    // this procedure resolves the pax-list_item at _PTR from this pax-list

    if( this->PAXES > 1 )
    {
        //std::cout << "ABC+" << std::endl;
        if( _PTR == this->first_pax_item )
        {
            //std::cout << "XX+" << std::endl;
            // we want extract the first list item
            this->first_pax_item = _PTR->next;
            _PTR->next->previous = nullptr;
            _PTR->previous = nullptr;
            _PTR->next = nullptr;
            this->PAXES--;
            //std::cout << "XX-" << std::endl;
        }
        else
        {
            if( _PTR == this->last_pax_item )
            {
                //std::cout << "YY+" << std::endl;
                // we want to extract the last list item
                this->last_pax_item = _PTR->previous;
                _PTR->previous->next = nullptr;
                _PTR->previous = nullptr;
                _PTR->next = nullptr;
                this->PAXES--;
                //std::cout << "YY-" << std::endl;
            }
            else
            {
                //std::cout << "ZZ+" << _PTR->get_ref2pax()->ID << std::endl;
                //std::cout << "ZZ+next" << _PTR->next << std::endl;
                //std::cout << "ZZ+prev" << _PTR->previous << std::endl;
                // we want to extract an item in the middle of the list
                _PTR->previous->next = _PTR->next;
                //std::cout << "ZZ1" << _PTR->get_ref2pax()->ID << std::endl;
                _PTR->next->previous = _PTR->previous;
                //std::cout << "ZZ2" << _PTR->get_ref2pax()->ID << std::endl;
                _PTR->next = nullptr;
                //std::cout << "ZZ3" << _PTR->get_ref2pax()->ID << std::endl;
                _PTR->previous = nullptr;
                this->PAXES--;
                //std::cout << "ZZ-" << std::endl;
            }
        }
        //std::cout << "ABC-" << std::endl;
    }
    else
    {
        //std::cout << "DEF+" << std::endl;
        // there is only one item before the removal
        this->first_pax_item = nullptr;
        this->last_pax_item = nullptr;
        this->PAXES--;
        //std::cout << "DEF-" << std::endl;
    }
}

void BUSSIM_PAX_LIST::insert_item_after(class BUSSIM_PAX_LISTITEM *_PTR,class BUSSIM_PAX_LISTITEM *_REFPTR)
{
    // this procedure inserts the paxlistitem at _PTR next after the paxlist item at _REFPTR

    if( this->PAXES > 0 )
    {
        // there are already stored items in this pax list
        if( _REFPTR == this->last_pax_item )
        {
            // we append _PTR after the last so far stored item
            _PTR->previous = this->last_pax_item;
            _PTR->previous->next = _PTR;
            _PTR->next = nullptr;
            this->last_pax_item = _PTR;
            this->PAXES++;
        }
        else
        {
            // insert _PTR somewhere in the list
            std::cout << "option in BUSSIM_PAX_LIST::insert_item_after not yet implemented" << std::endl;
            exit(0);
        }
    }
    else
    {
        // no listitem stored so far
        _PTR->previous = nullptr;
        _PTR->next = nullptr;
        this->first_pax_item = _PTR;
        this->last_pax_item = _PTR;
        this->PAXES++;
    }

    // this->print();
}

class BUSSIM_PAX_LISTITEM *BUSSIM_PAX_LIST::get_next_pax_item(class BUSSIM_PAX_LISTITEM *_Ptr)
{
    return( _Ptr->next );
}

BUSSIM_PAX::BUSSIM_PAX()
{
    this->ID = -1;
    this->PAXNAME = "undefined pax";
    this->Show_Up_Time = -1.0;
    this->Origin_Node = -1;
    this->Dest_Node = -1;

    this->VELOCITY = BUSSIM_PAX_WALKING_V_KMH;

    this->Current_List = BUSSIM_PAX_UNKNOWN;
    this->Current_Activity = BUSSIM_PAX_NOTHING;

    this->TravelPath = nullptr;

    this->Ref2CurrentTPSeg = nullptr;

    this->Ref2WalkingPaxList = nullptr;

    this->Ref2PaxInList = nullptr;

    this->Ref2Vehicle = nullptr;

    this->Ref2PointWaitingList = nullptr;
    this->WalkingStartPlatform = -1;
}

BUSSIM_PAX::BUSSIM_PAX(long _ID, std::string _PAXNAME, double _SHOW_UP_TIME, long _ORIGIN_NODE, long _DESTINATION_NODE)
{
    this->ID = _ID;
    this->PAXNAME = _PAXNAME;
    this->Show_Up_Time = _SHOW_UP_TIME;
    this->Origin_Node = _ORIGIN_NODE;
    this->Dest_Node = _DESTINATION_NODE;

    // the default values applying to all passengers
    this->VELOCITY = BUSSIM_PAX_WALKING_V_KMH;

    this->Current_List = BUSSIM_PAX_UNKNOWN;
    this->Current_Activity = BUSSIM_PAX_NOTHING;

    // initialize the travel path of this passenger
    this->TravelPath = new class BUSSIM_TP("travel path of pax " + std::to_string(this->ID),this);

    // initializes the reference to the current travelpath segment of this passenger
    this->Ref2CurrentTPSeg = nullptr;

    // initializes the reference to the position of this pax in the list of walking transfer paxes
    this->Ref2WalkingPaxList = nullptr;

    // initializes the reference to the position of this pax in the list of inside paxes (PAX_IN)
    this->Ref2PaxInList = nullptr;

    this->Ref2Vehicle = nullptr;

    this->Ref2PointWaitingList = nullptr;
    this->WalkingStartPlatform = -1;

}

BUSSIM_PAX::~BUSSIM_PAX()
{

}

void BUSSIM_PAX::print(void)
{
    std::cout << "\t " << this->ID
                << "\t " << this->PAXNAME
                << "\t " << this->Show_Up_Time
                << "\t " << this->Origin_Node
                << "\t " << this->Dest_Node
                << "\t " << this->Current_List
                << "\t " << this->Current_Activity << std::endl;
}

void BUSSIM_PAX::specify(long _ID, std::string _PAXNAME, double _SHOW_UP_TIME, long _ORIGIN_NODE, long _DESTINATION_NODE)
{
    this->ID = _ID;
    this->PAXNAME = _PAXNAME;
    this->Show_Up_Time = _SHOW_UP_TIME;
    this->Origin_Node = _ORIGIN_NODE;
    this->Dest_Node = _DESTINATION_NODE;

    // the default values applying to all passengers
    this->Current_List = BUSSIM_PAX_UNKNOWN;
    this->Current_Activity = BUSSIM_PAX_NOTHING;
}

double BUSSIM_PAX::get_show_up_time(void)
{
    return( this->Show_Up_Time);
}

void BUSSIM_PAX::get_shortest_od_path_dijkstra(class BUSSIM_NETWORK *_NET)
{
    // this procedure determines the shortest path from PAX->ORIG to PAX->DEST
    // using the Dijkstra Algorithm

    //_NET->ARC[753].length = 0.0008;
    //std::cout << "prepare to start Dijkstra to get path from " << this->Origin_Node << " to " << this->Dest_Node << std::endl;

    // define the point structure
    struct SPP_POINT {
        int EXT_ID;
        int ID;
        double LABELVALUE;
        int PRED;
        int permanent;
    };

    // phase 1 - determine the pax-travel-path

    // declare the array of nodes of this network
    struct SPP_POINT *NODE;
    NODE = new struct SPP_POINT[_NET->POINTS];

    // set the initial values for the dijkstra algorithm
    for(int i=0 ; i < _NET->POINTS ; i++)
    {
        NODE[i].EXT_ID = _NET->PT[i].ID;
        NODE[i].ID = i;
        NODE[i].LABELVALUE = BUSSIM_BIG_M;
        NODE[i].PRED = -1;
        NODE[i].permanent = 0;
    }

    // allocate memory for the local distance matrix
    double **DISTMAT;
    DISTMAT = new double* [_NET->POINTS];
    for( int i=0 ; i < _NET->POINTS ; i++ )
    {
        DISTMAT[i] = new double [_NET->POINTS];
        for( int j=0 ; j < _NET->POINTS ; j++ )
            DISTMAT[i][j] = BUSSIM_BIG_M;
    }

    // fill the distance matrix
    // we set the distance matrix for arcs that are covered by a line or are of the walking type
    for( int a=0 ; a < _NET->ARCS ; a++ )
    {
        if( 1 < 2)
        {
            if( _NET->ARC[a].ROUTING_TYPE == 1 )
            {
                // we have a transfer arc (by 0.99-multiplikation: avoidance of use of other arcs / Ringlinienproblem)
                DISTMAT[_NET->ARC[a].orig_node][_NET->ARC[a].dest_node] = 0.99*_NET->ARC[a].length;
            }
            else
            {
                // we have a regular arc - check if it served
                if( _NET->ARC[a].SERVED_BY_REGULAR_LINE == 1 )
                {
                    DISTMAT[_NET->ARC[a].orig_node][_NET->ARC[a].dest_node] = 1.0*_NET->ARC[a].length;
                }
            }
        }
    }

    if( this->ID == 37 )
    {
        std::cout << std::endl;
        std::cout << _NET->ARC[566].ID<< " / " << _NET->ARC[566].length
            << " / " << _NET->ARC[566].SERVED_BY_REGULAR_LINE
            << " / " << _NET->SERVICE[12][_NET->ARC[566].orig_node][_NET->ARC[566].dest_node]<< std::endl;
        std::cout << _NET->ARC[567].ID<< " / " << _NET->ARC[567].length
            << " / " << _NET->ARC[567].SERVED_BY_REGULAR_LINE
            << " / " << _NET->SERVICE[12][_NET->ARC[567].orig_node][_NET->ARC[567].dest_node]
            << std::endl;
        std::cout << _NET->ARC[568].ID<< " / " << _NET->ARC[568].length
            << " / " << _NET->ARC[568].SERVED_BY_REGULAR_LINE
            << " / " << _NET->SERVICE[12][_NET->ARC[568].orig_node][_NET->ARC[568].dest_node]
            << std::endl;
        std::cout << _NET->ARC[569].ID<< " / " << _NET->ARC[569].length
            << " / " << _NET->ARC[569].SERVED_BY_REGULAR_LINE
            << " / " << _NET->SERVICE[12][_NET->ARC[569].orig_node][_NET->ARC[569].dest_node]<< std::endl;
        std::cout << _NET->ARC[618].ID<< " / " << _NET->ARC[618].length
            << " / " << _NET->ARC[618].SERVED_BY_REGULAR_LINE
            << " / " << _NET->SERVICE[12][_NET->ARC[618].orig_node][_NET->ARC[618].dest_node]<< std::endl;
    }
    //exit(0);

    // perform dijkstra algorithm steps
    long START_NODE = this->Origin_Node;
    long STOP_NODE = this->Dest_Node;


    // to store the local code of the next permanently labelled node
    double MinLabelValue;
    int NextNodePerm;
    NextNodePerm = START_NODE;
    NODE[NextNodePerm].LABELVALUE = 0.0;
    NODE[NextNodePerm].PRED = -1;

    for(int CNTR=0 ; CNTR < _NET->POINTS ; CNTR++)
    {
        // std::cout << "DIJKSTRA STEP " << CNTR << "/"<<_NET->POINTS<<" (NextNodePerm=" << NextNodePerm << ")" << std::endl;
        // mark NextNodePerm as permanent
        NODE[NextNodePerm].permanent = 1;

        // update temporarily labelled nodes
        for(int CNTR_A=0 ; CNTR_A < _NET->POINTS ; CNTR_A++)
        {
            if( NODE[CNTR_A].permanent < 1 )
            {
                // we check only temporarily labelled nodes
                if( DISTMAT[NextNodePerm][CNTR_A] < BUSSIM_BIG_M )
                {
                    // we have another path extension
                    if( (NODE[NextNodePerm].LABELVALUE + DISTMAT[NextNodePerm][CNTR_A])
                                            < NODE[CNTR_A].LABELVALUE )
                    {
                        // we have a path length reduction
                        NODE[CNTR_A].LABELVALUE = NODE[NextNodePerm].LABELVALUE + DISTMAT[NextNodePerm][CNTR_A];
                        NODE[CNTR_A].PRED = NextNodePerm;
                    }
                }
            }
        }

        // identify the next NextNodePerm
        NextNodePerm = -1;
        MinLabelValue = BUSSIM_BIG_M;
        for(int CNTR_A=0 ; CNTR_A < _NET->POINTS ; CNTR_A++)
        {
            if( NODE[CNTR_A].permanent < 1 )
            {
                if( NODE[CNTR_A].LABELVALUE <= MinLabelValue )
                {
                    MinLabelValue = NODE[CNTR_A].LABELVALUE;
                    NextNodePerm = CNTR_A;
                }
            }
        }
    }

    if( _NET->POINTS > 0 )
    {
        for( int i=0 ; i < _NET->POINTS ; i++ )
            delete[] DISTMAT[i];
        delete[] DISTMAT;
    }

    // reconstruct the determined path (if such a path exists)
    if( NODE[STOP_NODE].LABELVALUE < BUSSIM_BIG_M )
    {
        // there exists a shortest path from current post to start node of specified line

        //std::cout << "Total Duration " << NODE[STOP_NODE].LABELVALUE << std::endl;
        //std::cout << "-> final node=" << NODE[STOP_NODE].ID << std::endl;
        //int PredNode = NODE[STOP_NODE].PRED;

        int PredNode = STOP_NODE;
        //std::cout << "------" << std::endl;
        while(PredNode >= 0)
        {
            //std::cout << "Total Duration " << NODE[PredNode].LABELVALUE << std::endl;
            //std::cout << "-> final node=" << NODE[PredNode].ID << std::endl;
            if( NODE[PredNode].PRED >= 0)
            {
                //std::cout << "Arc " << NODE[PredNode].PRED << "->" << PredNode << std::endl;
                class BUSSIM_ARC _ARC;
                _ARC = _NET->get_arc_by_od(NODE[PredNode].PRED,PredNode);
                //std::cout << "Arc " << _ARC.orig_node << "->" << _ARC.dest_node;
                //std::cout << " Arc-ID: " << _ARC.ID;
                //std::cout << " Type=" << _ARC.ROUTING_TYPE;
                //std::cout << std::endl;

                // create the segment
                class BUSSIM_TP_SEG *_NewSeg;
                if( _ARC.ROUTING_TYPE == BUSSIM_RT_VEHICLE )
                {
                    _NewSeg = new class BUSSIM_TP_SEG(BUSSIM_PAX_RIDING,-2,-1.0,-1.0,&_NET->ARC[_ARC.ID],nullptr);
                    this->TravelPath->insert_as_first_segment(_NewSeg);
                }
                if( _ARC.ROUTING_TYPE == BUSSIM_RT_WALK )
                {
                    _NewSeg = new class BUSSIM_TP_SEG(BUSSIM_PAX_WALKING,-2,-1.0,-1.0,&_NET->ARC[_ARC.ID],nullptr);
                    this->TravelPath->insert_as_first_segment(_NewSeg);
                }

            }
            PredNode = NODE[PredNode].PRED;
        }

    }
    else
    {
        // we cannot complete the vehicle rotation and schedule
        std::cout << "completion of path of passenger " << this->ID << " impossible! (A)" << std::endl;
        exit(0);
    }

    if( this->ID == 37 )
        {
    std::cout << "GAGA-START" << std::endl;
    this->TravelPath->print_on_console(_NET);
    std::cout << "GAGA-END" << std::endl;
        }

    // phase 2 consolidate the list of included non-infrastructure stops
    class BUSSIM_TP_SEG *Ptr;
    Ptr = this->TravelPath->REF2FIRST_TP_SEG;
    long stop_cntr = 0;
    while( Ptr != nullptr )
    {
        if( Ptr->REF2ARC != nullptr )
        {
            if( Ptr->REF2ARC->ROUTING_TYPE == BUSSIM_RT_VEHICLE )
            {
                if( _NET->PT[Ptr->REF2ARC->orig_node].INFRASTRUCTURE == 0 )
                {
                    //std::cout << "point with ID " << Ptr->REF2ARC->orig_node << std::endl;
                    stop_cntr++;
                }
            }
            if( Ptr->REF2ARC->ROUTING_TYPE == BUSSIM_RT_WALK )
            {
                if( _NET->PT[Ptr->REF2ARC->orig_node].INFRASTRUCTURE == 0 )
                {
                    //std::cout << "point with ID " << Ptr->REF2ARC->orig_node << std::endl;
                    stop_cntr++;
                }
            }
        }
        if( Ptr->next == nullptr )
        {
            if( Ptr->REF2ARC->ROUTING_TYPE == BUSSIM_RT_VEHICLE )
            {
                if( _NET->PT[Ptr->REF2ARC->dest_node].INFRASTRUCTURE == 0 )
                {
                    //std::cout << "point with ID " << Ptr->REF2ARC->dest_node << std::endl;
                    stop_cntr++;
                }
            }
            if( Ptr->REF2ARC->ROUTING_TYPE == BUSSIM_RT_WALK )
            {
                if( _NET->PT[Ptr->REF2ARC->dest_node].INFRASTRUCTURE == 0 )
                {
                    //std::cout << "point with ID " << Ptr->REF2ARC->dest_node << std::endl;
                    stop_cntr++;
                }
            }

        }
        Ptr = Ptr->next;
    }

    struct INT_PT {
        long ID;
        long NETWORK_ID;
        long LABEL_VALUE;
        long PERMANENT;
        long PRED;

    };

    struct INT_PT *STOPLIST;
    STOPLIST = new struct INT_PT[stop_cntr];

    Ptr = this->TravelPath->REF2FIRST_TP_SEG;
    stop_cntr = 0;
    while( Ptr != nullptr )
    {
        if( Ptr->REF2ARC != nullptr )
        {
            if( Ptr->REF2ARC->ROUTING_TYPE == BUSSIM_RT_VEHICLE )
            {
                if( _NET->PT[Ptr->REF2ARC->orig_node].INFRASTRUCTURE == 0 )
                {
                    STOPLIST[stop_cntr].ID = stop_cntr;
                    STOPLIST[stop_cntr].NETWORK_ID = Ptr->REF2ARC->orig_node;
                    STOPLIST[stop_cntr].LABEL_VALUE = BUSSIM_BIG_M;
                    STOPLIST[stop_cntr].PERMANENT = 0;
                    STOPLIST[stop_cntr].PRED = -1;
                    stop_cntr++;
                }
            }
            if( Ptr->REF2ARC->ROUTING_TYPE == BUSSIM_RT_WALK )
            {
                if( _NET->PT[Ptr->REF2ARC->orig_node].INFRASTRUCTURE == 0 )
                {
                    STOPLIST[stop_cntr].ID = stop_cntr;
                    STOPLIST[stop_cntr].NETWORK_ID = Ptr->REF2ARC->orig_node;
                    STOPLIST[stop_cntr].LABEL_VALUE = BUSSIM_BIG_M;
                    STOPLIST[stop_cntr].PERMANENT = 0;
                    STOPLIST[stop_cntr].PRED = -1;
                    stop_cntr++;
                }
            }

        }
        if( Ptr->next == nullptr )
        {
            if( Ptr->REF2ARC->ROUTING_TYPE == BUSSIM_RT_VEHICLE )
            {
                if( _NET->PT[Ptr->REF2ARC->dest_node].INFRASTRUCTURE == 0 )
                {
                    STOPLIST[stop_cntr].ID = stop_cntr;
                    STOPLIST[stop_cntr].NETWORK_ID = Ptr->REF2ARC->dest_node;
                    STOPLIST[stop_cntr].LABEL_VALUE = BUSSIM_BIG_M;
                    STOPLIST[stop_cntr].PERMANENT = 0;
                    STOPLIST[stop_cntr].PRED = -1;
                    stop_cntr++;
                }
            }
            if( Ptr->REF2ARC->ROUTING_TYPE == BUSSIM_RT_WALK )
            {
                if( _NET->PT[Ptr->REF2ARC->dest_node].INFRASTRUCTURE == 0 )
                {
                    STOPLIST[stop_cntr].ID = stop_cntr;
                    STOPLIST[stop_cntr].NETWORK_ID = Ptr->REF2ARC->dest_node;
                    STOPLIST[stop_cntr].LABEL_VALUE = BUSSIM_BIG_M;
                    STOPLIST[stop_cntr].PERMANENT = 0;
                    STOPLIST[stop_cntr].PRED = -1;
                    stop_cntr++;
                }
            }
        }
        Ptr = Ptr->next;
    }

    if( this->ID == 37 )
    {
   std::cout << "Path at the end of phase 2" << std::endl;
    for(int i=0 ; i < stop_cntr ; i++ )
        std::cout << "i=" << STOPLIST[i].ID << ": " << STOPLIST[i].NETWORK_ID << std::endl;
    }


    // phase 3 - create the list of auxiliary arcs

    // define the arc type
    struct _AUX_ARC {
        long ID;
        long INT_START_NODE;
        long EXT_START_NODE;
        long INT_ZIEL_NODE;
        long EXT_ZIEL_NODE;
        long LINE_ID;
        long LENGTHVALUE;
    };

    // determine the number of possible arcs
    int arc_cntr=0;
    int serviced=0; // to count the number of available lines
    for( int l=0 ; l < _NET->EXT_LINES ; l++ )
    {
        if( _NET->LINE[l].type == BUSSIM_REGULARLINE )
        {
            for( long a=0 ; a < stop_cntr-1 ; a++ )
                for( long b=a+1 ; b < stop_cntr ; b++ )
                    if( _NET->SERVICE[l][STOPLIST[a].NETWORK_ID][STOPLIST[b].NETWORK_ID] > 0.5 )
                    {

                        //std::cout << "LINE " << _NET->LINE[l].LineName << "("<<_NET->LINE[l].ID<<"): ";
                        //std::cout << " from " << STOPLIST[a].NETWORK_ID << " to " << STOPLIST[b].NETWORK_ID;
                        //std::cout << std::endl;

                        arc_cntr++;
                    }
        }
    }

    // now add the transfer arcs
    for( long s=0 ; s < stop_cntr-1 ; s++ )
    {
        serviced = 0;
        for( long l=0 ; l < _NET->EXT_LINES ; l++ )
            if( _NET->SERVICE[l][STOPLIST[s].NETWORK_ID][STOPLIST[s+1].NETWORK_ID] > 0 )
                serviced++;

        if( serviced==0 )
        {
            //std::cout << "Transfer:";
            //std::cout << " from " << STOPLIST[s].NETWORK_ID << " to " << STOPLIST[s+1].NETWORK_ID;
            //std::cout << std::endl;
            arc_cntr++;
        }
    }
    //std::cout << "we have " << arc_cntr << " auxiliary arcs" << std::endl;

    // ready to create the auxiliary arcs
    struct _AUX_ARC *AUX_ARC;
    AUX_ARC = new struct _AUX_ARC[arc_cntr];

    arc_cntr=0;
    for( int l=0 ; l < _NET->EXT_LINES ; l++ )
    {
        if( _NET->LINE[l].type == BUSSIM_REGULARLINE )
        {
            for( long a=0 ; a < stop_cntr-1 ; a++ )
                for( long b=a+1 ; b < stop_cntr ; b++ )
                    if( _NET->SERVICE[l][STOPLIST[a].NETWORK_ID][STOPLIST[b].NETWORK_ID] > 0.5 )
                    {
                        AUX_ARC[arc_cntr].ID = arc_cntr;
                        AUX_ARC[arc_cntr].LINE_ID = _NET->LINE[l].ID;
                        AUX_ARC[arc_cntr].INT_START_NODE = STOPLIST[a].ID;
                        AUX_ARC[arc_cntr].EXT_START_NODE = STOPLIST[a].NETWORK_ID;
                        AUX_ARC[arc_cntr].INT_ZIEL_NODE = STOPLIST[b].ID;
                        AUX_ARC[arc_cntr].EXT_ZIEL_NODE = STOPLIST[b].NETWORK_ID;
                        if( b != (stop_cntr-1) )
                            AUX_ARC[arc_cntr].LENGTHVALUE = 1;
                        else
                            AUX_ARC[arc_cntr].LENGTHVALUE = 0;

                        //std::cout << "REGULARLINE " << _NET->LINE[l].LineName << "("<<_NET->LINE[l].ID<<"): ";
                        //std::cout << " from " << STOPLIST[a].NETWORK_ID << " to " << STOPLIST[b].NETWORK_ID;
                        //std::cout << std::endl;

                        arc_cntr++;
                    }
        }
    }

    // now add the transfer arcs
    for( long s=0 ; s < stop_cntr-1 ; s++ )
    {
        //std::cout << "s=" << s << " START: "<< STOPLIST[s].NETWORK_ID << "->" << STOPLIST[s+1].NETWORK_ID << std::endl;
        serviced = 0;
        for( long l=0 ; l < _NET->EXT_LINES ; l++ )
            if( _NET->SERVICE[l][STOPLIST[s].NETWORK_ID][STOPLIST[s+1].NETWORK_ID] > 0.5 )
            {
                //std::cout << "served by line "  << l << std::endl;
                serviced++;
            }


        //std::cout << "serviced=" << serviced << std::endl;

        if( serviced==0 )
        {
            AUX_ARC[arc_cntr].ID = arc_cntr;
            AUX_ARC[arc_cntr].LINE_ID = -1;
            AUX_ARC[arc_cntr].INT_START_NODE = STOPLIST[s].ID;
            AUX_ARC[arc_cntr].EXT_START_NODE = STOPLIST[s].NETWORK_ID;
            AUX_ARC[arc_cntr].INT_ZIEL_NODE = STOPLIST[s+1].ID;
            AUX_ARC[arc_cntr].EXT_ZIEL_NODE = STOPLIST[s+1].NETWORK_ID;
            if( (s+1) != (stop_cntr-1) )
                AUX_ARC[arc_cntr].LENGTHVALUE = 1;
            else
                AUX_ARC[arc_cntr].LENGTHVALUE = 0;
            //std::cout << "->Transfer on ARC "<< AUX_ARC[arc_cntr].ID<<":";
            //std::cout << " from " << STOPLIST[s].NETWORK_ID << " to " << STOPLIST[s+1].NETWORK_ID;
            //std::cout << std::endl;
            arc_cntr++;
        }
        //std::cout << "s=" << "END: " << STOPLIST[s].NETWORK_ID << "->" << STOPLIST[s+1].NETWORK_ID << std::endl;
    }


    for( long a=0 ; a < arc_cntr ; a++ )
    {
        std::cout << AUX_ARC[a].ID << "\t";
        std::cout << AUX_ARC[a].LINE_ID << "\t";
        std::cout << AUX_ARC[a].EXT_START_NODE << "\t";
        std::cout << AUX_ARC[a].EXT_ZIEL_NODE << "\t";
        std::cout << AUX_ARC[a].LENGTHVALUE;
        std::cout << std::endl;
    }


    //std::cout << "we have reached end of phase 4" << std::endl;
    // phase 4 - find the shortest transfer connection

    // initialize the distance matrix
    double **DIST_2;
    DIST_2 = new double* [stop_cntr];
    for( int i=0 ; i < stop_cntr ; i++ )
    {
        DIST_2[i] = new double [stop_cntr];
        for( int j=0 ; j < stop_cntr ; j++ )
            DIST_2[i][j] = BUSSIM_BIG_M;
    }

    // to store the best service
    int **LINE_2;
    LINE_2 = new int* [stop_cntr];
    for( int i=0 ; i < stop_cntr ; i++ )
    {
        LINE_2[i] = new int [stop_cntr];
        for( int j=0 ; j < stop_cntr ; j++ )
            LINE_2[i][j] = -1;  // go be feet
    }

    // reduce the distance for all auxiliary arcs
    for( int a=0 ; a < arc_cntr ; a++ )
    {
        if( AUX_ARC[a].LENGTHVALUE < DIST_2[AUX_ARC[a].INT_START_NODE][AUX_ARC[a].INT_ZIEL_NODE] )
        {
            DIST_2[AUX_ARC[a].INT_START_NODE][AUX_ARC[a].INT_ZIEL_NODE] = AUX_ARC[a].LENGTHVALUE;
            LINE_2[AUX_ARC[a].INT_START_NODE][AUX_ARC[a].INT_ZIEL_NODE] = AUX_ARC[a].LINE_ID;
        }
    }

    // execute the Dijkstra steps

    // init - specify the origin and the destination location
    long START_OP = 0;
    long STOP_OP = stop_cntr-1;
    long NextOpPerm = START_OP;
    STOPLIST[NextOpPerm].LABEL_VALUE = 0.0;
    STOPLIST[NextOpPerm].PRED = -1;

    for(int CNTR=0 ; CNTR < stop_cntr ; CNTR++)
    {
        // std::cout << "DIJKSTRA STEP " << CNTR << "/"<<_NET->POINTS<<" (NextNodePerm=" << NextNodePerm << ")" << std::endl;
        // mark NextNodePerm as permanent
        STOPLIST[NextOpPerm].PERMANENT = 1;

        // update temporarily labelled nodes
        for(int CNTR_A=0 ; CNTR_A < stop_cntr ; CNTR_A++)
        {
            if( STOPLIST[CNTR_A].PERMANENT < 1 )
            {
                // we check only temporarily labelled nodes
                if( DIST_2[NextOpPerm][CNTR_A] < BUSSIM_BIG_M )
                {
                    // we have another path extension
                    if( (STOPLIST[NextOpPerm].LABEL_VALUE + DIST_2[NextOpPerm][CNTR_A])
                                            < STOPLIST[CNTR_A].LABEL_VALUE )
                    {
                        // we have a path length reduction
                        STOPLIST[CNTR_A].LABEL_VALUE = STOPLIST[NextOpPerm].LABEL_VALUE + DIST_2[NextOpPerm][CNTR_A];
                        STOPLIST[CNTR_A].PRED = NextOpPerm;
                    }
                }
            }
        }

        // identify the next NextNodePerm
        NextOpPerm = -1;
        MinLabelValue = BUSSIM_BIG_M;
        for(int CNTR_A=0 ; CNTR_A < stop_cntr ; CNTR_A++)
        {
            if( STOPLIST[CNTR_A].PERMANENT < 1 )
            {
                if( STOPLIST[CNTR_A].LABEL_VALUE <= MinLabelValue )
                {
                    MinLabelValue = STOPLIST[CNTR_A].LABEL_VALUE;
                    NextOpPerm = CNTR_A;
                }
            }
        }
    }

    // re-construct the path

    // reconstruct the determined path (if such a path exists)
    int cntr_2=0;
    if( STOPLIST[STOP_OP].LABEL_VALUE < BUSSIM_BIG_M )
    {
        // there exists a shortest path from current post to start node of specified line

        //std::cout << "Total Duration " << NODE[STOP_NODE].LABELVALUE << std::endl;
        //std::cout << "-> final node=" << NODE[STOP_NODE].ID << std::endl;
        //int PredNode = NODE[STOP_NODE].PRED;

        long PredOP = STOP_OP;
        // std::cout << "--ABC----" << std::endl;
        while(PredOP >= 0)
        {
            //std::cout << "Total Duration " << NODE[PredNode].LABELVALUE << std::endl;
            //std::cout << "-> final node=" << NODE[PredNode].ID << std::endl;
            if( STOPLIST[PredOP].PRED >= 0)
            {
                //std::cout << "AUXARC from " << STOPLIST[STOPLIST[PredOP].PRED].NETWORK_ID << "->" << STOPLIST[STOPLIST[PredOP].ID].NETWORK_ID;
                //std::cout << "\t using line " << LINE_2[STOPLIST[PredOP].PRED][STOPLIST[PredOP].ID];
                //std::cout << std::endl;
                cntr_2++;
            }
            PredOP = STOPLIST[PredOP].PRED;
        }

    }
    else
    {
        // we cannot complete the vehicle rotation and schedule
        std::cout << "completion of detailed travel path of passenger " << this->ID << " impossible! (B)" << std::endl;
        exit(0);
    }

    //std::cout << "we have " << cntr_2 << " main segments in the path of this passenger";
    //std::cout << std::endl;

    // save the determine path
    struct _AUX_SEG {
        long START;
        long STOP;
        long line;
    };

    struct _AUX_SEG *TMP_SEG;
    TMP_SEG = new struct _AUX_SEG[cntr_2];

    int aux_cntr=0;
    if( STOPLIST[STOP_OP].LABEL_VALUE < BUSSIM_BIG_M )
    {
        // there exists a shortest path from current post to start node of specified line

        long PredOP = STOP_OP;
        while(PredOP >= 0)
        {
            if( STOPLIST[PredOP].PRED >= 0)
            {
                TMP_SEG[cntr_2 - aux_cntr - 1].START = STOPLIST[STOPLIST[PredOP].PRED].NETWORK_ID;
                TMP_SEG[cntr_2 - aux_cntr - 1].STOP = STOPLIST[STOPLIST[PredOP].ID].NETWORK_ID;
                TMP_SEG[cntr_2 - aux_cntr - 1].line = LINE_2[STOPLIST[PredOP].PRED][STOPLIST[PredOP].ID];
                aux_cntr++;
            }
            PredOP = STOPLIST[PredOP].PRED;
        }

    }
    else
    {
        // we cannot complete the vehicle rotation and schedule
        std::cout << "completion of detailed travel path of passenger " << this->ID << " impossible (2)!" << std::endl;
        exit(0);
    }

    /*
    if( this->ID == 5836 )
        {
            std::cout << "-------------------------------------" << std::endl;
            for( long i=0 ; i < cntr_2 ; i++ )
            {
                std::cout << "i=" << i << "\t";
                std::cout << TMP_SEG[i].START << "->" << TMP_SEG[i].STOP;
                std::cout << "\t line=" << TMP_SEG[i].line;
                std::cout << std::endl;
            }
            std::cout << "-------------------------------------" << std::endl;
            this->TravelPath->print_on_console(_NET);
            std::cout << "-------------------------------------" << std::endl;
        }
    */

    Ptr = this->TravelPath->REF2FIRST_TP_SEG;
    for( long i=0 ; i < cntr_2 ; i++ )
    {
        Ptr->Line = TMP_SEG[i].line;
        Ptr = Ptr->next;
    }
    //std::cout << "ABCDEF-START" << std::endl;
    //this->TravelPath->print_on_console(_NET);
    //std::cout << "ABCDEF-ENDE" << std::endl;
    // phase 5 - complete the passenger path

    Ptr = this->TravelPath->REF2FIRST_TP_SEG;
    class BUSSIM_TP_SEG *Ptr2, *NewOp;
    for( long i=0 ; i < cntr_2 ; i++ )
    {
        if( TMP_SEG[i].line >= 0 )
        {
            // we have a riding part

            // check if this is the first segment using a riding service
            if( this->TravelPath->FIRST_LINE == -2 )
                this->TravelPath->FIRST_LINE = TMP_SEG[i].line;

            // look for the end (TMP_SEG[i].STOP)
            if (Ptr != nullptr)
            {

            if (Ptr->next != nullptr)
            {
                // create a waiting segment
                NewOp = new class BUSSIM_TP_SEG(BUSSIM_PAX_WAITING,-2,0.0,0.0,nullptr,&_NET->PT[Ptr->REF2ARC->orig_node]);
                // insert a waiting segment
                this->TravelPath->insert_before(Ptr,NewOp);
                //this->TravelPath->print_on_console(_NET);
                NewOp = nullptr;
                Ptr2 = Ptr;
                while( Ptr2->REF2ARC->dest_node != TMP_SEG[i].STOP )
                {
                    Ptr2->Line = TMP_SEG[i].line;
                    Ptr2 = Ptr2->next;
                }
                Ptr2->Line = TMP_SEG[i].line;
                Ptr = Ptr2->next;
            }
            else
                Ptr = Ptr->next;
            }
        }
        else
        {
            // we have a transfer part
            if( Ptr != nullptr )
            {
                // check if this is the first segment requires a walk
                if( this->TravelPath->FIRST_LINE == -2 )
                    this->TravelPath->FIRST_LINE = -1;

                Ptr2 = Ptr;
                while( Ptr2->REF2ARC->dest_node != TMP_SEG[i].STOP )
                {
                    Ptr2->Line = TMP_SEG[i].line;
                    Ptr2 = Ptr2->next;
                }
                Ptr2->Line = TMP_SEG[i].line;
                Ptr = Ptr2->next;
            }
        }
    }

    //std::cout << "GAGA-17" << std::endl;
    delete[] TMP_SEG;

    for( int i=0 ; i < stop_cntr ; i++ )
        delete[] LINE_2[i];
    delete[] LINE_2;

    for( int i=0 ; i < stop_cntr ; i++ )
        delete[] DIST_2[i];
    delete[] DIST_2;


    if( this->TravelPath->SEGMENTS == 1 )
    {
        // we have to find a service for the one and only ride
        if( this->TravelPath->REF2FIRST_TP_SEG->get_line() < 0 )
        {
            for( int s=0 ; s < _NET->EXT_LINES ; s++ )
            {
                if( _NET->LINE[s].type == BUSSIM_REGULARLINE )
                {
                    if( _NET->SERVICE[s][this->TravelPath->REF2FIRST_TP_SEG->get_ref2arc()->orig_node][this->TravelPath->REF2FIRST_TP_SEG->get_ref2arc()->dest_node] > 0.5 )
                    {
                        this->TravelPath->REF2FIRST_TP_SEG->Line = s;
                        s = _NET->EXT_LINES+1;
                    }
                }
            }
            if( this->TravelPath->REF2FIRST_TP_SEG->get_line() < 0 )
            {
                std::cout << "CANNOT FIND A VALID TRAVEl PATH FOR PAX " << this->ID << std::endl;
                exit(0);
            }
        }
    }
    //std::cout << "GAGA-18" << std::endl;

    delete[] AUX_ARC;
    delete[] STOPLIST;

    // 5a setzen der Linien fr Walkings
    Ptr = this->TravelPath->REF2FIRST_TP_SEG;
    while( Ptr != nullptr )
    {
        if( Ptr->Activity == BUSSIM_PAX_WALKING )
            Ptr->Line = -1;
        Ptr = Ptr->next;
    }

    // std::cout << "GAGA-19" << std::endl;

    // 5b vertauschen WAITING/WALKING
    class BUSSIM_TP_SEG *TmpPtr;
    long NewWaitPos=0;
    if( this->TravelPath->SEGMENTS > 1 )
        Ptr = this->TravelPath->REF2FIRST_TP_SEG->next;
    else
        Ptr = this->TravelPath->REF2FIRST_TP_SEG;
    while( Ptr->next != nullptr )
    {
        if( Ptr->Activity == BUSSIM_PAX_WAITING )
        {
            if( Ptr->next->Activity == BUSSIM_PAX_WALKING )
            {
                //std::cout<<"GAGA-19a" << std::endl;
                // swap of Ptr and Ptr->next necessary
                TmpPtr = Ptr->next;
                NewWaitPos = TmpPtr->REF2ARC->dest_node;
                this->TravelPath->dissolve_segment(TmpPtr);
                this->TravelPath->insert_before(Ptr,TmpPtr);
                Ptr->REF2POINT = &_NET->PT[NewWaitPos];
                //std::cout<<"GAGA-19b" << std::endl;
            }
        }
        Ptr = Ptr->next;
    }


    // std::cout << "GAGA-20" << std::endl;

    // step 5c ensure that the first segment is a waiting segment
    Ptr = this->TravelPath->REF2FIRST_TP_SEG;
    if( Ptr != nullptr )
    {
        // there is a first segment
        if( Ptr->Activity != BUSSIM_PAX_WAITING)
        {
            // create a waiting segment
            NewOp = new class BUSSIM_TP_SEG(BUSSIM_PAX_WAITING,-2,0.0,0.0,nullptr,&_NET->PT[Ptr->REF2ARC->orig_node]);
            // insert a waiting segment
            this->TravelPath->insert_before(Ptr,NewOp);
        }
    }
    // step 5d look for the first aktivity <> BUSSIM_WAITING and save the line-number
    Ptr = this->TravelPath->REF2FIRST_TP_SEG;
    while( Ptr != nullptr )
    {
        if( Ptr->Activity != BUSSIM_PAX_WAITING )
        {
            this->TravelPath->FIRST_LINE = Ptr->Line;
            Ptr = nullptr;
        }
        else
        {
            Ptr = Ptr->next;
        }

    }

    // check 5e look if there is a walking activity without a subsequent waiting activity
    if( this->TravelPath->SEGMENTS > 1)
    {
        Ptr = this->TravelPath->REF2FIRST_TP_SEG->next;

        while( Ptr != nullptr )
        {
            if( Ptr->Activity == BUSSIM_PAX_WALKING )
            {
                if( Ptr->next != nullptr )
                {
                    if( Ptr->next->Activity != BUSSIM_PAX_WAITING )
                    {
                        // we have to add a waiting activity
                        // create a waiting segment
                        NewOp = new class BUSSIM_TP_SEG(BUSSIM_PAX_WAITING,-1,0.0,0.0,nullptr,&_NET->PT[Ptr->REF2ARC->dest_node]);
                        // insert a waiting segment
                        this->TravelPath->insert_before(Ptr->next,NewOp);
                    }
                }
            }

                Ptr = Ptr->next;

        }

    }
    this->Ref2CurrentTPSeg = nullptr;
}

void BUSSIM_PAX::display_on_screen(class BUSSIM_NETWORK *_NET)
{
    this->TravelPath->print_on_console(_NET);
}

BUSSIM_TP_SEG::BUSSIM_TP_SEG()
{
    this->Activity = BUSSIM_PAX_NOTHING;
    this->Enter_Point = -1;
    this->Leave_Point = -1;
    this->Line = -2;      //(-1 = walking, >=0 lines)

    this->POS = 0.0;

    this->Enter_Time = -1.0;
    this->Leave_Time = -1.0;

    this->REF2ARC = nullptr;
    this->REF2POINT = nullptr;

    this->REF2VEHICLE = nullptr;

    this->previous = nullptr;
    this->next = nullptr;

}

BUSSIM_TP_SEG::BUSSIM_TP_SEG(long _Activity, long _Line, double _Enter_Time, double _Leave_Time, class BUSSIM_ARC *_REF2ARC, class BUSSIM_POINT *_REF2POINT)
{
    this->Activity = _Activity;
    this->Enter_Point = -1;
    this->Leave_Point = -1;
    this->Line = -2;      //(-1 = walking, >=0 lines)

    this->POS = 0.0;

    this->Enter_Time = _Enter_Time;
    this->Leave_Time = _Leave_Time;

    this->REF2ARC = _REF2ARC;
    this->REF2POINT = _REF2POINT;

    this->REF2VEHICLE = nullptr;

    this->previous = nullptr;
    this->next = nullptr;

}

BUSSIM_TP_SEG::~BUSSIM_TP_SEG()
{
    this->REF2ARC = nullptr;
    this->REF2POINT = nullptr;

    this->REF2VEHICLE = nullptr;

    this->previous = nullptr;
    this->next = nullptr;
}

BUSSIM_TP::BUSSIM_TP()
{
    this->SEGMENTS = 0;
    this->DESC = "undef travel path of a passenger";

    this->FIRST_LINE = -2;

    this->REF2PAX = nullptr;

    this->REF2FIRST_TP_SEG = nullptr;
    this->REF2LAST_TP_SEG = nullptr;

}

BUSSIM_TP::BUSSIM_TP(std::string _DESC, class BUSSIM_PAX *_REF2PAX)
{
    this->SEGMENTS = 0;
    this->DESC = _DESC;

    this->FIRST_LINE = -2;

    this->REF2PAX =_REF2PAX;

    this->REF2FIRST_TP_SEG = nullptr;
    this->REF2LAST_TP_SEG = nullptr;

}

BUSSIM_TP::~BUSSIM_TP()
{

}

void BUSSIM_TP::append_segment(class BUSSIM_TP_SEG *_SEG)
{

}

void BUSSIM_TP::print_on_console(class BUSSIM_NETWORK *_NET)
{
    class BUSSIM_TP_SEG *_TP_SEG;

    long SEG_CNTR=0;

    _TP_SEG = this->REF2FIRST_TP_SEG;

    std::cout << this->DESC << " (" << this->SEGMENTS << " segments)" << std::endl;
    while( _TP_SEG != nullptr )
    {
        if( _TP_SEG->Activity == BUSSIM_PAX_RIDING )
        {
            if( _TP_SEG->REF2ARC != nullptr )
            {
                std::cout << "SEG "<< SEG_CNTR<<": ON ARC: " << _TP_SEG->REF2ARC->ID;
                std::cout << "\t RIDING\t ";
                std::cout << _TP_SEG->REF2ARC->orig_node << "->" << _TP_SEG->REF2ARC->dest_node;
                std::cout << " on line " << _TP_SEG->Line << std::endl;
            }
        }
        if( _TP_SEG->Activity == BUSSIM_PAX_WALKING )
        {
            if( _TP_SEG->REF2ARC != nullptr )
            {
                std::cout << "SEG "<< SEG_CNTR<<": ON ARC: " << _TP_SEG->REF2ARC->ID;
                std::cout << "\t WALKING\t ";
                std::cout << _TP_SEG->REF2ARC->orig_node << "->" << _TP_SEG->REF2ARC->dest_node;
                std::cout << " on line " << _TP_SEG->Line;
                std::cout << " length_screen=" << _TP_SEG->REF2ARC->length_screen << std::endl;
            }
        }
        if( _TP_SEG->Activity == BUSSIM_PAX_WAITING )
        {
            if( _TP_SEG->REF2POINT != nullptr )
            {
                std::cout << "SEG "<< SEG_CNTR<<": AT POINT: " << _TP_SEG->REF2POINT->ID;
                std::cout << "\t WAITING\t ";
                std::cout << std::endl;
            }
        }
        if( _TP_SEG->Activity == BUSSIM_PAX_NOTHING )
        {
            std::cout << "SEG "<< SEG_CNTR<<": pax does nothing" << std::endl;
        }
        SEG_CNTR++;
        _TP_SEG = _TP_SEG->next;
    }
}

void BUSSIM_TP::insert_before(class BUSSIM_TP_SEG *_RefPtr, class BUSSIM_TP_SEG *_OpPtr)
{
    if( _RefPtr != nullptr )
    {
        if( _RefPtr == this->REF2FIRST_TP_SEG )
        {
            // insert at beginning
            _RefPtr->previous = _OpPtr;
            _OpPtr->next = _RefPtr;
            this->REF2FIRST_TP_SEG = _OpPtr;
            this->SEGMENTS++;
        }
        else
        {
            // insert not as first segment
            _OpPtr->previous = _RefPtr->previous;
            _OpPtr->next = _RefPtr;
            _RefPtr->previous->next = _OpPtr;
            _RefPtr->previous = _OpPtr;
            this->SEGMENTS++;
        }
    }
}

void BUSSIM_TP::insert_as_first_segment(class BUSSIM_TP_SEG *_SEG)
{
    // this procedure inserts the segment at _SEG as the first TP_segment
    // in this travel path

    if( this->REF2FIRST_TP_SEG != nullptr )
    {
        // at least one segment stored so far
        _SEG->previous = nullptr;
        _SEG->next = this->REF2FIRST_TP_SEG;
        _SEG->next->previous = _SEG;
        this->REF2FIRST_TP_SEG = _SEG;
    }
    else
    {
        // there are no segments stored so far
        _SEG->previous = nullptr;
        _SEG->next = nullptr;
        this->REF2FIRST_TP_SEG = _SEG;
        this->REF2LAST_TP_SEG = _SEG;
    }

    // increase the number of stored segments by 1
    this->SEGMENTS++;
}

void BUSSIM_TP::print_on_screen(class BUSSIM_NETWORK *_NET)
{
    class BUSSIM_TP_SEG *_TP_SEG;

    long SEG_CNTR=0;

    _TP_SEG = this->REF2FIRST_TP_SEG;

    std::cout << this->DESC << " (" << this->SEGMENTS << " segments)" << std::endl;
    while( _TP_SEG != nullptr )
    {
        if( _TP_SEG->Activity == BUSSIM_PAX_RIDING )
        {
            if( _TP_SEG->REF2ARC != nullptr )
            {
                glColor3f((GLfloat)0,(GLfloat)1.00,(GLfloat)0.00);
                glLineWidth((GLfloat)5);
                glBegin(GL_LINES);
                    //glLineWidth((GLfloat)30);
                    glVertex2f(_NET->ARC[_TP_SEG->REF2ARC->ID].orig_screen_x,
                               _NET->ARC[_TP_SEG->REF2ARC->ID].orig_screen_y);
                    glVertex2f(_NET->ARC[_TP_SEG->REF2ARC->ID].dest_screen_x,
                               _NET->ARC[_TP_SEG->REF2ARC->ID].dest_screen_y);
                glEnd();
            }
        }
        if( _TP_SEG->Activity == BUSSIM_PAX_WALKING )
        {
            if( _TP_SEG->REF2ARC != nullptr )
            {
                //std::cout << "SEG "<< SEG_CNTR<<": ON ARC: " << _TP_SEG->REF2ARC->ID;
                //std::cout << "\t WALKING\t ";
                //std::cout << _TP_SEG->REF2ARC->orig_node << "->" << _TP_SEG->REF2ARC->dest_node;
                //std::cout << " on line " << _TP_SEG->Line << std::endl;
                glColor3f((GLfloat)1.0,(GLfloat)0.00,(GLfloat)0.00);
                glLineWidth((GLfloat)15);
                glBegin(GL_LINES);
                    //glLineWidth((GLfloat)30);
                    glVertex2f(_NET->ARC[_TP_SEG->REF2ARC->ID].orig_screen_x,
                               _NET->ARC[_TP_SEG->REF2ARC->ID].orig_screen_y);
                    glVertex2f(_NET->ARC[_TP_SEG->REF2ARC->ID].dest_screen_x,
                               _NET->ARC[_TP_SEG->REF2ARC->ID].dest_screen_y);
                glEnd();
            }
        }
        if( _TP_SEG->Activity == BUSSIM_PAX_WAITING )
        {
            if( _TP_SEG->REF2POINT != nullptr )
            {
                //std::cout << "SEG "<< SEG_CNTR<<": AT POINT: " << _TP_SEG->REF2POINT->ID;
                //std::cout << "\t WAITING\t ";
                //std::cout << std::endl;
                glColor3f((GLfloat)0.0,(GLfloat)0.00,(GLfloat)1.00);
                glLineWidth((GLfloat)15);
                glBegin(GL_LINES);
                    //glLineWidth((GLfloat)30);
                    glVertex2f(_NET->PT[_TP_SEG->REF2POINT->ID].screen_x,
                               _NET->PT[_TP_SEG->REF2POINT->ID].screen_y);
                    glVertex2f(_NET->PT[_TP_SEG->REF2POINT->ID].screen_x+0.01,
                               _NET->PT[_TP_SEG->REF2POINT->ID].screen_y+0.01);
                glEnd();
            }
        }
        if( _TP_SEG->Activity == BUSSIM_PAX_NOTHING )
        {
            std::cout << "SEG "<< SEG_CNTR<<": pax does nothing" << std::endl;
        }
        SEG_CNTR++;
        _TP_SEG = _TP_SEG->next;
    }
}

void BUSSIM_PAX::proceed_on_travelpath(class BUSSIM_NETWORK *_NET)
{
    // this procedure sets the Pointer to the next segment in the TravelPath
    // of a passenger

    if( this->Ref2CurrentTPSeg != nullptr )
    {
        // store the segment completion time
        this->Ref2CurrentTPSeg->Leave_Time = _NET->CURRENT_TIME;

        // goto next segment
        this->Ref2CurrentTPSeg = this->Ref2CurrentTPSeg->next;

        // check if there is a next element
        if( this->Ref2CurrentTPSeg != nullptr )
        {
            // store the segment enter time
            this->Ref2CurrentTPSeg->Enter_Time = _NET->CURRENT_TIME;

            // update the passengers current activity attribut
            this->Current_Activity = this->Ref2CurrentTPSeg->Activity;
        }

    }
    else
    {
        // no segment selected so far, check if there is at least one
        if( this->TravelPath->REF2FIRST_TP_SEG != nullptr )
        {
            // there is a first travel path segment, direct to it
            this->Ref2CurrentTPSeg = this->TravelPath->REF2FIRST_TP_SEG;

            // store the segment enter time
            this->Ref2CurrentTPSeg->Enter_Time = _NET->CURRENT_TIME;

            // update the passengers current activity attribut
            this->Current_Activity = this->Ref2CurrentTPSeg->Activity;
        }
    }
}

class BUSSIM_PAX *BUSSIM_PAX_LISTITEM::get_ref2pax(void)
{
    return( this->ref2pax );
}

double BUSSIM_TP_SEG::get_pos(void)
{
    return( this->POS );
}

class BUSSIM_ARC *BUSSIM_TP_SEG::get_ref2arc(void)
{
    return( this->REF2ARC );
}

long BUSSIM_TP_SEG::get_line(void)
{
    return( this->Line );
}

long BUSSIM_TP::get_first_line(void)
{
    return( this->FIRST_LINE );
}

class BUSSIM_PAX_LISTITEM *BUSSIM_PAX_LISTITEM::get_previous(void)
{
    return( this->previous );
}

class BUSSIM_PAX_LISTITEM *BUSSIM_PAX_LISTITEM::get_next(void)
{
    return( this->next );
}

void BUSSIM_PAX::update_pax_pos_walking(class BUSSIM_NETWORK *_NET)
{
    // this procedure updates the this->Ref2CurrentTPSeg->POS-attribute of a pax that is currently
    // walking along a transfer arc in its travel path
    //
    // if, after update, this->Ref2CurrentTPSeg->POS >= 1 then this pax has completed its current segment

    // first, we check, if we have to go to the next segment in the pax's travelpath
    /*
    if( _NET->CURRENT_TIME >= 500.0 )
    {
        std::cout << "We have pax " << this->ID << std::endl;
        if( this->Ref2CurrentTPSeg->REF2ARC != nullptr )
            std::cout << "arc address " << this->Ref2CurrentTPSeg->REF2ARC->ID << std::endl;
        this->TravelPath->print_on_console(_NET);
    }
    */
    if( this->Ref2CurrentTPSeg->get_activity() == BUSSIM_PAX_WAITING )
    {
        //std::cout << "move pax " << this->ID<< " from init wait to init walk" << std::endl;
        this->proceed_on_travelpath(_NET);
        _NET->NUM_PAX_IN_WAITING--;
    }

    // check if it needed to remove this passenger from a walking pax waiting list

    // update POS

    // calculate the additional walking distance during _DeltaTime in kilometers
    double add_distance_km;
    add_distance_km = (this->VELOCITY) / 60.0 * (double)(BUSSIM_TIME_STEP);

    // scale it into the screened coordinate system
    double add_distance_screen;
    add_distance_screen = (add_distance_km - 0) / (_NET->xmax - _NET->xmin) *2.0;

    // determine the start and stop pos of the currently used arc
    double vec_orig_x;
    double vec_orig_y;
    double vec_dest_x;
    double vec_dest_y;

    vec_orig_x = this->Ref2CurrentTPSeg->REF2ARC->orig_screen_x;
    vec_orig_y = this->Ref2CurrentTPSeg->REF2ARC->orig_screen_y;

    vec_dest_x = this->Ref2CurrentTPSeg->REF2ARC->dest_screen_x;
    vec_dest_y = this->Ref2CurrentTPSeg->REF2ARC->dest_screen_y;

    // determine the new pos-vector
    double new_pos_vec_x;
    double new_pos_vec_y;

    new_pos_vec_x = this->Ref2CurrentTPSeg->POS*(vec_dest_x - vec_orig_x) + (vec_dest_x - vec_orig_x) / (double)(this->Ref2CurrentTPSeg->REF2ARC->length_screen)*add_distance_screen;
    new_pos_vec_y = this->Ref2CurrentTPSeg->POS*(vec_dest_y - vec_orig_y) + (vec_dest_y - vec_orig_y) / (double)(this->Ref2CurrentTPSeg->REF2ARC->length_screen)*add_distance_screen;

    // determine length of new pos-vector
    double new_pos_vec_length;
    new_pos_vec_length = sqrt(pow(new_pos_vec_x,2)+pow(new_pos_vec_y,2));

    // update POS
    if( this->Ref2CurrentTPSeg->REF2ARC->length_screen > 0 )
        this->Ref2CurrentTPSeg->POS = new_pos_vec_length / this->Ref2CurrentTPSeg->REF2ARC->length_screen;
    else
        this->Ref2CurrentTPSeg->POS = 1.0;

    //std::cout << "current pos " << this->Ref2CurrentTPSeg->POS << std::endl;
    if( this->Ref2CurrentTPSeg->POS >= 1.0 )
    {
        /*
        if( this->ID == 51 )
        {
            std::cout << "pax 51 has reached its walking destination " << std::endl; //this->Ref2CurrentTPSeg->REF2POINT->ID << std::endl;

        }
        */
        // pax has reached the end of the arc
        // check out passenger from walking list
        _NET->PAX_WALKING->extract_item(this->Ref2WalkingPaxList);


        //std::cout << "GHI" << std::endl;
        delete this->Ref2WalkingPaxList;
        //std::cout << "JKL" << std::endl;
        this->Ref2WalkingPaxList=nullptr;
        //std::cout << "MNO" << std::endl;
        // goto next segment in this passenger's travel path
        //std::cout << "PQR" << std::endl;
        this->proceed_on_travelpath(_NET);

        /*
        if( this->ID == 51 )
        {
            this->TravelPath->print_on_console(_NET);
            std::cout << "pax 51 has reached its walking destination 2" << std::endl; //this->Ref2CurrentTPSeg->REF2POINT->ID << std::endl;
            if( this->Ref2CurrentTPSeg->REF2ARC != nullptr )
            {
                std::cout << "on arc " << this->Ref2CurrentTPSeg->REF2ARC->ID << std::endl;
            }
            if( this->Ref2CurrentTPSeg->REF2POINT != nullptr )
            {
                std::cout << "on platform " << this->Ref2CurrentTPSeg->REF2POINT->ID << std::endl;
            }

            //exit(0);
        }
        */

        // inspect the next travelpath segment
        if( this->Ref2CurrentTPSeg != nullptr )
        {
            // analyse the passenger's next activity
            if( this->Ref2CurrentTPSeg->Activity == BUSSIM_PAX_WAITING )
            {
                // passenger is waiting

                // add the pax item to the station's waitinglist
                _NET->PT[this->Ref2CurrentTPSeg->REF2POINT->ID].WAITINGLIST[this->Ref2CurrentTPSeg->get_next()->get_line()]->append_pax(this,0);
                _NET->NUM_PAX_IN_WAITING++;

            }
        }
        else
        {
            // passenger has reached its final destination

            // release the memory
            delete this->Ref2WalkingPaxList;
            this->Ref2WalkingPaxList = nullptr;

            // move the passenger into the list of arrived passengers
            _NET->PAX_ARRIVED->append_pax(this,0);

            // remove this pax from the PAX_IN-list
            _NET->PAX_IN->extract_item(this->Ref2PaxInList);

            delete this->Ref2PaxInList;
            this->Ref2PaxInList = nullptr;

        }
    }
}

long BUSSIM_TP::get_deboard_node_of_ride(class BUSSIM_TP_SEG *_RefPtr, class BUSSIM_NETWORK *_NET)
{
    // this procedure returns the terminal node of the final arc served by
    // the line found at the end of RefPtr

    long RetVal = -17;

    if( _RefPtr != nullptr )
    {
        if( _RefPtr->get_activity() == BUSSIM_PAX_RIDING )
        {
            //std::cout << "ok, we have a RIDING TP-part to be started" << std::endl;
            long CurrentLine;
            CurrentLine = _RefPtr->get_line();

            //std::cout << "ok, the next line to take is " << CurrentLine << std::endl;

            if( CurrentLine >= 0 )
            {
                //std::cout << "ok, we have a real line" << std::endl;

                int TOBESTOPPED = -1;
                class BUSSIM_TP_SEG *TpPtr;
                TpPtr = _RefPtr;

                while( TOBESTOPPED < 0 )
                {
                    if( TpPtr->get_next() != nullptr )
                    {
                        // there is a next activity ...
                        if( TpPtr->get_next()->get_activity() == BUSSIM_PAX_RIDING )
                        {
                            // ... and it is RIDING
                            //std::cout << "arc" << TpPtr->get_next()->get_ref2arc()->ID << std::endl;
                            TpPtr = TpPtr->get_next();
                        }
                        else
                        {
                            // ... but it is not RIDING
                            TOBESTOPPED = 1;
                        }
                    }
                    else
                    {
                        // no next activity
                        TOBESTOPPED = 1;
                    }
                }

                if( TpPtr != nullptr )
                    RetVal = _NET->PT[TpPtr->get_ref2arc()->dest_node].ID;
            }
        }
    }

    //std::cout << "DONE" << std::endl;
    return(RetVal);
}

long BUSSIM_TP_SEG::get_activity(void)
{
    return(this->Activity);
}

class BUSSIM_POINT *BUSSIM_TP_SEG::get_ref2point(void)
{
    return(this->REF2POINT);
}

class BUSSIM_TP_SEG *BUSSIM_TP_SEG::get_next(void)
{
    return(this->next);
}
class BUSSIM_TP_SEG *BUSSIM_TP_SEG::get_previous(void)
{
    return(this->previous);
}

class BUSSIM_VEHICLE *BUSSIM_TP_SEG::get_ref2vehicle(void)
{
    return(this->REF2VEHICLE);
}

void BUSSIM_TP_SEG::set_ref2vehicle(class BUSSIM_VEHICLE *_PtrVeh)
{
    this->REF2VEHICLE = _PtrVeh;
}

void BUSSIM_TP::dissolve_segment(class BUSSIM_TP_SEG *_OpPtr)
{
    if( _OpPtr != nullptr )
    {
        if( this->REF2FIRST_TP_SEG != nullptr )
        {
            if( this->REF2FIRST_TP_SEG != this->REF2LAST_TP_SEG )
            {
                // at least two segments stored
                if( _OpPtr == this->REF2FIRST_TP_SEG )
                {
                    // we want to remove the first stored segment
                    this->REF2FIRST_TP_SEG = this->REF2FIRST_TP_SEG->next;
                    _OpPtr->next->previous = nullptr;
                    _OpPtr->next = nullptr;
                    _OpPtr->previous = nullptr;
                }
                else
                {
                    if( _OpPtr == this->REF2LAST_TP_SEG )
                    {
                        // we want to remove the last stored segment
                        this->REF2LAST_TP_SEG = this->REF2LAST_TP_SEG->previous;
                        _OpPtr->previous->next = nullptr;
                        _OpPtr->next = nullptr;
                        _OpPtr->previous = nullptr;
                    }
                    else
                    {
                        // we want to remove a segment in the middle
                        _OpPtr->previous->next = _OpPtr->next;
                        _OpPtr->next->previous = _OpPtr->previous;
                        _OpPtr->previous = nullptr;
                        _OpPtr->next = nullptr;
                    }
                }
            }
            else
            {
                // there is only one segment stored
                this->REF2FIRST_TP_SEG = nullptr;
                this->REF2LAST_TP_SEG = nullptr;
            }
            this->SEGMENTS--;
        }
        else
        {
            // path is empty => nothing to do here
        }
    }
}
