#ifndef BUSSIM_POINT_H_INCLUDED
#define BUSSIM_POINT_H_INCLUDED

#include "bussim_global.h"
#include <string>

class BUSSIM_POINT {
public:
    int ID;                 // unique identifier of this point

    std::string SHORTNAME;     // added in version 3_00 to host official short name of stop

    double world_longitude; // added in version 1_05 to host the geographic x-coordinate
    double world_latitude;  // added in version 1_05 to host the geographic y-coordinate

    double world_x;         // the x-coordinate of this point in the real world
    double world_y;         // the y-ccordinate of this point in the real world

    double screen_x;        // the x-coordinate of this point on the screen
    double screen_y;        // the y-ccordinate of this point on the screen

    int label_pos;          // direction of label position from center of point
    double label_x_offset;  // horizontal distance of label from center of point
    double label_y_offset;  // vertical distance of label from center of point

    int type;               // regular or service stop

    int INFRASTRUCTURE;     // V3.00 if this flag is set (=1) then no vehicle will wait here
                            //          except if the next arc is occupied due to block conditions

    int INFRATYPE;          // V3.00 defines type of infrastructure point: switch or track position point

    std::string STOPNAME;        // V3.00 mainly used to label a vehicle with its current destination

    double default_waiting_time;    // V3.00 default waiting time at this stop

    int IS_POSITIONED;      // V3.00 to check if label position is already fixed;

    BUSSIM_POINT();         // this is the default constructor
    BUSSIM_POINT(double _X, double _Y);     // this is an overloaded constructor to directly set the coordinates
    ~BUSSIM_POINT();       // this is the default constructor

    class BUSSIM_STOP *ref2stop;    // V3.00 refers to a stop-object

    int ASSIGNED_OUTBOUND_ARCS;
    struct BUSSIM_ASSIGNED_ARC *first_outbound_arc;
    struct BUSSIM_ASSIGNED_ARC *last_outbound_arc;

    class BUSSIM_PAX_LIST **WAITINGLIST;    // used to host waiting passengers
    // WAITINGLIST[0],...WAITINGLIST[EXT_LINES-1] for pax waiting for a line
    // WAITINGLIST[EXT_LINES] for pax who start with a transfer after

    long *PlatformPosInLine;        // used to determine the position of this platform along a line

    // this procedure calculates the coordinates of this point on the screen
    void calculate_screen_coordinates(class BUSSIM_NETWORK *_NET, double xmin, double xmax, double ymin, double ymax);

    // this procedure prints this point on the screen
    void print_on_screen(void);
    // this procedures prints the label of this point on the screen
    void print_on_screen_small(class BUSSIM_NETWORK *_NET);
    // this procedures also prints the label of this point on the screen - but in another kind
    void print_label_on_screen(class BUSSIM_NETWORK *_NET);
    // this procedures also prints the short label of this point on the screen - but in another kind
    void print_label_on_screen_short(class BUSSIM_NETWORK *_NET);
    // this procedure declares a point to an infrastructure point
    void set_as_infrastructure_point(void);
    // this procedure declares a point to a non-infrastructure point
    void unset_as_infrastructure_point(void);
    // this procedure specifies a regular stop node/ from V3_00
    void specify_regular(double _LONG, double _LAT, int _STOP_ID, class BUSSIM_NETWORK *_NET);
    // this procedure specifies an infrsstructure node / from V3_00
    //void specify_infra(double _LONG, double _LAT, std::string _STOPNAME, std::string _SHORTNAME);
    // this procedure specifies a switch node / from V3_00
    void specify_switch(double _LONG, double _LAT, int _STOP_ID, std::string _STOPNAME, std::string _SHORTNAME, class BUSSIM_NETWORK *_NET);
    // this procedure specifies a track position node / from V3_00
    void specify_trackpos(double _LONG, double _LAT, int _STOP_ID, std::string _STOPNAME, std::string _SHORTNAME, class BUSSIM_NETWORK *_NET);
    // this procedure specifies a depot node / from V3_00
    void specify_depot(double _LONG, double _LAT, int _STOP_ID, class BUSSIM_NETWORK *_NET);
    // this procedure specifies a siding position / from V3.00
    void specify_siding(double _LONG, double _LAT, int _STOP_ID, class BUSSIM_NETWORK *_NET);
    // procedure to determine point label position (added in V3.00)
    void set_label_pos_auto(class BUSSIM_NETWORK *_NET);
    // procedure to print waiting list on screen
    void print_waiting_lists(double _XPOS, double _YPOS, class BUSSIM_NETWORK *_NET);
    // procedure adds ref to pax into the adequate waiting list initially
    void init_add_to_waiting_list(class BUSSIM_PAX *_REF2PAX, class BUSSIM_NETWORK *_NET);
};

#endif // BUSSIM_POINT_H_INCLUDED
