//#include "bussim_schedule.h"
#include "bussim_global.h"
#include <stdio.h>
#include <iostream>

BUSSIM_SEGMENT::BUSSIM_SEGMENT()
{
    // this is the standard constructor

    this->ID = -1;

    this->pos_in_schedule = -1;

    this->ref2arc = nullptr;

    this->time_in = -1.0;
    this->time_out = -1.0;

    this->ref2duty = nullptr;

    this->park_pos_on_arc = -1.0;

    this->on_line = -1;
    this->dest_desc = "not yet defined";

    this->ref2schedule = nullptr;

    this->prev = nullptr;
    this->next = nullptr;

    this->ref2DeBoardPaxList = nullptr;
    this->DeboardAtEnd = 0;
}

BUSSIM_SEGMENT::BUSSIM_SEGMENT(int ID, class BUSSIM_ARC *ref2arc,
                   double time_in,
                   double time_out,
                   class BUSSIM_DUTY *ref2duty,
                   int on_line,
                   std::string dest_desc,
                   class BUSSIM_SCHEDULE *ref2schedule,
                   class BUSSIM_SEGMENT *prev,
                   class BUSSIM_SEGMENT *next,
                   class BUSSIM_NETWORK *_NET,
                   int _INC)
{
    // this is the standard constructor

    this->ID = ID;

    this->ref2arc = ref2arc;

    this->time_in = time_in;
    this->time_out = time_out;

    this->ref2duty = ref2duty;

    this->on_line = on_line;
    this->dest_desc = dest_desc;

    this->park_pos_on_arc = 1.0;

    this->ref2schedule = ref2schedule;

    this->pos_in_schedule = -1;

    this->prev = prev;
    this->next = next;

    this->ref2DeBoardPaxList = nullptr;

    this->PosDeboardPaxList = -1;

    if( _NET->PT[this->ref2arc->dest_node].INFRASTRUCTURE < 1 )
    {
        this->DeboardAtEnd = 1;
    }
    else
    {
        this->DeboardAtEnd = 0;
    }

}

BUSSIM_SEGMENT::~BUSSIM_SEGMENT()
{
    // this is the standard destructor
}

BUSSIM_SCHEDULE::BUSSIM_SCHEDULE()
{
    // the standard constructor
    this->ID = -1;

    this->segments = 0;

    this->ref2veh = nullptr;

    this->first_segment = nullptr;
    this->last_segment = nullptr;
    this->current_segment = nullptr;
}

BUSSIM_SCHEDULE::BUSSIM_SCHEDULE(int ID, class BUSSIM_VEHICLE *ref2veh)
{
    // an overloaded constructor
    this->ID = ID;

    this->segments = 0;

    this->ref2veh = ref2veh;

    this->first_segment = nullptr;
    this->last_segment = nullptr;
    this->current_segment = nullptr;
}

BUSSIM_SCHEDULE::~BUSSIM_SCHEDULE()
{
    // the standard destructor
}

void BUSSIM_SCHEDULE::append_segment(class BUSSIM_SEGMENT *Ptr, class BUSSIM_NETWORK *_NET)
{
    // used to append the segment at Ptr to the existing list of segments

    Ptr->pos_in_schedule = this->segments;
    if( this->first_segment != nullptr )
    {
        // there is still at least one segment
        Ptr->ID = _NET->SEGMENT_CNTR;
        Ptr->prev = this->last_segment;
        Ptr->next = nullptr;
        Ptr->ref2schedule = this;
        Ptr->prev = this->last_segment;
        this->last_segment->next = Ptr;
        this->last_segment = Ptr;
    }
    else
    {
        // no segment saved so far
        Ptr->ID = _NET->SEGMENT_CNTR;
        Ptr->prev = nullptr;
        Ptr->next = nullptr;
        Ptr->ref2schedule = this;
        this->first_segment = Ptr;
        this->last_segment = Ptr;
        this->current_segment = nullptr;    // it is set if the vehicle is actived
    }

    Ptr = nullptr;

    // adjust the counter
    this->segments++;
    _NET->SEGMENT_CNTR++;
}

void BUSSIM_SCHEDULE::print_on_shell(class BUSSIM_NETWORK *_NET)
{
    class BUSSIM_SEGMENT *Ptr2Seg;

    Ptr2Seg = this->first_segment;
    while( Ptr2Seg != nullptr )
    {
        if( Ptr2Seg->ref2duty != nullptr )
            std::cout << "vehicle " << this->ref2veh->ID << ": "
                << "segment-ID=" << Ptr2Seg->ID
                << " servicing line " << Ptr2Seg->on_line
                << " towards " << Ptr2Seg->dest_desc
                << " on arc " << Ptr2Seg->ref2arc->ID
                << " from " << _NET->PT[Ptr2Seg->ref2arc->orig_node].STOPNAME
                << " (" <<  _NET->PT[Ptr2Seg->ref2arc->orig_node].ID
                << ") -> " << _NET->PT[Ptr2Seg->ref2arc->dest_node].STOPNAME
                << " (" <<  _NET->PT[Ptr2Seg->ref2arc->dest_node].ID << ") "
                << " in: " << Ptr2Seg->time_in
                << " out: " << Ptr2Seg->time_out
                << " park pos: " << Ptr2Seg->park_pos_on_arc
                << " duty-ID " << Ptr2Seg->ref2duty->ID
                << " DeboardAtEnd " << Ptr2Seg->DeboardAtEnd
                << " PlatformPosNum " << Ptr2Seg->PosDeboardPaxList << std::endl;
        else
                std::cout << "vehicle " << this->ref2veh->ID << ": "
                << "segment-ID=" << Ptr2Seg->ID
                << " servicing line " << Ptr2Seg->on_line
                << " towards " << Ptr2Seg->dest_desc
                << " on arc " << Ptr2Seg->ref2arc->ID
                << " from " << _NET->PT[Ptr2Seg->ref2arc->orig_node].STOPNAME
                << " (" <<  _NET->PT[Ptr2Seg->ref2arc->orig_node].ID
                << ") -> " << _NET->PT[Ptr2Seg->ref2arc->dest_node].STOPNAME
                << " (" <<  _NET->PT[Ptr2Seg->ref2arc->dest_node].ID << ") "
                << " in: " << Ptr2Seg->time_in
                << " out: " << Ptr2Seg->time_out
                << " park pos: " << Ptr2Seg->park_pos_on_arc
                << " duty-ID " << -1
                << " DeboardAtEnd " << Ptr2Seg->DeboardAtEnd
                << " PlatformPosNum " << Ptr2Seg->PosDeboardPaxList << std::endl;

        Ptr2Seg = Ptr2Seg->next;
    }
}

void BUSSIM_SCHEDULE::clean(class BUSSIM_NETWORK *_NET)
{
    class BUSSIM_SEGMENT *Ptr2Seg;

    Ptr2Seg = this->last_segment;
    while( Ptr2Seg != nullptr )
    {
        // schedule is non-empty
        Ptr2Seg = this->last_segment;
        if( Ptr2Seg == this->first_segment)
        {
            // we have reached the last segment
            delete Ptr2Seg;
            this->first_segment = nullptr;
            this->last_segment = nullptr;
            Ptr2Seg = nullptr;
            this->segments--;
        }
        else
        {
            // more than one segment left
            this->last_segment = Ptr2Seg->prev;
            Ptr2Seg->prev->next = nullptr;
            delete Ptr2Seg;
            this->segments--;
        }
    }
}

void BUSSIM_SCHEDULE::print_on_shell_remaining_duty(class BUSSIM_NETWORK *_NET)
{
    class BUSSIM_SEGMENT *Ptr2Seg;

    Ptr2Seg = this->current_segment;
    long CurrentDuty;
    CurrentDuty = this->current_segment->ref2duty->ID;
    std::cout << "remaining schedule of vehicle " << this->ID << " in duty " << CurrentDuty << std::endl;
    while( (Ptr2Seg != nullptr) )
    {
        if( Ptr2Seg->ref2duty->ID == CurrentDuty )
        {


        if( Ptr2Seg->ref2duty != nullptr )
            std::cout << "vehicle " << this->ref2veh->ID << ": "
                << "segment-ID=" << Ptr2Seg->ID
                << " servicing line " << Ptr2Seg->on_line
                << " towards " << Ptr2Seg->dest_desc
                << " on arc " << Ptr2Seg->ref2arc->ID
                << " from " << _NET->PT[Ptr2Seg->ref2arc->orig_node].STOPNAME
                << " (" <<  _NET->PT[Ptr2Seg->ref2arc->orig_node].ID
                << ") -> " << _NET->PT[Ptr2Seg->ref2arc->dest_node].STOPNAME
                << " (" <<  _NET->PT[Ptr2Seg->ref2arc->dest_node].ID << ") "
                << " in: " << Ptr2Seg->time_in
                << " out: " << Ptr2Seg->time_out
                << " park pos: " << Ptr2Seg->park_pos_on_arc
                << " duty-ID " << Ptr2Seg->ref2duty->ID
                << " DeboardAtEnd " << Ptr2Seg->DeboardAtEnd
                << " PlatformPosNum " << Ptr2Seg->PosDeboardPaxList << std::endl;
        else
                std::cout << "vehicle " << this->ref2veh->ID << ": "
                << "segment-ID=" << Ptr2Seg->ID
                << " servicing line " << Ptr2Seg->on_line
                << " towards " << Ptr2Seg->dest_desc
                << " on arc " << Ptr2Seg->ref2arc->ID
                << " from " << _NET->PT[Ptr2Seg->ref2arc->orig_node].STOPNAME
                << " (" <<  _NET->PT[Ptr2Seg->ref2arc->orig_node].ID
                << ") -> " << _NET->PT[Ptr2Seg->ref2arc->dest_node].STOPNAME
                << " (" <<  _NET->PT[Ptr2Seg->ref2arc->dest_node].ID << ") "
                << " in: " << Ptr2Seg->time_in
                << " out: " << Ptr2Seg->time_out
                << " park pos: " << Ptr2Seg->park_pos_on_arc
                << " duty-ID " << -1
                << " DeboardAtEnd " << Ptr2Seg->DeboardAtEnd
                << " PlatformPosNum " << Ptr2Seg->PosDeboardPaxList << std::endl;

        Ptr2Seg = Ptr2Seg->next;
        }
        else
        {
            Ptr2Seg = nullptr;
        }
    }
}
