//#include "bussim_vehicle.h"
#include "bussim_global.h"
//#include "bussim_bunchpoint.h"
//#include "bussim_rotation.h"
//#include "bussim_schedule.h"

#include <math.h>
#include <iostream>
#include <stdio.h>

//#include <GL/gl.h>
//#include <freeglut/freeglut.h>

extern int AUTOMATIC_MODE;
extern int DISPLAY_VEHICLE_LABELS;

BUSSIM_VEHICLE::BUSSIM_VEHICLE()
{
    this->ID = -1;
    this->on_line = -1;
    this->velocity = -1.0;
    this->ACTIVE = 0;
    this->WAITING = 0;
    this->VISIBLE = 1;

    this->WAIT_FOR_BLOCK = 0;
    this->IS_BLOCKED = 0;

    this->last_update_time = -1.0;
    this->activation_time = -1.0;

    this->ON_ARC.dest_node = -1;
    this->ON_ARC.orig_node = -1;
    this->ON_ARC.dest_screen_x = -1.0;
    this->ON_ARC.dest_screen_y = -1.0;
    this->ON_ARC.length_screen = -1.0;
    this->ON_ARC.orig_screen_x = -1.0;
    this->ON_ARC.orig_screen_y = -1.0;

    this->POS = -1.0;

    this->park_arc.dest_node = -1;
    this->park_arc.orig_node = -1;
    this->park_arc.dest_screen_x = -1.0;
    this->park_arc.dest_screen_y = -1.0;
    this->park_arc.length_screen = -1.0;
    this->park_arc.orig_screen_x = -1.0;
    this->park_arc.orig_screen_y = -1.0;

    this->park_pos = -1.0;

    this->remaing_waiting_time = -1.0;

    this->km_travelled_on_line = 0;
    this->start_time_on_line = -1.0;

    this->km_travelled_on_arc = 0;
    this->start_time_on_arc = -1.0;

    this->Thwarted = 0;

    this->rotation = nullptr;

    this->next_arc.ID = -1;     // to be set in the activate function
    this->last_visited_node = -1;     // to be set in the activate function
    this->last_node_arrival_time = -1.0;    // to be set in the activate function

    this->SCHEDULE = new class BUSSIM_SCHEDULE(this->ID,this);

    this->DeBoardPaxList = nullptr;
    this->PAX_LIST_NUM = 0;

    this->PAX_ON_BOARD = 0;

    this->capacity = BUSSIM_BIG_M;

    this->vehicle_category = BUSSIM_VEHCAT_TRAM;
}

void BUSSIM_VEHICLE::configure(int _ID, int _VEH_CAT, double _VELOCITY, int _CAPACITY, class BUSSIM_ARC _ON_ARC, class BUSSIM_ARC _PARK_ARC, class BUSSIM_NETWORK *_NET)
{
    this->ID = _ID;
    this->on_line = -1;
    this->velocity = _VELOCITY;
    this->ACTIVE = 0;
    this->WAITING = 0;
    this->VISIBLE = 1;

    this->WAIT_FOR_BLOCK = 0;
    this->IS_BLOCKED = 0;

    this->last_update_time = -1.0;
    this->activation_time = -1.0;

    /*
    this->ON_ARC.dest_node = -1;
    this->ON_ARC.orig_node = -1;
    this->ON_ARC.dest_screen_x = -1.0;
    this->ON_ARC.dest_screen_y = -1.0;
    this->ON_ARC.length_screen = -1.0;
    this->ON_ARC.orig_screen_x = -1.0;
    this->ON_ARC.orig_screen_y = -1.0;
    */

    this->ON_ARC = _ON_ARC;

    this->capacity = _CAPACITY;

    this->vehicle_category = _VEH_CAT;

    // sort this vehicle in the queue assoc with this arc
    _NET->ARC[this->ON_ARC.ID].append_arrived_vehicle(this->ID);
    this->POS = 0.0;  // 1.0
    /*
    this->POS = -1.0;
    */
    //this->POS = _POS;

    this->park_arc = _PARK_ARC;
    this->park_pos = -1.0;
    this->remaing_waiting_time = -1.0;

    this->km_travelled_on_line = 0;
    this->start_time_on_line = -1.0;

    this->km_travelled_on_arc = 0;
    this->start_time_on_arc = -1.0;

    this->Thwarted = 0;

    this->rotation = new struct BUSSIM_ROTATION(_NET->ROTATIONS);
    _NET->ROTATIONS++;;

    this->next_arc.ID = -1;     // to be set in the activate function
    this->last_visited_node = -1;     // to be set in the activate function

    this->SCHEDULE = new class BUSSIM_SCHEDULE(this->ID,this);

    this->DeBoardPaxList = nullptr;
    this->PAX_LIST_NUM = 0;
}

BUSSIM_VEHICLE::BUSSIM_VEHICLE(int _ID, int _ACTIVE, int _WAITING, double _VELOCITY, double _LAST_UPDATE_TIME, double _POS, double _REMAINING_WAITING_TIME, class BUSSIM_ARC _ON_ARC)
{
    this->ID = _ID;
    this->on_line = -1;
    this->velocity = _VELOCITY;
    this->ACTIVE = _ACTIVE;
    this->WAITING = _WAITING;
    this->VISIBLE = 1;

    this->last_update_time = _LAST_UPDATE_TIME;

    this->ON_ARC = _ON_ARC;

    this->POS = _POS;
    this->remaing_waiting_time = _REMAINING_WAITING_TIME;

    this->km_travelled_on_line = 0;
    this->start_time_on_line = -1.0;

    this->km_travelled_on_arc = 0;
    this->start_time_on_arc = -1.0;

    this->Thwarted = 0;
    this->IS_BLOCKED = 0;

    this->rotation = nullptr;

    this->next_arc.ID = -1;     // to be set in the activate function
    this->last_visited_node = -1;     // to be set in the activate function

    this->activation_time = 0.0;

    this->DeBoardPaxList = nullptr;

    this->PAX_ON_BOARD = 0;
}

BUSSIM_VEHICLE::~BUSSIM_VEHICLE()
{

}

void BUSSIM_VEHICLE::update(class BUSSIM_NETWORK *_NETWORK)
{
    int RES;

    //std::cout << "POS" << this->POS << std::endl;
    if( (_NETWORK->CURRENT_TIME > 0) && (BUSSIM_TIME_STEP > -5 ) )
    {

    if( this->ACTIVE > 0 )
    { // CASE A START
        double add_distance_screen=0.0;;
        // proceeds only for an active vehicle

        if( this->WAITING > 0 )
        { // CASE B - START
            this->remaing_waiting_time -= (BUSSIM_TIME_STEP);

            if( this->remaing_waiting_time <= 0)
            { // CASE C - START

                // vehicle is ready to go
                this->remaing_waiting_time = 0.0;
                this->WAITING = 0;

                // this is the next served arc
                if( this->ON_ARC.pos < _NETWORK->LINE[this->on_line].ARCS - 1)
                { // case D - START

                    // end of line has not yet reached: goto next arc

                    // first check, if the block logic allows the transfer to the next arc
                    if( _NETWORK->ARC[this->ON_ARC.ID].get_allowance(&(_NETWORK->ARC[_NETWORK->LINE[this->on_line].USED_ARC[this->ON_ARC.pos+1].ID])) > 0 )
                    { // case E - START

                        // the next block allows this vehicle to proceed

                        // check if this vehicle is the next in the queue
                        RES = _NETWORK->ARC[this->ON_ARC.ID].remove_vehicle_from_queue(this->ID,_NETWORK);

                        if( RES >= 0 )
                        { // case F - START

                            // this vehicle can be moved forward

                            // set this vehicle to be unblocked
                            this->IS_BLOCKED = 0;

                            // remove the vehicle from the blocks of the current arc
                            _NETWORK->ARC[this->ON_ARC.ID].check_out_vehicle();

                            // shift vehicle to the next arc
                            this->ON_ARC = _NETWORK->LINE[this->on_line].USED_ARC[this->ON_ARC.pos+1];

                            // check in the vehicle in all blocks assoc with the next arc
                            _NETWORK->ARC[this->ON_ARC.ID].check_in_vehicle();

                            // reset the wait for block flag of this vehicle
                            this->WAIT_FOR_BLOCK = 0;

                            this->km_travelled_on_arc = 0.0;
                            this->start_time_on_arc = _NETWORK->CURRENT_TIME;

                            // sort this vehicle in the queue assoc with this arc
                            _NETWORK->ARC[this->ON_ARC.ID].append_arrived_vehicle(this->ID);
                            this->POS = 0.0;
                        } // case F - END
                        else
                        { // case G - START

                            char TMP[256];
                            sprintf(TMP,"vehicle\t%d\t on line\t %d\t is b-blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NETWORK->ARC[this->ON_ARC.ID].first_waiting_vehicle->veh_id,_NETWORK->VEHICLE[_NETWORK->ARC[this->ON_ARC.ID].first_waiting_vehicle->veh_id].on_line,this->ON_ARC.ID);
                            class BUSSIM_EVENT *NewEvent;
                            NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                            _NETWORK->append_event(NewEvent);
                            NewEvent = nullptr;
                            if( this->IS_BLOCKED < 1 )
                                _NETWORK->DETECTED_BUNCHES++;

                            if( this->IS_BLOCKED < 1 )
                                this->Thwarted++;
                            this->IS_BLOCKED = 1;

                        } // case G - END
                    } // case E - END
                    else
                    { // case H - START
                        // the next arc is blocked and this vehicle cannot proceed
                        char TMPA[256];
                        sprintf(TMPA,"vehicle\t%d\t on line\t %d\t is waiting for block to enter arc\t%d",
                                        this->ID,
                                        this->on_line,
                                        _NETWORK->ARC[_NETWORK->LINE[this->on_line].USED_ARC[this->ON_ARC.pos+1].ID].ID);
                        class BUSSIM_EVENT *NewEvent;
                        NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_WAIT_FOR_BLOCK,TMPA);
                        _NETWORK->append_event(NewEvent);
                        NewEvent = nullptr;
                        if( this->WAIT_FOR_BLOCK < 1 )
                            _NETWORK->BLOCK_WAIT++;
                        this->WAIT_FOR_BLOCK = 1;

                    } // case H - END
                } // case D - ENDE
                else
                { // case I - START
                    // the end of a line has reached: goto to subsequent line to be served
                    this->rotation->current_duty_num++;
                    if( _NETWORK->BETRIEBS_ENDE < 1)
                    { // case J - START

                        if( this->rotation->current_duty->next == nullptr )
                        { // case L - START
                            // no next line found de-active vehicle
                            char TMP[256];
                            sprintf(TMP,"vehicle\t%d\t no new task\t at end of line %d\t deactivated at stop %d",this->ID,this->on_line,this->ON_ARC.dest_node);
                            class BUSSIM_EVENT *NewEvent;
                            NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_VEHICLE_DEACTIVATION,TMP);
                            _NETWORK->append_event(NewEvent);
                            NewEvent = nullptr;

                            // remove vehicle from queue
                            RES = _NETWORK->ARC[this->ON_ARC.ID].remove_vehicle_from_queue(this->ID,_NETWORK);

                            // disable vehicle
                            this->ACTIVE = 0;
                        } // case L - END
                        else
                        { // case M - START
                            // go to next duty in this rotation
                            this->rotation->current_duty = this->rotation->current_duty->next;

                            // remove this vehicle from the arc to be left

                            // first check if the next arc is allowed to be entered
                            if( _NETWORK->ARC[this->ON_ARC.ID].get_allowance(&(_NETWORK->ARC[_NETWORK->LINE[this->rotation->current_duty->TRIP_ID].USED_ARC[0].ID]) ) > 0)
                            { // case N - START

                                // no block constraint prevents the progress of the vehicle
                                RES = _NETWORK->ARC[this->ON_ARC.ID].remove_vehicle_from_queue(this->ID,_NETWORK);
                                if( RES >= 0)
                                { // case P - START
                                    // this vehicle is the next in the queue

                                    this->IS_BLOCKED = 0;

                                    // remove the vehicle from the blocks of the current arc
                                    _NETWORK->ARC[this->ON_ARC.ID].check_out_vehicle();

                                    // shift vehicle to the next arc
                                    this->ON_ARC = _NETWORK->LINE[this->rotation->current_duty->TRIP_ID].USED_ARC[0];

                                    // check-in the vehicle to the blocks of the new arc
                                    _NETWORK->ARC[this->ON_ARC.ID].check_in_vehicle();

                                    // reset the wait for block attribute of this vehicle
                                    this->WAIT_FOR_BLOCK = 0;

                                    // sort this vehicle in the queue assoc with this arc
                                    _NETWORK->ARC[this->ON_ARC.ID].append_arrived_vehicle(this->ID);

                                    this->km_travelled_on_arc = 0.0;
                                    this->start_time_on_arc = _NETWORK->CURRENT_TIME;

                                    //this->on_line = _NETWORK->LINE[this->on_line].ID_NEXT_LINE;
                                    this->on_line = this->rotation->current_duty->TRIP_ID;
                                    this->km_travelled_on_line = 0;
                                    this->start_time_on_line = _NETWORK->CURRENT_TIME;
                                    this->POS = 0.0;
                                } // case P - END
                                else
                                { // case Q - START
                                    char TMP[256];
                                    sprintf(TMP,"vehicle\t%d\t on line\t %d\t is a-blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NETWORK->ARC[this->ON_ARC.ID].first_waiting_vehicle->veh_id,_NETWORK->VEHICLE[_NETWORK->ARC[this->ON_ARC.ID].first_waiting_vehicle->veh_id].on_line,this->ON_ARC.ID);
                                    class BUSSIM_EVENT *NewEvent;
                                    NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                                    _NETWORK->append_event(NewEvent);
                                    NewEvent = nullptr;
                                    if( this->IS_BLOCKED < 1 )
                                        _NETWORK->DETECTED_BUNCHES++;

                                    if( IS_BLOCKED < 1 )
                                        this->Thwarted++;
                                    this->IS_BLOCKED = 1;

                                } // case Q - END
                            } // case N - END
                            else
                            { // case O - START
                                // the next arc is blocked and this vehicle cannot proceed
                                char TMPA[256];
                                sprintf(TMPA,"vehicle\t%d\t on line\t %d\t is waiting for block to enter arc\t%d",
                                        this->ID,
                                        this->on_line,
                                        _NETWORK->ARC[_NETWORK->LINE[this->on_line].USED_ARC[this->ON_ARC.pos+1].ID].ID);
                                class BUSSIM_EVENT *NewEvent;
                                NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_WAIT_FOR_BLOCK,TMPA);
                                _NETWORK->append_event(NewEvent);
                                NewEvent = nullptr;
                                //_NETWORK->DETECTED_BUNCHES++;
                                //this->Thwarted++;
                                if( this->WAIT_FOR_BLOCK < 1 )
                                _NETWORK->BLOCK_WAIT++;
                                this->WAIT_FOR_BLOCK = 1;
                            } // case O - END
                        } // case M - END
                    } // case J - END
                    else
                    { // case K - START
                        // de-active vehicle due to betriebsende

                        char TMP[256];
                        sprintf(TMP,"vehicle\t%d\t end of work\t end of line %d\t deactivated at stop %d",this->ID,this->on_line,this->ON_ARC.dest_node);
                        class BUSSIM_EVENT *NewEvent;
                        NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_VEHICLE_END_OF_WORK,TMP);
                        _NETWORK->append_event(NewEvent);
                        NewEvent = nullptr;

                        this->ACTIVE = 0;
                    } // case K - END
                } // case I - END

                // vehicle is at the beginning of the next arc

                char TMP[256];
                sprintf(TMP,"vehicle\t%d\t line\t%d\t stop\t%d",this->ID,this->on_line,this->ON_ARC.orig_node);
                class BUSSIM_EVENT *NewEvent;
                NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_DEPARTURE,TMP);

                _NETWORK->append_event(NewEvent);
                NewEvent = nullptr;

            } // CASE C - START
        } // CASE B - END

        if( this->WAITING < 1)
        { // CASE R - START
            // vehicle is moving: update it's position

            // calculate the additional driving distance during _DeltaTime in kilometers
            double add_distance_km;
            //double add_distance_pos;

            add_distance_km = (this->velocity) / 60.0 * (double)(BUSSIM_TIME_STEP);

            // scale it into the screened coordinate system
            add_distance_screen = (add_distance_km - 0) / (_NETWORK->xmax - _NETWORK->xmin) *2.0;

            // determine the new pos-vector
            double new_pos_vec_x;
            double new_pos_vec_y;

            new_pos_vec_x = this->POS*(this->ON_ARC.dest_screen_x - this->ON_ARC.orig_screen_x) + (this->ON_ARC.dest_screen_x - this->ON_ARC.orig_screen_x) / (double)(this->ON_ARC.length_screen)*add_distance_screen;
            new_pos_vec_y = this->POS*(this->ON_ARC.dest_screen_y - this->ON_ARC.orig_screen_y) + (this->ON_ARC.dest_screen_y - this->ON_ARC.orig_screen_y) / (double)(this->ON_ARC.length_screen)*add_distance_screen;

            // determine length of new pos-vector
            double new_pos_vec_length;
            new_pos_vec_length = sqrt(pow(new_pos_vec_x,2)+pow(new_pos_vec_y,2));


            // now check if vehicle has reached destination of ON_ARC
            if( (new_pos_vec_length / this->ON_ARC.length_screen) < 1 )
            { // case S start
                // not yet

                // check if repos is possible without overtaking
                int overtake=0;
                //overtake = this->check_for_takeover_4(_NETWORK,&add_distance_km);

                if( overtake == 0 )
                {
                    // repositioning possible
                    this->POS = (new_pos_vec_length / this->ON_ARC.length_screen);
                    // save the travel distance increase of this vehicle
                    this->km_travelled_on_line += add_distance_km;

                    // reset the on_arc-properties
                    this->km_travelled_on_arc += add_distance_km;
                }
                else
                {
                    // vehicle cannot move forward

                    class BUSSIM_BUNCHPOINT *A;
                    A = new class BUSSIM_BUNCHPOINT;
                    A->screen_x = new_pos_vec_x + this->ON_ARC.orig_screen_x;
                    A->screen_y = new_pos_vec_y + this->ON_ARC.orig_screen_y;

                    _NETWORK->append_bunch_point(A);
                    delete A;

                }
            } // case S end
            else
            { // case T start
                // has reached end on arc

                // update the last stop information
                this->next_arc = _NETWORK->ARC[this->ON_ARC.dest_node];
                this->last_visited_node = this->ON_ARC.dest_node;
                this->last_node_arrival_time = _NETWORK->CURRENT_TIME + (double)(this->ID)/10000.0;;

                // generate the arrival event
                char TMP[256];
                sprintf(TMP,"vehicle\t%d\t line\t%d\t stop\t%d",this->ID,this->on_line,this->ON_ARC.dest_node);
                class BUSSIM_EVENT *NewEvent;
                NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_ARRIVAL,TMP);
                _NETWORK->append_event(NewEvent);
                NewEvent = nullptr;

                // check if repos is possible without overtaking
                int overtake=0;

                overtake = this->check_for_takeover_4(_NETWORK,&add_distance_km);

                if( overtake == 0 )
                { // case U start
                    this->POS = 1.0;
                    // set the waiting time ...
                    //std::cout << _NETWORK->PT[this->ON_ARC.dest_node].INFRASTRUCTURE << std::endl;
                    if( _NETWORK->PT[this->ON_ARC.dest_node].INFRASTRUCTURE < 1)
                    {
                        if( this->WAIT_FOR_BLOCK < 1 )
                            this->remaing_waiting_time = BUSSIM_MEAN_WAITING_TIME - BUSSIM_DEVIATION_WAITING_TIME + (double)(rand() % 1001)/1000.0*2.0*BUSSIM_DEVIATION_WAITING_TIME;
                        else
                            this->remaing_waiting_time = BUSSIM_TIME_STEP;
                        // ... and let the vehicle wait
                        this->WAITING = 1;
                    }
                    else
                    {
                        this->remaing_waiting_time = BUSSIM_TIME_STEP;
                        this->WAITING = 1;
                    }
                    // save the travel distance increase of this vehicle
                    this->km_travelled_on_line += add_distance_km;
                    this->km_travelled_on_arc += add_distance_km;
                } // case U end
                else
                { // case V start
                    // vehicle cannot move forward

                    class BUSSIM_BUNCHPOINT *A;
                    A = new class BUSSIM_BUNCHPOINT;
                    A->screen_x = new_pos_vec_x + this->ON_ARC.orig_screen_x;
                    A->screen_y = new_pos_vec_y + this->ON_ARC.orig_screen_y;

                    _NETWORK->append_bunch_point(A);
                    delete A;

                } // case V end
            } // case T end
        } // CASE R - END
        this->last_update_time += BUSSIM_TIME_STEP;

    } // CASE A END
}
}

void BUSSIM_VEHICLE::update_new(class BUSSIM_NETWORK *_NETWORK)
{
    int RES;

    struct BUSSIM_TRANSFER_SITUATION *SIT;
    SIT = new struct BUSSIM_TRANSFER_SITUATION;
    SIT->PAX_ARRIVING = 0;
    SIT->PAX_DEPARTING = 0;
    SIT->CALCULATED_TRANSFER_TIME = 0.0;

    if( (_NETWORK->CURRENT_TIME > 0) )
    {
    if( this->ACTIVE > 0 )
    { // CASE A START
        double add_distance_screen=0.0;;
        // proceeds only for an active vehicle

        if( this->WAITING > 0 )
        { // CASE B - START
            this->remaing_waiting_time -= (BUSSIM_TIME_STEP);

            if( this->remaing_waiting_time <= 0)
            { // CASE C - START

                // vehicle is ready to go and to enter the next arc in the schedule
                this->remaing_waiting_time = 0.0;
                this->WAITING = 0;

                // try to go to the next segment from the schedule
                if( this->SCHEDULE->current_segment->next != nullptr )
                { // case D - START

                    // end of line has not yet reached: goto next arc

                    // first check, if the block logic allows the transfer to the next arc
                    if( _NETWORK->ARC[this->ON_ARC.ID].get_allowance((this->SCHEDULE->current_segment->next->ref2arc)) > 0 )
                    { // case E - START

                        // the next block allows this vehicle to proceed

                        // check if this vehicle is the next in the queue
                        RES = _NETWORK->ARC[this->ON_ARC.ID].remove_vehicle_from_queue(this->ID,_NETWORK);

                        if( RES >= 0 )
                        { // case F - START

                            // this vehicle can be moved forward

                            // set this vehicle to be unblocked
                            this->IS_BLOCKED = 0;

                            // remove the vehicle from the blocks of the current arc
                            _NETWORK->ARC[this->ON_ARC.ID].check_out_vehicle();

                            // shift vehicle to the next arc
                            this->ON_ARC = *(this->SCHEDULE->current_segment->next->ref2arc);

                            // goto the next segment in the schedule of this vehicle
                            this->SCHEDULE->current_segment
                                = this->SCHEDULE->current_segment->next;

                            // update the current line served by this vehicle
                            this->on_line = this->SCHEDULE->current_segment->on_line;

                            // check in the vehicle in all blocks assoc with the next arc
                            _NETWORK->ARC[this->ON_ARC.ID].check_in_vehicle();

                            // reset the wait for block flag of this vehicle
                            this->WAIT_FOR_BLOCK = 0;

                            this->km_travelled_on_arc = 0.0;
                            this->start_time_on_arc = _NETWORK->CURRENT_TIME;

                            // sort this vehicle in the queue assoc with this arc
                            _NETWORK->ARC[this->ON_ARC.ID].append_arrived_vehicle(this->ID);
                            this->POS = 0.0;

                            // vehicle is at the beginning of the next arc
                            char TMP[256];
                            sprintf(TMP,"vehicle\t%d\t line\t%d\t departure from stop\t%d",this->ID,this->on_line,this->ON_ARC.orig_node);
                            class BUSSIM_EVENT *NewEvent;
                            NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_DEPARTURE,TMP);

                            _NETWORK->append_event(NewEvent);
                            NewEvent = nullptr;

                            this->proceed_pax_on_board(_NETWORK);

                        } // case F - END
                        else
                        { // case G - START

                            char TMP[256];
                            sprintf(TMP,"vehicle\t%d\t on line\t %d\t is b-blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NETWORK->ARC[this->ON_ARC.ID].first_waiting_vehicle->veh_id,_NETWORK->VEHICLE[_NETWORK->ARC[this->ON_ARC.ID].first_waiting_vehicle->veh_id].on_line,this->ON_ARC.ID);
                            class BUSSIM_EVENT *NewEvent;
                            NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                            _NETWORK->append_event(NewEvent);
                            NewEvent = nullptr;
                            if( this->IS_BLOCKED < 1 )
                                _NETWORK->DETECTED_BUNCHES++;

                            if( this->IS_BLOCKED < 1 )
                                this->Thwarted++;
                            this->IS_BLOCKED = 1;

                        } // case G - END
                    } // case E - END
                    else
                    { // case H - START
                        // the next arc is blocked and this vehicle cannot proceed
                        char TMPA[256];
                        sprintf(TMPA,"vehicle\t%d\t on line\t %d\t is waiting for block to enter arc\t%d",
                                        this->ID,
                                        this->on_line,
                                        this->SCHEDULE->current_segment->next->ID);
                        class BUSSIM_EVENT *NewEvent;
                        NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_WAIT_FOR_BLOCK,TMPA);
                        _NETWORK->append_event(NewEvent);
                        NewEvent = nullptr;
                        if( this->WAIT_FOR_BLOCK < 1 )
                            _NETWORK->BLOCK_WAIT++;
                        this->WAIT_FOR_BLOCK = 1;

                    } // case H - END


                } // case D - ENDE
                else
                { // case I - START
                    // we have reached the end of the schedule
                    // deactivate this vehicle


                    char TMP[256];
                    sprintf(TMP,"*vehicle\t%d\t no new task\t at end of line %d\t deactivated at stop %d",this->ID,this->on_line,this->ON_ARC.dest_node);
                    class BUSSIM_EVENT *NewEvent;
                    NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_VEHICLE_DEACTIVATION,TMP);
                    _NETWORK->append_event(NewEvent);
                    NewEvent = nullptr;

                    // remove vehicle from queue
                    RES = _NETWORK->ARC[this->ON_ARC.ID].remove_vehicle_from_queue(this->ID,_NETWORK);

                    // disable vehicle
                    this->ACTIVE = 0;
                    this->on_line = -1;

                } // case I - END



            } // CASE C - START
            else
            { // case z - start
                // vehicle is waiting - nothing to do now
            } // case z - end
        } // CASE B - END

        if( this->ACTIVE > 0 )
        {


        if( this->WAITING < 1)
        { // CASE R - START
            // vehicle is moving: update it's position

            // calculate the additional driving distance during _DeltaTime in kilometers
            double add_distance_km;
            //double add_distance_pos;

            add_distance_km = (this->velocity) / 60.0 * (double)(BUSSIM_TIME_STEP);

            // scale it into the screened coordinate system
            add_distance_screen = (add_distance_km - 0) / (_NETWORK->xmax - _NETWORK->xmin) *2.0;

            // determine the new pos-vector
            double new_pos_vec_x;
            double new_pos_vec_y;

            new_pos_vec_x = this->POS*(this->ON_ARC.dest_screen_x - this->ON_ARC.orig_screen_x) + (this->ON_ARC.dest_screen_x - this->ON_ARC.orig_screen_x) / (double)(this->ON_ARC.length_screen)*add_distance_screen;
            new_pos_vec_y = this->POS*(this->ON_ARC.dest_screen_y - this->ON_ARC.orig_screen_y) + (this->ON_ARC.dest_screen_y - this->ON_ARC.orig_screen_y) / (double)(this->ON_ARC.length_screen)*add_distance_screen;

            // determine length of new pos-vector
            double new_pos_vec_length;
            new_pos_vec_length = sqrt(pow(new_pos_vec_x,2)+pow(new_pos_vec_y,2));


            // now check if vehicle has reached destination of ON_ARC
            if( (new_pos_vec_length / this->ON_ARC.length_screen) < 1.0 )
            { // case S start
                // The vehicle is still in the middle of the current arc

                // check if repos is possible without overtaking
                int overtake=0;

                // aktivieren?
                overtake = this->check_for_takeover_4(_NETWORK,&add_distance_km);

                if( overtake == 0 )
                { // case W start
                    // repositioning possible
                    this->POS = (new_pos_vec_length / this->ON_ARC.length_screen);

                    // save the travel distance increase of this vehicle
                    this->km_travelled_on_line += add_distance_km;

                    // update the travel distance counter
                    this->km_travelled_on_arc += add_distance_km;
                } // case W end
                else
                { // case X start
                    // vehicle cannot move forward

                    class BUSSIM_BUNCHPOINT *A;
                    A = new class BUSSIM_BUNCHPOINT;
                    A->screen_x = new_pos_vec_x + this->ON_ARC.orig_screen_x;
                    A->screen_y = new_pos_vec_y + this->ON_ARC.orig_screen_y;

                    _NETWORK->append_bunch_point(A);
                    delete A;

                } // case X end
            } // case S end
            else
            { // case T start
                // has reached end on arc

                // update the last stop information
                // ggf. weg?
                this->next_arc = _NETWORK->ARC[this->ON_ARC.ID];
                this->last_visited_node = this->ON_ARC.dest_node;
                this->last_node_arrival_time = _NETWORK->CURRENT_TIME + (double)(this->ID)/10000.0;;

                // generate the arrival event
                char TMP[256];
                sprintf(TMP,"vehicle\t%d\t line\t%d\t stop\t%d",this->ID,this->on_line,this->ON_ARC.dest_node);
                class BUSSIM_EVENT *NewEvent;
                NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_ARRIVAL,TMP);
                _NETWORK->append_event(NewEvent);
                NewEvent = nullptr;

                // check if repos is possible without overtaking
                int overtake=0;

                overtake = this->check_for_takeover_4(_NETWORK,&add_distance_km);

                if( overtake == 0 )
                { // case U start
                    this->POS = 1.0;
                    // set the waiting time ...
                    if( this->SCHEDULE->current_segment->next != nullptr )
                    {
                        // case UU start
                        //this->POS = 1.00;
                        // there is a next arc in the schedule
                        if( _NETWORK->PT[this->ON_ARC.dest_node].INFRASTRUCTURE < 1)
                        {
                            // case UUA start
                            // the vehicle has reached a stop with passengers
                            if( this->WAIT_FOR_BLOCK < 1 )
                            {
                                // case UUAA start
                                // we can go to the stop because the vehicle is not blocked now
                                //this->remaing_waiting_time = BUSSIM_MEAN_WAITING_TIME - BUSSIM_DEVIATION_WAITING_TIME + (double)(rand() % 1001)/1000.0*2.0*BUSSIM_DEVIATION_WAITING_TIME;

                                // let PAX deboard who have reached their deboard node
                                //std::cout << "start deliver " << std::endl;
                                this->deliver_pax(&_NETWORK->PT[this->ON_ARC.dest_node], _NETWORK);
                                //std::cout << "end deliver " << std::endl;

                                // check if the current segment is the last one of the current duty
                                if( this->SCHEDULE->current_segment->ref2duty->ID != this->SCHEDULE->current_segment->next->ref2duty->ID)
                                {
                                    // we have reached the end point of a duty: reset the paxlists of this vehicle
                                    this->remove_deboard_pax_lists(_NETWORK);
                                    this->init_deboard_pax_lists(_NETWORK,this->SCHEDULE->current_segment->next);
                                }

                                // let PAX enter

                                //std::cout << "start pickup " << std::endl;
                                this->pickup_waiting_pax(&_NETWORK->PT[this->ON_ARC.dest_node], _NETWORK);
                                //std::cout << "end pickup " << std::endl;

                                // calculate remaining waiting time
                                // step (i) - calculate the schedule-based waiting time (without pax activity)
                                this->remaing_waiting_time = std::max(0.0,this->SCHEDULE->current_segment->next->time_in - _NETWORK->CURRENT_TIME) + (double)(rand() % 1001)/1000.0*1.0*BUSSIM_DEVIATION_WAITING_TIME;

                                // step (ii) - calculate the pax-oriented changing time
                                this->determine_transfer_time(SIT,_NETWORK);
                                this->remaing_waiting_time = std::max(this->remaing_waiting_time, SIT->CALCULATED_TRANSFER_TIME);
                                    // std::max(0.0,this->SCHEDULE->current_segment->next->time_in - _NETWORK->CURRENT_TIME) + (double)(rand() % 1001)/1000.0*1.0*BUSSIM_DEVIATION_WAITING_TIME;

                                //this->proceed_pax_on_board(_NETWORK);
                                // case UUAA end
                            }
                            else
                            {
                                // case UUAB start
                                // the vehicle is blocked currently: set the minimal possible waiting time
                                this->remaing_waiting_time = BUSSIM_TIME_STEP;
                                // case UUAB end
                            }

                            // ... and let the vehicle wait
                            this->WAITING = 1;
                            // case UUA end
                        }
                        else
                        {
                            // case UUB start
                            // the vehicle has reached an infrastructure node (i.e. gauge):
                            // set the minimal possible waiting time

                            // check if the current segment is the last one of the current duty
                            if( this->SCHEDULE->current_segment->ref2duty->ID != this->SCHEDULE->current_segment->next->ref2duty->ID)
                            {
                                // we have reached the end point of a duty: reset the paxlists of this vehicle
                                this->remove_deboard_pax_lists(_NETWORK);
                                this->init_deboard_pax_lists(_NETWORK,this->SCHEDULE->current_segment->next);
                            }

                            this->remaing_waiting_time = BUSSIM_TIME_STEP;
                            this->WAITING = 1;
                            //this->proceed_pax_on_board(_NETWORK);
                            // case UUB end
                        }
                        // this->proceed_pax_on_board(_NETWORK);
                        //end case UU
                    }
                    else
                    {
                        // case UU3 start
                        // renew this block
                        // no next arc in the schedule
                        /*
                        std::cout << "Data Error: A line must not start / end with an infrastructure node" << std::endl;
                        std::cout << "vehicle id=" << this->ID << std::endl;
                        std::cout << "on arc id=" << this->ON_ARC.ID << std::endl;
                        std::cout << "node dest= " << this->ON_ARC.dest_node << std::endl;
                        std::cout << "INFRA=" << _NETWORK->PT[this->ON_ARC.dest_node].INFRASTRUCTURE << std::endl;
                        exit(0);
                        */
                        if( _NETWORK->PT[this->ON_ARC.dest_node].INFRASTRUCTURE < 1)
                        {
                            // the vehicle has reached a stop with passengers
                            if( this->WAIT_FOR_BLOCK < 1 )
                                this->remaing_waiting_time = BUSSIM_TIME_STEP; //BUSSIM_MEAN_WAITING_TIME - BUSSIM_DEVIATION_WAITING_TIME + (double)(rand() % 1001)/1000.0*2.0*BUSSIM_DEVIATION_WAITING_TIME;
                            else
                                this->remaing_waiting_time = BUSSIM_TIME_STEP;
                            // ... and let the vehicle wait
                            this->WAITING = 1;
                        }
                        else
                        {
                            // the vehicle has reached an infrastructure node (i.e. gauge)
                            this->remaing_waiting_time = BUSSIM_TIME_STEP;
                            this->WAITING = 1;
                        }
                        //this->proceed_pax_on_board(_NETWORK);
                        // case UU3 end
                    }

                    // save the travel distance increase of this vehicle
                    this->km_travelled_on_line += add_distance_km;
                    this->km_travelled_on_arc += add_distance_km;

                } // case U end
                else
                { // case V start
                    // vehicle cannot move forward

                    class BUSSIM_BUNCHPOINT *A;
                    A = new class BUSSIM_BUNCHPOINT;
                    A->screen_x = new_pos_vec_x + this->ON_ARC.orig_screen_x;
                    A->screen_y = new_pos_vec_y + this->ON_ARC.orig_screen_y;

                    _NETWORK->append_bunch_point(A);
                    delete A;

                } // case V end
            } // case T end
        } // CASE R - END
        this->last_update_time += BUSSIM_TIME_STEP;
        }
    } // CASE A END
}
}

void BUSSIM_VEHICLE::print_on_screen(class BUSSIM_NETWORK *_NET)
{
    if( this->vehicle_category == BUSSIM_VEHCAT_TRAM )
        this->print_on_screen_tram(_NET);
    if( this->vehicle_category == BUSSIM_VEHCAT_BUS )
        this->print_on_screen_bus(_NET);

}

void BUSSIM_VEHICLE::print_on_screen_tram(class BUSSIM_NETWORK *_NET)
{
    // first determine the print color: if this is assigned to a line take the line color
    // otherwise use a light gray
    double RED, GREEN, BLUE;
    if( this->on_line >= 0)
    {
        RED = _NET->LINE[this->on_line].RED;
        GREEN = _NET->LINE[this->on_line].GREEN;
        BLUE = _NET->LINE[this->on_line].BLUE;
    }
    else
    {
        RED = 0.41;     //DimGrey
        GREEN = 0.41;     //DimGrey
        BLUE = 0.41;     //DimGrey
    }

    // second decide if this vehicle must be displayed
    int MUST_BE_DISPLAYED = 0;
    if( this->on_line >= 0)
    {
        // vehicle is on a line
        if( _NET->LINE[this->on_line].displayed > 0 )
            MUST_BE_DISPLAYED = 1;
        else
            MUST_BE_DISPLAYED = 0;
    }
    else
    {
        // vehicle is not on a line
        MUST_BE_DISPLAYED = 1; // vehicle must be displayed
    }

    // decide if this vehicle is a regular one
    int IS_REGULAR_LINE;
    if( this->on_line >= 0)
    {
        // vehicle is on a line
        if( _NET->LINE[this->on_line].type == BUSSIM_REGULARLINE )
            IS_REGULAR_LINE = 1;
        else
            IS_REGULAR_LINE = 0;
    }
    else
    {
        // vehicle is not on a line
        IS_REGULAR_LINE = 1; // vehicle must be displayed
    }

    // execute the displaying
    if( MUST_BE_DISPLAYED > 0)
        if( this->ACTIVE >= 0 )
        {
            // determine the current position of this on the simulation window
            float start_x;
            float start_y;

            start_x = _NET->PT[this->ON_ARC.orig_node].screen_x + this->POS * (_NET->PT[this->ON_ARC.dest_node].screen_x - _NET->PT[this->ON_ARC.orig_node].screen_x);
            start_y = _NET->PT[this->ON_ARC.orig_node].screen_y + this->POS * (_NET->PT[this->ON_ARC.dest_node].screen_y - _NET->PT[this->ON_ARC.orig_node].screen_y);

            // set the color
            glColor4f(RED,GREEN,BLUE, 0.0f);

            // find out the current size w.r.t. the current zoom
            double r = BUSSIM_VEHICLE_SIZE;
            r = r / (double)(pow(1.05,_NET->Current_Zoom_Level));

            // distinguish if the vehicle is currently waiting for a block
            if( this->WAIT_FOR_BLOCK < 1)
            {
                // this is not waiting for a block

                // specify the direction vector
                double x_diff, diff_orth_x;
                double y_diff, diff_orth_y;

                x_diff = _NET->PT[this->ON_ARC.dest_node].screen_x - _NET->PT[this->ON_ARC.orig_node].screen_x;
                y_diff = _NET->PT[this->ON_ARC.dest_node].screen_y - _NET->PT[this->ON_ARC.orig_node].screen_y;

                // .. and its length
                double length_diff;
                length_diff = sqrt(pow(x_diff,2)+pow(y_diff,2));

                // find the orthogonale vector
                diff_orth_x = y_diff;
                diff_orth_y = -x_diff;

                // .. its length
                double length_orth;
                length_orth = sqrt(pow(diff_orth_x,2)+pow(diff_orth_y,2));

                // define vectors
                double x_dir, y_dir, x_orth, y_orth;

                x_dir = r / length_diff * x_diff / 2.0;
                y_dir = r / length_diff * y_diff / 2.0;

                x_orth = r / length_orth * diff_orth_x / 2.0;
                y_orth = r / length_orth * diff_orth_y / 2.0;

                // draw the vehicle symbol
                glBegin(GL_POLYGON);
                    glVertex2f(start_x - 3*x_dir + 1.25*x_orth, start_y - 3*y_dir + 1.25*y_orth);
                    glVertex2f(start_x + 1*x_dir + 1.25*x_orth, start_y + 1*y_dir + 1.25*y_orth);
                    glVertex2f(start_x + 1*x_dir - 1.25*x_orth, start_y + 1*y_dir - 1.25*y_orth);
                    glVertex2f(start_x - 3*x_dir - 1.25*x_orth, start_y - 3*y_dir - 1.25*y_orth);
                    glVertex2f(start_x - 3*x_dir + 1.25*x_orth, start_y - 3*y_dir + 1.25*y_orth);
                glEnd();

                // print the arc for direction indication
                glLineWidth(3.1);
                glColor4f(0.0f,0.0f,0.0f,0.0f);
                glBegin(GL_LINES);
                    x_diff = _NET->PT[_NET->ARC[this->ON_ARC.ID].dest_node].screen_x - _NET->PT[_NET->ARC[this->ON_ARC.ID].orig_node].screen_x;
                    y_diff = _NET->PT[_NET->ARC[this->ON_ARC.ID].dest_node].screen_y - _NET->PT[_NET->ARC[this->ON_ARC.ID].orig_node].screen_y;

                    if( sqrt(pow(x_diff,2)+ pow(y_diff,2)) > 0 )
                    {
                        x_diff = r * (x_diff) / sqrt(pow(x_diff,2)+ pow(y_diff,2));
                        y_diff = r * (y_diff) / sqrt(pow(x_diff,2)+ pow(y_diff,2));
                    }
                    else
                    {
                        x_diff = 0;
                        y_diff = 0;
                    }

                    glVertex2f(start_x, start_y);
                    //glVertex2f(start_x + x_diff, start_y + y_diff);
                    glVertex2f(start_x + 2*x_dir, start_y + 2*y_dir);
                    //glColor4f(1.0f, 0.0f, 0.0f, 0.0f);
                glEnd();
            }
            else
            {
                // print the waiting vehicle as square symbol
                glBegin(GL_POLYGON);
                    glVertex2f(start_x - 0.8*r ,start_y + 0.8*r);
                    glVertex2f(start_x + 0.8*r ,start_y + 0.8*r);
                    glVertex2f(start_x + 0.8*r ,start_y - 0.8*r);
                    glVertex2f(start_x - 0.8*r ,start_y - 0.8*r);
                    glVertex2f(start_x - 0.8*r ,start_y + 0.8*r);
                glEnd();
            }

        glColor4f(RED, GREEN , BLUE, 0.0f);
        //glRasterPos2d(start_x+2*_NET->LINE[this->on_line].x_offset, start_y+1*_NET->LINE[this->on_line].y_offset);

        if( DISPLAY_VEHICLE_LABELS > 0 )
        {
            //double r;
            //r = 1.0 / (double)(pow(1.05,_NET->Current_Zoom_Level));
            char S[256];
            if( this->WAIT_FOR_BLOCK < 1)
            {
                glRasterPos2d(start_x+r*0.02, start_y-r*0.01);
                if(this->WAITING < 1)
                    if( IS_REGULAR_LINE > 0 )
                        sprintf(S,"V%d L%d (%.2f km, %d PL)",this->ID, this->on_line, this->km_travelled_on_line, this->PAX_LIST_NUM);
                    else
                        sprintf(S,"V%d L%d (S) (%.2f km, %d PL)",this->ID, this->on_line, this->km_travelled_on_line, this->PAX_LIST_NUM);
                else
                    sprintf(S,"V%d L%d (w %.1f min, %d PL)",this->ID, this->on_line, this->remaing_waiting_time, this->PAX_LIST_NUM);
                unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
                glutBitmapString(GLUT_BITMAP_HELVETICA_10, ubuf);
            }
            else
            {
                glRasterPos2d(start_x+r*0.02, start_y-r*0.01);
                sprintf(S,"V%d L%d(B on %d, %d PL)",this->ID, this->on_line, this->ON_ARC.ID, this->PAX_LIST_NUM);
                unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
                glutBitmapString(GLUT_BITMAP_HELVETICA_10, ubuf);

            }
        }
    }
}

void BUSSIM_VEHICLE::print_on_screen_bus(class BUSSIM_NETWORK *_NET)
{
    // first determine the print color: if this is assigned to a line take the line color
    // otherwise use a light gray
    double RED, GREEN, BLUE;
    if( this->on_line >= 0)
    {
        RED = _NET->LINE[this->on_line].RED;
        GREEN = _NET->LINE[this->on_line].GREEN;
        BLUE = _NET->LINE[this->on_line].BLUE;
    }
    else
    {
        RED = 0.41;     //DimGrey
        GREEN = 0.41;     //DimGrey
        BLUE = 0.41;     //DimGrey
    }

    // second decide if this vehicle must be displayed
    int MUST_BE_DISPLAYED = 0;
    if( this->on_line >= 0)
    {
        // vehicle is on a line
        if( _NET->LINE[this->on_line].displayed > 0 )
            MUST_BE_DISPLAYED = 1;
        else
            MUST_BE_DISPLAYED = 0;
    }
    else
    {
        // vehicle is not on a line
        MUST_BE_DISPLAYED = 1; // vehicle must be displayed
    }

    // decide if this vehicle is a regular one
    int IS_REGULAR_LINE;
    if( this->on_line >= 0)
    {
        // vehicle is on a line
        if( _NET->LINE[this->on_line].type == BUSSIM_REGULARLINE )
            IS_REGULAR_LINE = 1;
        else
            IS_REGULAR_LINE = 0;
    }
    else
    {
        // vehicle is not on a line
        IS_REGULAR_LINE = 1; // vehicle must be displayed
    }

    // execute the displaying
    if( MUST_BE_DISPLAYED > 0)
        if( this->ACTIVE >= 0 )
        {
            // determine the current position of this on the simulation window
            float start_x;
            float start_y;

            start_x = _NET->PT[this->ON_ARC.orig_node].screen_x + this->POS * (_NET->PT[this->ON_ARC.dest_node].screen_x - _NET->PT[this->ON_ARC.orig_node].screen_x);
            start_y = _NET->PT[this->ON_ARC.orig_node].screen_y + this->POS * (_NET->PT[this->ON_ARC.dest_node].screen_y - _NET->PT[this->ON_ARC.orig_node].screen_y);

            // set the color
            glColor4f(RED,GREEN,BLUE, 0.0f);

            // find out the current size w.r.t. the current zoom
            double r = BUSSIM_VEHICLE_SIZE;
            r = r / (double)(pow(1.05,_NET->Current_Zoom_Level));

            // distinguish if the vehicle is currently waiting for a block
            if( this->WAIT_FOR_BLOCK < 1)
            {
                // this is not waiting for a block

                // specify the direction vector
                double x_diff, diff_orth_x;
                double y_diff, diff_orth_y;

                x_diff = _NET->PT[this->ON_ARC.dest_node].screen_x - _NET->PT[this->ON_ARC.orig_node].screen_x;
                y_diff = _NET->PT[this->ON_ARC.dest_node].screen_y - _NET->PT[this->ON_ARC.orig_node].screen_y;

                // .. and its length
                double length_diff;
                length_diff = sqrt(pow(x_diff,2)+pow(y_diff,2));

                // find the orthogonale vector
                diff_orth_x = y_diff;
                diff_orth_y = -x_diff;

                // .. its length
                double length_orth;
                length_orth = sqrt(pow(diff_orth_x,2)+pow(diff_orth_y,2));

                // define vectors
                double x_dir, y_dir, x_orth, y_orth;

                x_dir = r / length_diff * x_diff / 2.0;
                y_dir = r / length_diff * y_diff / 2.0;

                x_orth = r / length_orth * diff_orth_x / 2.0;
                y_orth = r / length_orth * diff_orth_y / 2.0;

                // draw the vehicle symbol
                glLineWidth(1.0);
                glBegin(GL_LINE_LOOP);
                    glVertex2f(start_x - 3*x_dir + 1.25*x_orth, start_y - 3*y_dir + 1.25*y_orth);
                    glVertex2f(start_x + 1*x_dir + 1.25*x_orth, start_y + 1*y_dir + 1.25*y_orth);
                    glVertex2f(start_x + 1*x_dir - 1.25*x_orth, start_y + 1*y_dir - 1.25*y_orth);
                    glVertex2f(start_x - 3*x_dir - 1.25*x_orth, start_y - 3*y_dir - 1.25*y_orth);
                    glVertex2f(start_x - 3*x_dir + 1.25*x_orth, start_y - 3*y_dir + 1.25*y_orth);
                glEnd();

                // print the arc for direction indication
                glLineWidth(3.1);
                glColor4f(0.0f,0.0f,0.0f,0.0f);
                glBegin(GL_LINES);
                    x_diff = _NET->PT[_NET->ARC[this->ON_ARC.ID].dest_node].screen_x - _NET->PT[_NET->ARC[this->ON_ARC.ID].orig_node].screen_x;
                    y_diff = _NET->PT[_NET->ARC[this->ON_ARC.ID].dest_node].screen_y - _NET->PT[_NET->ARC[this->ON_ARC.ID].orig_node].screen_y;

                    if( sqrt(pow(x_diff,2)+ pow(y_diff,2)) > 0 )
                    {
                        x_diff = r * (x_diff) / sqrt(pow(x_diff,2)+ pow(y_diff,2));
                        y_diff = r * (y_diff) / sqrt(pow(x_diff,2)+ pow(y_diff,2));
                    }
                    else
                    {
                        x_diff = 0;
                        y_diff = 0;
                    }

                    glVertex2f(start_x, start_y);
                    //glVertex2f(start_x + x_diff, start_y + y_diff);
                    glVertex2f(start_x + 2*x_dir, start_y + 2*y_dir);
                    //glColor4f(1.0f, 0.0f, 0.0f, 0.0f);
                glEnd();
            }
            else
            {
                // print the waiting vehicle as square symbol
                glBegin(GL_LINES);
                    glVertex2f(start_x - 0.8*r ,start_y + 0.8*r);
                    glVertex2f(start_x + 0.8*r ,start_y + 0.8*r);
                    glVertex2f(start_x + 0.8*r ,start_y - 0.8*r);
                    glVertex2f(start_x - 0.8*r ,start_y - 0.8*r);
                    glVertex2f(start_x - 0.8*r ,start_y + 0.8*r);
                glEnd();
            }

        glColor4f(RED, GREEN , BLUE, 0.0f);
        //glRasterPos2d(start_x+2*_NET->LINE[this->on_line].x_offset, start_y+1*_NET->LINE[this->on_line].y_offset);

        if( DISPLAY_VEHICLE_LABELS > 0 )
        {
            //double r;
            //r = 1.0 / (double)(pow(1.05,_NET->Current_Zoom_Level));
            char S[256];
            if( this->WAIT_FOR_BLOCK < 1)
            {
                glRasterPos2d(start_x+r*0.02, start_y-r*0.01);
                if(this->WAITING < 1)
                    if( IS_REGULAR_LINE > 0 )
                        sprintf(S,"V%d L%d (%.2f km, %d PL)",this->ID, this->on_line, this->km_travelled_on_line, this->PAX_LIST_NUM);
                    else
                        sprintf(S,"V%d L%d (S) (%.2f km, %d PL)",this->ID, this->on_line, this->km_travelled_on_line, this->PAX_LIST_NUM);
                else
                    sprintf(S,"V%d L%d (w %.1f min, %d PL)",this->ID, this->on_line, this->remaing_waiting_time, this->PAX_LIST_NUM);
                unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
                glutBitmapString(GLUT_BITMAP_HELVETICA_10, ubuf);
            }
            else
            {
                glRasterPos2d(start_x+r*0.02, start_y-r*0.01);
                sprintf(S,"V%d L%d(B on %d, %d PL)",this->ID, this->on_line, this->ON_ARC.ID, this->PAX_LIST_NUM);
                unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
                glutBitmapString(GLUT_BITMAP_HELVETICA_10, ubuf);

            }
        }
    }
}

void BUSSIM_VEHICLE::assign_to_line(class BUSSIM_NETWORK *_NET, class BUSSIM_LINE *_LINE)
{
    this->ACTIVE = 0;
    this->WAITING = 0;
    //this->ID = _LINE->ID;
    this->remaing_waiting_time = 0;
    this->last_update_time = _NET->CURRENT_TIME;
    this->on_line = _LINE->ID;
    this->ON_ARC = _LINE->USED_ARC[0];
    // sort this vehicle in the queue assoc with this arc
    _NET->ARC[this->ON_ARC.ID].append_arrived_vehicle(this->ID);
    this->POS = 0.0;
    this->km_travelled_on_line = 0;

}

int BUSSIM_VEHICLE::check_for_takeover(class BUSSIM_NETWORK *_NET, double *ADD_DIST)
{
    int ret_val = 0;

    for( int v = 0 ; v < _NET->VEHICLES ; v++ )
    {
        if( _NET->VEHICLE[v].ID != this->ID)
        {
            if( _NET->VEHICLE[v].ACTIVE > 0)
                //if( this->WAITING < 1)
            {
                if( _NET->VEHICLE[v].on_line == this->on_line )
                {
                    // both vehicle are currently service the same line
                    if( _NET->VEHICLE[v].start_time_on_line < this->start_time_on_line )
                    {
                        // vehicle v has started earlier than this ...
                        if( _NET->VEHICLE[v].km_travelled_on_line <= (this->km_travelled_on_line + ADD_DIST[0])  ) // HERE
                        {
                            // ... v has not yet travelled so far
                            ret_val = 1;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT required takeover of vehicle " <<_NET->VEHICLE[v].ID << " by " << this->ID << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT travel distance of vehicle " << _NET->VEHICLE[v].ID << " on this arc so far: " << _NET->VEHICLE[v].km_travelled_on_arc << " (enter time: " << _NET->VEHICLE[v].start_time_on_arc <<")" << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT travel distance of vehicle " << this->ID << " on this arc so far: " << this->km_travelled_on_arc << " (enter time: " << this->start_time_on_arc <<")" << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT required add km " << ADD_DIST[0] << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT on line ("<< this->ON_ARC.orig_node << ";" << this->ON_ARC.dest_node << ") at POS " << this->POS << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT waiting ("<< _NET->VEHICLE[v].WAITING << ";" << this->WAITING << ") at POS " << this->POS << std::endl;

                            char TMP[256];
                            sprintf(TMP,"vehicle\t%d\t on line\t %d\t is blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NET->VEHICLE[v].ID,_NET->VEHICLE[v].on_line,this->ON_ARC.ID);
                            class BUSSIM_EVENT *NewEvent;
                            NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                            _NET->append_event(NewEvent);
                            NewEvent = nullptr;

                            _NET->DETECTED_BUNCHES++;
                            this->Thwarted++;
                            v = _NET->VEHICLES+1;
                        }
                    }
                }
            }
        }
    }

    return ret_val;
}

int BUSSIM_VEHICLE::check_for_takeover_crossline(class BUSSIM_NETWORK *_NET, double *ADD_DIST)
{
    int ret_val = 0;

    for( int v = 0 ; v < _NET->VEHICLES ; v++ )
    {
        if( _NET->VEHICLE[v].ID != this->ID)
        {
            if( _NET->VEHICLE[v].ACTIVE > 0)
            if( this->on_line != _NET->VEHICLE[v].on_line )
            {
                if( _NET->VEHICLE[v].ON_ARC.ID == this->ON_ARC.ID )
                {
                    // both vehicle are currently travelling on the same arc
                    if( _NET->VEHICLE[v].start_time_on_arc < this->start_time_on_arc )
                    {
                        // vehicle v has started earlier than this ...
                        if( _NET->VEHICLE[v].km_travelled_on_arc + 0.00001 < (this->km_travelled_on_arc + ADD_DIST[0])  )
                        {
                            // .. v but has not yet travelled as much km as this!
                            ret_val = 1;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": required takeover of vehicle " <<_NET->VEHICLE[v].ID << " by " << this->ID << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << _NET->VEHICLE[v].ID << " on this arc so far: " << _NET->VEHICLE[v].km_travelled_on_arc << " (enter time: " << _NET->VEHICLE[v].start_time_on_arc <<")" << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << this->ID << " on this arc so far: " << this->km_travelled_on_arc << " (enter time: " << this->start_time_on_arc <<")" << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": required add km " << ADD_DIST[0] << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": on shared arc ("<< this->ON_ARC.orig_node << ";" << this->ON_ARC.dest_node << ") at POS " << this->POS << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": waiting ("<< _NET->VEHICLE[v].WAITING << ";" << this->WAITING << ") at POS " << this->POS << std::endl;

                            char TMP[256];
                            sprintf(TMP,"vehicle\t%d\t on line\t %d\t is blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NET->VEHICLE[v].ID,_NET->VEHICLE[v].on_line,this->ON_ARC.ID);
                            class BUSSIM_EVENT *NewEvent;
                            NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                            _NET->append_event(NewEvent);
                            NewEvent = nullptr;

                            _NET->DETECTED_BUNCHES++;
                            this->Thwarted++;
                            //exit(0);
                            v = _NET->VEHICLES+1;
                        }
                    }
                    else
                    {
                        // hier weiter mit last_node und at_last_node

                    }
                }
            }
       }
    }

    return ret_val;
}

void BUSSIM_VEHICLE::activate(class BUSSIM_NETWORK *_NET)
{
    if( this->ACTIVE < 1)
    {
        if( this->rotation->first_duty != nullptr )
        {
            // there is a rotation available for this vehicle

            this->assign_to_line(_NET,&_NET->LINE[this->rotation->first_duty->TRIP_ID]);

            // save the information of the last stop
            this->next_arc = _NET->ARC[_NET->LINE[this->rotation->first_duty->TRIP_ID].USED_ARC[0].ID];
            this->last_visited_node  = _NET->ARC[_NET->LINE[this->rotation->first_duty->TRIP_ID].USED_ARC[0].ID].orig_node;
            this->last_node_arrival_time = _NET->CURRENT_TIME + (double)(this->ID)/10000.0;

            this->ACTIVE = 1;
            this->start_time_on_line = _NET->CURRENT_TIME;
            this->start_time_on_arc = _NET->CURRENT_TIME;

            // set pointer to the current duty of this vehicles
            this->rotation->current_duty = this->rotation->first_duty;
            char TMP[256];
            sprintf(TMP,"vehicle\t%d\t activated on line %d\t at stop %d",this->ID,_NET->LINE[this->rotation->first_duty->TRIP_ID].ID,_NET->LINE[this->rotation->first_duty->TRIP_ID].USED_ARC[0].ID);
            class BUSSIM_EVENT *NewEvent;
            NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_ACTIVATION,TMP);
            _NET->append_event(NewEvent);
            NewEvent = nullptr;
        }
    }
}

void BUSSIM_VEHICLE::activate_new(class BUSSIM_NETWORK *_NET)
{
    if( this->ACTIVE < 1)
    {
        this->SCHEDULE->current_segment = this->SCHEDULE->first_segment;
        if( this->SCHEDULE->first_segment != nullptr )
        {
            // create the initial deboard_pax_lists
            this->init_deboard_pax_lists(_NET,this->SCHEDULE->current_segment);

            /*std::cout << "vehicle " << this->ID << " activated" << std::endl;
            this->SCHEDULE->print_on_shell(_NET);
            this->print_paxlists_on_console(_NET);
            exit(0);
            */
            // there is a rotation available for this vehicle

            this->ACTIVE = 0;
            this->WAITING = 0;
            this->remaing_waiting_time = 0;
            this->last_update_time = _NET->CURRENT_TIME;
            this->on_line = this->SCHEDULE->current_segment->on_line;
            this->ON_ARC = *(this->SCHEDULE->current_segment->ref2arc);

            // sort this vehicle in the queue assoc with this arc
            //_NET->ARC[this->ON_ARC.ID].append_arrived_vehicle(this->ID);
            //this->POS = 0.0;
            this->km_travelled_on_line = 0;

            // save the information of the last stop
            // mglw. wieder reinnehmen, wg. take_over_check_4?
            // this->next_arc = _NET->ARC[this->ON_ARC.ID];
            // this->last_visited_node  = _NET->ARC[this->ON_ARC.ID].orig_node;
            // this->last_node_arrival_time = _NET->CURRENT_TIME + (double)(this->ID)/10000.0;

            this->ACTIVE = 1;
            this->start_time_on_line = _NET->CURRENT_TIME;
            this->start_time_on_arc = _NET->CURRENT_TIME;

            // set pointer to the current duty of this vehicles
            //this->rotation->current_duty = this->rotation->first_duty;
            char TMP[256];
            sprintf(TMP,"vehicle\t%d\t activated on line %d\t at stop %d",this->ID,this->on_line,this->ON_ARC.ID);
            class BUSSIM_EVENT *NewEvent;
            NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_ACTIVATION,TMP);
            _NET->append_event(NewEvent);
            NewEvent = nullptr;
        }
    }
}

int BUSSIM_VEHICLE::check_for_takeover_new(class BUSSIM_NETWORK *_NET, double *ADD_DIST)
{
    int ret_val;
    ret_val = 0;

    //std::cout << "------------ check vehicle " << this->ID << " ---------" << std::endl;

    for( int v = 0 ; v < _NET->VEHICLES ; v++ )
    {
        //std::cout << " compare with vehicle " << v << std::endl;
        if( _NET->VEHICLE[v].ID != this->ID )
        {
            if( _NET->VEHICLE[v].ACTIVE > 0)
            {
                //std::cout << "vehicle " << v << " is active " << std::endl;
                //if( this->on_line != _NET->VEHICLE[v].on_line )
                if( 1 < 2 )
                {
                    if( _NET->VEHICLE[v].ON_ARC.ID == this->ON_ARC.ID )
                    {
                        //std::cout << "AB" << std::endl;
                        //_NET->PRED[_NET->VEHICLE[v].ID][this->ID] = 0;

                        // both vehicle are currently travelling on the same arc
                        if( _NET->VEHICLE[v].start_time_on_arc < this->start_time_on_arc )
                        {
                            if( (_NET->VEHICLE[v].POS >= 1.0) && (this->POS >= 1.0) && (_NET->VEHICLE[v].WAITING > -1) )
                            {

                                ret_val = 1;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": required takeover of vehicle " <<_NET->VEHICLE[v].ID << " by " << this->ID << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << _NET->VEHICLE[v].ID << " on this arc so far: " << _NET->VEHICLE[v].km_travelled_on_arc << " (enter time: " << _NET->VEHICLE[v].start_time_on_arc <<")" << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << this->ID << " on this arc so far: " << this->km_travelled_on_arc << " (enter time: " << this->start_time_on_arc <<")" << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": required add km " << ADD_DIST[0] << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": on shared arc ("<< this->ON_ARC.orig_node << ";" << this->ON_ARC.dest_node << ") at POS " << this->POS << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": waiting ("<< _NET->VEHICLE[v].WAITING << ";" << this->WAITING << ") at POS " << this->POS << std::endl;

                                char TMP[256];
                                sprintf(TMP,"vehicle\t%d\t on line\t %d\t is blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NET->VEHICLE[v].ID,_NET->VEHICLE[v].on_line,this->ON_ARC.ID);
                                class BUSSIM_EVENT *NewEvent;
                                NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                                _NET->append_event(NewEvent);
                                NewEvent = nullptr;

                                _NET->DETECTED_BUNCHES++;
                                this->Thwarted++;
                            }
                            else
                            {


                            _NET->PRED[_NET->VEHICLE[v].ID][this->ID] = 1;
                            _NET->PRED[this->ID][_NET->VEHICLE[v].ID] = 0;
                            // vehicle v has started earlier than this ...
                            if( _NET->VEHICLE[v].km_travelled_on_arc + 0.00001 < (this->km_travelled_on_arc + ADD_DIST[0])  )
                            {
                                // .. v but has not yet travelled as much km as this!
                                ret_val = 1;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": required takeover of vehicle " <<_NET->VEHICLE[v].ID << " by " << this->ID << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << _NET->VEHICLE[v].ID << " on this arc so far: " << _NET->VEHICLE[v].km_travelled_on_arc << " (enter time: " << _NET->VEHICLE[v].start_time_on_arc <<")" << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << this->ID << " on this arc so far: " << this->km_travelled_on_arc << " (enter time: " << this->start_time_on_arc <<")" << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": required add km " << ADD_DIST[0] << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": on shared arc ("<< this->ON_ARC.orig_node << ";" << this->ON_ARC.dest_node << ") at POS " << this->POS << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": waiting ("<< _NET->VEHICLE[v].WAITING << ";" << this->WAITING << ") at POS " << this->POS << std::endl;

                                char TMP[256];
                                sprintf(TMP,"vehicle\t%d\t on line\t %d\t is blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NET->VEHICLE[v].ID,_NET->VEHICLE[v].on_line,this->ON_ARC.ID);
                                class BUSSIM_EVENT *NewEvent;
                                NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                                _NET->append_event(NewEvent);
                                NewEvent = nullptr;

                                _NET->DETECTED_BUNCHES++;
                                this->Thwarted++;
                                //exit(0);
                                v = _NET->VEHICLES+1;
                            }
                            }
                        }
                        else
                        {


                        }
                    }
                    else
                    {
                        //std::cout << "CD" << std::endl;
                        // the two vehicle are not on the same arc: check if they will be one the same arc in a moment
                        if( _NET->VEHICLE[v].next_arc.ID == this->next_arc.ID )
                        {
                            //std::cout << "EF" << std::endl;
                            if( _NET->VEHICLE[v].next_arc.ID >= 0 )
                            {
                                //std::cout << "GH" << std::endl;
                                if( this->next_arc.ID >= 0)
                                {
                                    //std::cout << "IJ" << std::endl;
                                    // both vehicles have been busy before
                                    if( _NET->VEHICLE[v].last_visited_node == this->last_visited_node )
                                    {
                                        //std::cout << "KL" << std::endl;
                                        // if( _NET->VEHICLE[v].last_node_arrival_time  <= this->last_node_arrival_time )
                                        {
                                            // falls gleich: immer nach vehicle ID
                                            // vehicle v arrived before this vehicle at last common stop
                                            if( _NET->VEHICLE[v].POS >= 1.0 )
                                            {
                                                //std::cout << "MN" << std::endl;
                                                // vehicle v has not yet left node last_visisted_node
                                                if( this->POS < 0.5 )
                                                {
                                                    //std::cout << "OP" << std::endl;

                                                    if( _NET->PRED[_NET->VEHICLE[v].ID][this->ID] > 0 )
                                                    {
                                                        //std::cout << "QR" << std::endl;
                                                        // this vehicle is going to leave the node last_visisted node :-(
                                                        ret_val = 1;

                                                        char TMP[256];
                                                        sprintf(TMP,"**vehicle\t%d\t on line\t %d\t is blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NET->VEHICLE[v].ID,_NET->VEHICLE[v].on_line,this->ON_ARC.ID);
                                                        class BUSSIM_EVENT *NewEvent;
                                                        NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                                                        _NET->append_event(NewEvent);
                                                        NewEvent = nullptr;

                                                        _NET->DETECTED_BUNCHES++;
                                                        this->Thwarted++;
                                                        //exit(0);
                                                        v = _NET->VEHICLES+1;
                                                    }
                                                }
                                            }
                                        }

                                    }
                                }

                            }
                        }
                    }
                }
            }
        }
    }

    return ret_val;
}

int BUSSIM_VEHICLE::check_for_takeover_4(class BUSSIM_NETWORK *_NET, double *ADD_DIST)
{
    int ret_val;
    ret_val = 0;

    class BUSSIM_ARC_WAITING_VEHICLE *Ptr;
    int POS_THIS = 0;
    int POS_V = 0;

    //std::cout << "vehicle tried to be moved " << this->ID << std::endl;
    for( int v = 0 ; v < _NET->VEHICLES ; v++ )
    {
        //std::cout << " compare with vehicle " << v << std::endl;
        if( _NET->VEHICLE[v].ID != this->ID )
        {
            // another vehicle to compare with?
            if( _NET->VEHICLE[v].ACTIVE > 0)
            {
                // is the other vehicle active (on the field of play)
                if( _NET->VEHICLE[v].ON_ARC.ID == this->ON_ARC.ID )
                {
                    // both vehicles are on the same arc
                    // now check their positions in the queue
                    //std::cout << "comparing " << _NET->VEHICLE[v].ID << " and " << this->ID << std::endl;
                    //std::cout << "HERE is the queue on ARC" << std::endl;
                    //Ptr = _NET->ARC[this->ON_ARC.ID].first_waiting_vehicle;
                    //while( Ptr != nullptr )
                    //{
                    //    std::cout << "V=" << Ptr->veh_id << " " << _NET->VEHICLE[Ptr->veh_id].ON_ARC.ID << std::endl;
                    //    Ptr = Ptr->next;
                    //}
                    POS_THIS = 0;
                    Ptr = _NET->ARC[this->ON_ARC.ID].first_waiting_vehicle;
                    while( Ptr != nullptr )
                    {
                        if( Ptr->veh_id != this->ID )
                        {
                            POS_THIS++;
                            Ptr = Ptr->next;
                        }
                        else
                            Ptr = nullptr;
                    }
                    POS_V = 0;
                    Ptr = _NET->ARC[this->ON_ARC.ID].first_waiting_vehicle;
                    while( Ptr != nullptr )
                    {
                        if( Ptr->veh_id != _NET->VEHICLE[v].ID )
                        {
                            POS_V++;
                            Ptr = Ptr->next;
                        }
                        else
                            Ptr = nullptr;
                    }

                    // now compare their positions in the queue on the same used arc
                    if( POS_THIS < POS_V )
                    {
                        // no danger for taking over since this is already before v
                    }
                    else
                    {
                        // here it can happen that vehicle v blocks this
                        if( _NET->VEHICLE[v].km_travelled_on_arc + 0.0001 < (this->km_travelled_on_arc + ADD_DIST[0])  )
                            {
                                // .. v but has not yet travelled as much km as this!
                                ret_val = 1;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": required takeover of vehicle " <<_NET->VEHICLE[v].ID << " by " << this->ID << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << _NET->VEHICLE[v].ID << " on this arc so far: " << _NET->VEHICLE[v].km_travelled_on_arc << " (enter time: " << _NET->VEHICLE[v].start_time_on_arc <<")" << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << this->ID << " on this arc so far: " << this->km_travelled_on_arc << " (enter time: " << this->start_time_on_arc <<")" << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": required add km " << ADD_DIST[0] << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": on shared arc ("<< this->ON_ARC.orig_node << ";" << this->ON_ARC.dest_node << ") at POS " << this->POS << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": waiting ("<< _NET->VEHICLE[v].WAITING << ";" << this->WAITING << ") at POS " << this->POS << std::endl;

                                char TMP[256];
                                sprintf(TMP,"vehicle\t%d\t on line\t %d\t is blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NET->VEHICLE[v].ID,_NET->VEHICLE[v].on_line,this->ON_ARC.ID);
                                class BUSSIM_EVENT *NewEvent;
                                NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                                _NET->append_event(NewEvent);
                                NewEvent = nullptr;

                                _NET->DETECTED_BUNCHES++;
                                this->Thwarted++;
                                //exit(0);
                                v = _NET->VEHICLES+1;
                            }
                    }
                }
            }
        }
    }
    return ret_val;
}

void BUSSIM_VEHICLE::rotation2schedule(class BUSSIM_NETWORK *_NET)
{
    // this procedure is used in the implicit schedule generation
    // it expands the rotation of this vehicle into a schedule

    // needed for the iterative scheduling of segments
    double last_out, last_in;
    int PlatformNumber;
    if( this->rotation->first_duty != nullptr )
    {
        std::cout << "vehicle "<< this->ID << " start schedule creation: " << std::endl;
        class BUSSIM_DUTY *Ptr2Duty;
        Ptr2Duty = this->rotation->first_duty;
        class BUSSIM_ARC ARC_IN_TRIP;
        class BUSSIM_SEGMENT *Ptr2Seg;

        // the initial start time
        last_in = this->activation_time;

        while( Ptr2Duty != nullptr )
        {
            // in which duty are we?

            //std::cout << "DUTY-ID=" << Ptr2Duty->ID << std::endl;
            PlatformNumber = 0;
            for( int i=0 ; i < _NET->LINE[Ptr2Duty->TRIP_ID].ARCS ; i++ )
            {
                //std::cout << "line: " << _NET->LINE[Ptr2Duty->TRIP_ID].ID << std::endl;

                /* std::cout << "->i=" << i << std::endl;
                std::cout << "arc-id: " << _NET->LINE[Ptr2Duty->TRIP_ID].USED_ARC[i].ID << std::endl;
                std::cout << "arc-ref: " << &_NET->ARC[_NET->LINE[Ptr2Duty->TRIP_ID].USED_ARC[i].ID] << std::endl;
                std::cout << "line: " << _NET->LINE[Ptr2Duty->TRIP_ID].ID << std::endl;
                std::cout << "destination: " << _NET->PT[_NET->ARC[_NET->LINE[Ptr2Duty->TRIP_ID].USED_ARC[(_NET->LINE[Ptr2Duty->TRIP_ID].ARCS-1)].ID].dest_node].STOPNAME<< std::endl;
                */
                // create the new segment
                // round arrival time if the node is a stop else
                // do not round in case that we have an infrastructure stop
                if( _NET->PT[_NET->LINE[Ptr2Duty->TRIP_ID].USED_ARC[i].dest_node].INFRASTRUCTURE < 1 )
                    last_out = ceil(last_in + 60.0*_NET->ARC[_NET->LINE[Ptr2Duty->TRIP_ID].USED_ARC[i].ID].length / this->velocity);
                else
                    last_out = (last_in + 60.0*_NET->ARC[_NET->LINE[Ptr2Duty->TRIP_ID].USED_ARC[i].ID].length / this->velocity);

                Ptr2Seg = new class BUSSIM_SEGMENT(_NET->SEGMENT_CNTR,
                                                   &_NET->ARC[_NET->LINE[Ptr2Duty->TRIP_ID].USED_ARC[i].ID],
                                                   last_in,
                                                   last_out,
                                                   Ptr2Duty, /*Ptr2Duty,*/
                                                   _NET->LINE[Ptr2Duty->TRIP_ID].ID,
                                                   _NET->PT[_NET->ARC[_NET->LINE[Ptr2Duty->TRIP_ID].USED_ARC[(_NET->LINE[Ptr2Duty->TRIP_ID].ARCS-1)].ID].dest_node].STOPNAME,
                                                   this->SCHEDULE,
                                                   nullptr,
                                                   nullptr,
                                                   _NET,
                                                   1);
                if( Ptr2Seg->DeboardAtEnd > 0 )
                {
                    Ptr2Seg->PosDeboardPaxList = PlatformNumber;
                    PlatformNumber++;
                }

                this->SCHEDULE->append_segment(Ptr2Seg,_NET);

                // add default waiting time if a regular stop
                if( _NET->PT[_NET->LINE[Ptr2Duty->TRIP_ID].USED_ARC[i].dest_node].INFRASTRUCTURE < 1 )
                    if( _NET->PT[_NET->LINE[Ptr2Duty->TRIP_ID].USED_ARC[i].dest_node].type == BUSSIM_NODE_LINE )
                        last_in = ceil(last_out + BUSSIM_MEAN_WAITING_TIME);
                    else
                        last_in = ceil(last_out + BUSSIM_DEPOT_WAITING_TIME);
                else
                    last_in = last_out;

                Ptr2Seg = nullptr;
            }
            Ptr2Duty = Ptr2Duty->next;
        }
        std::cout << "vehicle "<< this->ID << " schedule creation completed. " << std::endl;
    }
    else
    {
        std::cout << "vehicle "<< this->ID << " no unscheduled rotation found => no schedule extracted." << std::endl;
    }
}

void BUSSIM_VEHICLE::set_activation_time(double _activation_time)
{
    this->activation_time = _activation_time;
}

int BUSSIM_VEHICLE::compare_init_pos_with_schedule(class BUSSIM_NETWORK *_NET)
{
    // this procedure checks if the initial position of this vehicle coincides
    // with the first arc in the determined schedule
    //
    // returns 1 if vehicle is already on the first arc in the schedule
    // returns 0 in all other cases

    int RET_VAL = 0;

    if( this->SCHEDULE != nullptr )
    {
        // there is a schedule specified
        if( this->SCHEDULE->first_segment != nullptr )
        {
            // there is also a first segment specified
            if( this->SCHEDULE->first_segment->ref2arc->ID == this->ON_ARC.ID )
            {
                // everything is ok and the two arcs are equal
                RET_VAL = 1;
            }
        }
    }
    return(RET_VAL);
}

int BUSSIM_VEHICLE::compare_park_pos_with_schedule(class BUSSIM_NETWORK *_NET)
{
    // this procedure checks if the park position of this vehicle coincides
    // with the last arc in the determined schedule
    //
    // returns 1 if vehicle is already on the last arc in the schedule
    // returns 0 in all other cases

    int RET_VAL = 0;

    if( this->SCHEDULE != nullptr )
    {
        // there is a schedule specified
        if( this->SCHEDULE->last_segment != nullptr )
        {
            // there is also a first segment specified
            if( this->SCHEDULE->last_segment->ref2arc->ID == this->park_arc.ID )
            {
                // everything is ok and the two arcs are equal
                RET_VAL = 1;
            }
        }
    }
    return(RET_VAL);
}

void BUSSIM_VEHICLE::remove_deboard_pax_lists(class BUSSIM_NETWORK *_NET)
{
    // this procedure removes all paxlists (just to be sure) and remove them
    // and the end of a duty in the vehicles rotation

    // should be called if all passengers have deboarded at last stop of a duty

    if( this->PAX_LIST_NUM > 0 )
    {
        // iterate through the pax lists
        for( int i=0 ; i < this->PAX_LIST_NUM ; i++ )
        {
            // check first if the passenger lists are empty
            if( this->DeBoardPaxList[i].get_pax_num() > 0 )
            {
                std::cout << "Vehicle " << this->ID << ": non-empty paxlist before destroying!" << std::endl;
                std::cout << "currently serving line " << this->on_line << std::endl;
                this->DeBoardPaxList[i].print();
                std::cout << "Current Schedule Segment ID: " << this->SCHEDULE->current_segment->ID << std::endl;
                std::cout << "Current Duty: " << this->SCHEDULE->current_segment->ref2duty->ID << std::endl;
                this->SCHEDULE->print_on_shell(_NET);
                exit(0);
            }
            // dismantle connections to schedule / schedule segments
            this->DeBoardPaxList[i].get_ref2schedule_seg()->ref2DeBoardPaxList = nullptr;
            this->DeBoardPaxList[i].set_ref2schedule_seg(nullptr);

        }

        // delete the paxlists
        delete[] this->DeBoardPaxList;

        // re-set the paxlist counter
        this->PAX_LIST_NUM = 0;
    }

    this->DeBoardPaxList = nullptr;

    // resets the pax list counter
    this->PAX_LIST_NUM = 0;
}

void BUSSIM_VEHICLE::init_deboard_pax_lists(class BUSSIM_NETWORK *_NET, class BUSSIM_SEGMENT *_SEG)
{
    // this procedure installs all paxlists for the regular nodes in the duty found at _SEG

    // should be called at first stop of a duty before the first passengers board

    if( _SEG != nullptr )
    {
        // there is a segment we can use to initialize the pax list on board of this vehicle

        // first, identify the bussim_line object associated to the service on start
        if( _SEG->on_line >= 0 )
        {
            // there is a line: identify and store its ID
            // int Current_Line_Id;
            // Current_Line_Id = _NET->LINE[_SEG->on_line].ID;

            // the number of nodes with pax exchange
            int Current_Stop_Num;
            Current_Stop_Num = _NET->LINE[_SEG->on_line].REGULAR_STOP_NUM;

            // set the attribute of the vehicle
            this->PAX_LIST_NUM = Current_Stop_Num;

            // if necessary install the pax lists
            if( Current_Stop_Num > 0 )
            {
                // create the array of appropriate length
                this->DeBoardPaxList = new class BUSSIM_PAX_LIST[Current_Stop_Num];

                // connect segment in current duty with paxlist and set name
                class BUSSIM_SEGMENT *TmpSeg;
                TmpSeg = _SEG;

                this->PAX_ON_BOARD = 0;
                // scan all segments of the current duty and create and connect
                // paxlist for each non-infrastructure node
                int STOP_NUM = 0;
                while( (TmpSeg != nullptr) && (TmpSeg->ref2duty->ID == _SEG->ref2duty->ID) )
                {
                    //std::cout << "segment id= " << TmpSeg->ID << std::endl;
                    if( TmpSeg->DeboardAtEnd > 0 )
                    {
                        //std::cout << "GAGA" << std::endl;
                        // sets connection from schedule segment to paxlist...
                        TmpSeg->ref2DeBoardPaxList = &this->DeBoardPaxList[STOP_NUM];
                        // ... and from paxlist to schedule segment
                        this->DeBoardPaxList[STOP_NUM].set_ref2schedule_seg(TmpSeg);
                        // specifies the title of the paxlist
                        TmpSeg->ref2DeBoardPaxList->set_listname("pax debording at " + std::to_string(_NET->PT[_NET->ARC[TmpSeg->ref2arc->ID].dest_node].ID)+ "("+std::to_string(STOP_NUM)+")");
                        STOP_NUM++;
                    }
                    TmpSeg = TmpSeg->next;
                }
            }
        }
    }
    //hier weiter
}

void BUSSIM_VEHICLE::print_paxlists_on_console(class BUSSIM_NETWORK *_NET)
{
    std::cout << "this is vehicle " << this->ID << std::endl;
    std::cout << "stops= " << this->PAX_LIST_NUM << std::endl;
    for( int i=0 ; i < this->PAX_LIST_NUM ; i++ )
        this->DeBoardPaxList[i].print();
}

void BUSSIM_VEHICLE::pickup_waiting_pax(class BUSSIM_POINT *_REF2POINT, class BUSSIM_NETWORK *_NET)
{
    int abc;

    // we first have to find out the line operated by the vehicle when leaving
    // the platform at _REF2POINT. For this, we have a look to the next segment
    // of this vehicle's schedule (if it exists at all)

    /*
    std::cout << "CurrentTime=" << _NET->CURRENT_TIME << std::endl;
    std::cout << "Vehicle=" << this->ID << std::endl;
    std::cout << "current duty=" << this->SCHEDULE->current_segment->ref2duty->ID << std::endl;
    this->SCHEDULE->print_on_shell_remaining_duty(_NET);
    std::cout << "arrived at platform " << _REF2POINT->ID << std::endl;
    std::cout << "operating on line " << this->on_line << std::endl;
    std::cout << "Waiting paxes" << std::endl;
    _REF2POINT->print_waiting_lists(0,0,_NET);
    */

    if( (this->on_line < _NET->EXT_LINES) && (this->on_line >= 0) )
    {
        // we have a valid line (not a deployment or a parking line)

        if ( _REF2POINT->WAITINGLIST[this->on_line]->get_pax_num() > 0 )
        {
           // there are passengers to board the vehicle
            class BUSSIM_PAX_LISTITEM *Ptr2Pax;
            while( _REF2POINT->WAITINGLIST[this->on_line]->get_pax_num() >0 )
            {
                // set the reference to the first pax in the waiting list
                Ptr2Pax = _REF2POINT->WAITINGLIST[this->on_line]->get_first_pax_item();

                //std::cout << "pickup of pax " << Ptr2Pax->get_ref2pax()->ID << std::endl;

                // dissolve pax from waiting list
               _REF2POINT->WAITINGLIST[this->on_line]->extract_item(Ptr2Pax);

                //std::cout << "pax extracted from local waiting list " << std::endl;
                // find out deboarding node number
                abc = Ptr2Pax->get_ref2pax()->TravelPath->get_deboard_node_of_ride(Ptr2Pax->get_ref2pax()->Ref2CurrentTPSeg->get_next(),_NET);

                //std::cout << "got deboard node " << abc << std::endl;
                // insert this pax into the adequate debording list

               //this->DeBoardPaxList[_NET->PT[abc].PlatformPosInLine[NEXT_LINE_NUMBER]-1].append_pax(Ptr2Pax->get_ref2pax(),0);
                //std::cout << "to be put in deboard list with ID " << _NET->PT[abc].PlatformPosInLine[this->on_line] << std::endl;

                this->DeBoardPaxList[_NET->PT[abc].PlatformPosInLine[this->on_line]].append_pax(Ptr2Pax->get_ref2pax(),0);
                _NET->NUM_PAX_IN_WAITING--;

                Ptr2Pax->get_ref2pax()->Ref2Vehicle = this;

                // increase the number of hosted passengers
                this->PAX_ON_BOARD++;
                //exit(0);

                //std::cout << "fertig" << std::endl;
            }

        }
    }
}

void BUSSIM_VEHICLE::deliver_pax(class BUSSIM_POINT *_REF2POINT, class BUSSIM_NETWORK *_NET)
{
    // this procedure deboards all passengers who wants to get out here

    // identify the current platform position in the current duty
    int CurrentNodePosInDuty;
    CurrentNodePosInDuty = this->SCHEDULE->current_segment->PosDeboardPaxList;

    // distribute the paxes from DeBoardList[CurrentNodePosInDuty] into the waiting lists at the
    // node at th end of _REF2POINT


    class BUSSIM_PAX_LISTITEM *Ptr2PaxItem;
    class BUSSIM_PAX_LISTITEM *Ptr2PaxInListItem;

    //std::cout << "We are here to deliver at point " << _REF2POINT->ID << std::endl;
    //std::cout << "Vehicle=" << this->ID << std::endl;
    //std::cout << "on line=" << this->on_line << std::endl;

    //this->SCHEDULE->print_on_shell(_NET);
    //std::cout << "we have " << this->DeBoardPaxList[CurrentNodePosInDuty].get_pax_num() << " pax to get off" << std::endl;
    //this->print_paxlists_on_console(_NET);
    while( this->DeBoardPaxList[CurrentNodePosInDuty].get_pax_num() >0 )
    {
        //std::cout << "A" << std::endl;
        // set the reference to the first pax in the waiting list
        Ptr2PaxItem = this->DeBoardPaxList[CurrentNodePosInDuty].get_first_pax_item();
        //std::cout << "B" << std::endl;
       // dissolve the first passenger from this current deboarding list
        this->DeBoardPaxList[CurrentNodePosInDuty].extract_item(Ptr2PaxItem);
        //std::cout << "C" << std::endl;
        // what to do with the current passenger? evaluate the next segment
        Ptr2PaxItem->get_ref2pax()->Ref2Vehicle = nullptr;
        //std::cout << "D " << Ptr2PaxItem->get_ref2pax()->Ref2CurrentTPSeg << std::endl;
        //std::cout << "we have pax " << Ptr2PaxItem->get_ref2pax()->ID << std::endl;
        //Ptr2PaxItem->get_ref2pax()->TravelPath->print_on_console(_NET);
        if( Ptr2PaxItem->get_ref2pax()->Ref2CurrentTPSeg->get_next() != nullptr )
        {
            //std::cout << "D1" << std::endl;
            // waiting or walking?
            if( Ptr2PaxItem->get_ref2pax()->Ref2CurrentTPSeg->get_next()->get_activity() == BUSSIM_PAX_WAITING )
            {
                // sort pax into waiting list at node in segment
                //std::cout << "D2" << std::endl;
                _REF2POINT->WAITINGLIST[Ptr2PaxItem->get_ref2pax()->Ref2CurrentTPSeg->get_next()->get_next()->get_line()]->append_pax(Ptr2PaxItem->get_ref2pax(),0);
                //std::cout << "D3" << std::endl;
                _NET->NUM_PAX_IN_WAITING++;
                //std::cout << "D4" << std::endl;
                Ptr2PaxItem->get_ref2pax()->proceed_on_travelpath(_NET);
                //std::cout << "D5" << std::endl;
            }
            else
            {
                //std::cout << "WorW-B" << std::endl;
                // sort pax into waiting list at node in segment for upcoming walking transfer
                //_REF2POINT->WAITINGLIST[_NET->EXT_LINES]->append_pax(Ptr2PaxItem->get_ref2pax(),0);
                // std::cout << "D6" << std::endl;
                // add this pax to the list of walking pax
                _NET->PAX_WALKING->append_pax(Ptr2PaxItem->get_ref2pax(),1);
                // std::cout << "D7" << std::endl;
                // let the pax go to the next travelpath segment
                Ptr2PaxItem->get_ref2pax()->proceed_on_travelpath(_NET);
                // std::cout << "D8" << std::endl;
            }
        }
        else
        {
            //std::cout << "COMPLETED" << std::endl;
            // travel path completed: we have to delete this pax from PAX_IN and put it in PAX_ARRIVED
            // std::cout << "D9" << std::endl;
            // identify paxlistitem of pax in PAX_IN-list and move this paxlistitem into PAX_ARRIVED
            Ptr2PaxInListItem = Ptr2PaxItem->get_ref2pax()->Ref2PaxInList;
            // std::cout << "D10" << std::endl;
            // disconnect pax to IN-List item
            Ptr2PaxItem->get_ref2pax()->Ref2PaxInList = nullptr;
            // std::cout << "D11" << std::endl;
            // remove item from PAX_IN-list
            _NET->PAX_IN->extract_item(Ptr2PaxInListItem);
            // std::cout << "D12" << std::endl;
            // destroy paxlistitem from simulation
            delete Ptr2PaxInListItem;
            // std::cout << "D13" << std::endl;
            Ptr2PaxInListItem = nullptr;
            // std::cout << "D14" << std::endl;
            // append the current pax to this list of processed paxes
            _NET->PAX_ARRIVED->append_pax(Ptr2PaxItem->get_ref2pax(),0);
            // std::cout << "D15" << std::endl;
        }
        this->PAX_ON_BOARD--;
    }


}

void BUSSIM_VEHICLE::proceed_pax_on_board(class BUSSIM_NETWORK *_NET)
{
    // this procedure proceeds all on board pax to their next travel path segment
    for( int i=0 ; i < this->PAX_LIST_NUM ; i++ )
    {
        class BUSSIM_PAX_LISTITEM *Ptr;
        Ptr = this->DeBoardPaxList[i].get_first_pax_item();
        while( Ptr != nullptr )
        {
            //if( Ptr->get_ref2pax()->ID == 4 )
            //    std::cout << "proceed pax 4 in vehicle " << this->ID << " on arc " << this->ON_ARC.ID << std::endl;
            Ptr->get_ref2pax()->proceed_on_travelpath(_NET);
            Ptr = Ptr->get_next();
        }
    }
}

void BUSSIM_VEHICLE::print_paxlist_on_console(class BUSSIM_NETWORK *_NET)
{
    // this procedure prints the current pos of the vehicle and the paxes on the console

    std::cout << "Aktuelle Zeit:" << _NET->CURRENT_TIME << std::endl;
    std::cout << "on arc " << this->ON_ARC.ID
                << "("
                <<_NET->ARC[this->ON_ARC.ID].orig_node
                <<"-"
                <<_NET->ARC[this->ON_ARC.ID].dest_node
                <<")"
                << std::endl;

    class BUSSIM_PAX_LISTITEM *Ptr;

    for( int i=0 ; i < this->PAX_LIST_NUM ; i++ )
    {

        if( this->DeBoardPaxList[i].get_pax_num() > 0 )
        {
            this->DeBoardPaxList[i].print();
            Ptr = this->DeBoardPaxList[i].get_first_pax_item();
            std::cout << "Pax " << Ptr->get_ref2pax()->ID;
            if( Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2arc() != nullptr )
                    std::cout << " on arc " << Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2arc()->ID;
                if( Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2point() != nullptr )
                    std::cout << " at point " << Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2point()->ID;
                std::cout << std::endl;
            Ptr = Ptr->get_next();
            while( Ptr != nullptr )
            {
                Ptr = this->DeBoardPaxList[i].get_next_pax_item(Ptr);
                std::cout << "Pax " << Ptr->get_ref2pax()->ID << ":";
                if( Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2arc() != nullptr )
                    std::cout << " on arc " << Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2arc()->ID;
                if( Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2point() != nullptr )
                    std::cout << " at point " << Ptr->get_ref2pax()->Ref2CurrentTPSeg->get_ref2point()->ID;
                std::cout << std::endl;
                Ptr = Ptr->get_next();
            }
        }
    }
}

void BUSSIM_VEHICLE::determine_transfer_time(struct BUSSIM_TRANSFER_SITUATION *_SIT, class BUSSIM_NETWORK *_NET)
{
    _SIT->CALCULATED_TRANSFER_TIME =
        std::max(0.0,this->SCHEDULE->current_segment->next->time_in - _NET->CURRENT_TIME) + (double)(rand() % 1001)/1000.0*1.0*BUSSIM_DEVIATION_WAITING_TIME;
}
